@@ -1,5 +1,5 @@
 /*************************************************************************************
- *  Copyright (C) 2007 by Aleix Pol <aleixpol@gmail.com>                             *
+ *  Copyright (C) 2007 by Aleix Pol <aleixpol@kde.org>                               *
  *                                                                                   *
  *  This program is free software; you can redistribute it and/or                    *
  *  modify it under the terms of the GNU General Public License                      *
@@ -19,55 +19,58 @@
 #include "function.h"
 
 #include "variables.h"
-#include "analitza.h"
+#include "analyzer.h"
 #include "expression.h"
 #include "functionimpl.h"
+#include "functionfactory.h"
 
 #include <KLocale>
 #include <cmath>
 
+using Analitza::ExpressionType;
+
 function::function()
 	: m_function(0), m_show(true), m_color(Qt::black)
 {}
 
-// #include "container.h"
-
-function::function(const QString &name, const Expression& newFunc, Variables* v, const QColor& color)
-	: m_function(0), m_show(true), m_color(color), m_name(name)
+function::function(const QString &name, const Analitza::Expression& newFunc, Analitza::Variables* v,
+				   const QColor& color, double uplimit, double downlimit)
+	: m_function(0), m_expression(newFunc), m_show(true), m_color(color), m_name(name)
 {
 	if(newFunc.isCorrect()) {
-//		objectWalker(newFunc.tree());
-		QString firstBVar=newFunc.isLambda() ? newFunc.bvarList()[0] : "x";
-		if(firstBVar=="x")
-			m_function=new FunctionY(newFunc, v);
-		else if(firstBVar=="y")
-			m_function=new FunctionX(newFunc, v);
-		else if(firstBVar=="q")
-			m_function=new FunctionPolar(newFunc, v);
-		else
-			m_err << "Function type not recognized";
+		Analitza::Analyzer a(v);
+		a.setExpression(newFunc);
+		
+		m_expression=a.dependenciesToLambda();
+        a.setExpression(m_expression);
+		
+		QStringList bvars=m_expression.bvarList();
+		
+		//TODO: turn into assertion
+		if(!FunctionFactory::self()->contains(bvars))                                        
+			m_err << i18n("Function type not recognized");
+		else {
+			bool correct=true;
+			ExpressionType expected=FunctionFactory::self()->type(bvars);
+			ExpressionType actual=a.type();
+			
+			correct &= actual.canReduceTo(expected);
+			
+			if(correct) {
+				m_function=FunctionFactory::self()->item(bvars, m_expression, v);
+				if(downlimit!=uplimit)
+					m_function->setLimits(downlimit, uplimit);
+			} else
+				m_err << i18n("Function type not correct for functions depending on %1", bvars.join(i18n(", ")));
+		}
 	} else {
-		m_err << "The expression is not correct";
+		m_err << i18n("The expression is not correct");
 	}
 }
 
-/*FunctionImpl* copy(FunctionImpl* fi)
-{
-	FunctionX *fx;
-	FunctionY *fy;
-	FunctionPolar *fp;
-	if(fx=dynamic_cast<FunctionX*>(fi))
-		return new FunctionX(*fx);
-	else if(fy=dynamic_cast<FunctionY*>(fi))
-		return new FunctionY(*fy);
-	else if(fp=dynamic_cast<FunctionPolar*>(fi))
-		return new FunctionPolar(*fp);
-	else 
-		return 0;
-}*/
-
 function::function(const function& f)
-	: m_function(0), m_show(f.m_show), m_color(f.m_color), m_name(f.m_name), m_err(f.m_err)
+	: m_function(0), m_expression(f.expression()), m_show(f.m_show), m_color(f.m_color)
+	, m_name(f.m_name), m_err(f.m_err)
 {
 	if(f.m_function)
 		m_function=f.m_function->copy();
@@ -75,15 +78,13 @@ function::function(const function& f)
 
 function::~function()
 {
-	if(m_function)
-		delete m_function;
+	delete m_function;
 }
 
 function function::operator=(const function& f)
 {
 	if(&f!=this) {
-		if(m_function)
-			delete m_function;
+		delete m_function;
 		
 		if(f.m_function) {
 			m_function=f.m_function->copy();
@@ -91,6 +92,7 @@ function function::operator=(const function& f)
 			Q_ASSERT(m_function);
 		} else
 			m_function=0;
+		m_expression=f.m_expression;
 		m_show=f.m_show;
 		m_color=f.m_color;
 		m_name=f.m_name;
@@ -99,10 +101,24 @@ function function::operator=(const function& f)
 	return *this;
 }
 
-void function::update_points(const QRect& viewport, unsigned int max_res)
+void function::update_points(const QRect& viewport)
 {
 	Q_ASSERT(m_function);
-	m_function->updatePoints(viewport, max_res);
+	Q_ASSERT(resolution()>2);
+	
+	m_function->updatePoints(viewport);
+	Q_ASSERT(m_function->points.size()>=2);
+}
+
+void function::setResolution(unsigned int resolution)
+{
+	Q_ASSERT(m_function);
+	m_function->setResolution(resolution);
+}
+
+uint function::resolution() const
+{
+	return m_function->resolution();
 }
 
 function::Axe function::axeType() const
@@ -121,19 +137,10 @@ QLineF function::derivative(const QPointF & p) const
 	return m_function->derivative(p);
 }
 
-QStringList function::bvars() const
-{
-	return m_function->bvarList();
-}
-
-Analitza * function::analitza() const
-{
-	Q_ASSERT(m_function);
-	return &m_function->func;
-}
-
 const QVector<QPointF>& function::points() const
 {
+	Q_ASSERT(m_function);
+	Q_ASSERT(m_function->points.size()>1);
 	return m_function->points;
 }
 
@@ -151,33 +158,20 @@ bool function::isCorrect() const
 QStringList function::errors() const
 {
 	QStringList err(m_err);
-	if(m_function)
+	if(m_function) {
 		err += m_function->m_err;
+		err += m_function->func.errors();
+	}
 	return err;
 }
 
-QString function::toString() const
+const Analitza::Expression& function::expression() const
 {
-	Q_ASSERT(m_function);
-	return m_function->toString();
+	return m_expression;
 }
 
-QStringList function::supportedBoundedVars()
-{
-	QStringList ret;
-	ret.append("x");
-	ret.append("y");
-	ret.append("q");
-	return ret;
-}
-
-const Expression& function::expression() const
-{
-	return *analitza()->expression();
-}
-
-
 QList<int> function::jumps() const
 {
 	return m_function->m_jumps;
 }
+
