@@ -22,13 +22,14 @@
 #include "spectrum.h"
 #include "kalziumutils.h"
 
+#include <config-kalzium.h>
+
 #include <element.h>
 
 #include <kdebug.h>
 #include <klocale.h>
 #include <math.h>
 
-#include <QCursor>
 #include <QKeyEvent>
 #include <QSizePolicy>
 #include <QPainter>
@@ -58,6 +59,7 @@ SpectrumWidget::SpectrumWidget( QWidget *parent )
 	setMinimumSize( 400, 230 );
 	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
 	setAttribute( Qt::WA_OpaquePaintEvent, true );
+	setContextMenuPolicy( Qt::PreventContextMenu );
 }
 
 void SpectrumWidget::paintEvent( QPaintEvent * /*e*/ )
@@ -281,8 +283,8 @@ void SpectrumWidget::slotZoomOut()
     if ( startValue < 0.0 )
         startValue = 0.0;
 
-    if ( endValue > 800.0 )
-        endValue = 800.0;
+    if ( endValue > 10000.0 )
+        endValue = 40000.0;
 
     setBorders( startValue, endValue );
 }
@@ -322,10 +324,12 @@ void SpectrumWidget::mouseMoveEvent( QMouseEvent *e )
 
 void SpectrumWidget::mousePressEvent(  QMouseEvent *e )
 {
-    if (  e->button() == Qt::LeftButton )
+    if (  e->button() == Qt::LeftButton ) {
         m_LMBPointPress = e->pos();
-    if (  e->button() == Qt::RightButton )
-        slotZoomOut();
+    }
+    if (  e->button() == Qt::RightButton ) {
+        restart();
+    }
     
     findPeakFromMouseposition( Wavelength( ( double )e->pos().x()/width() ) );
 }
