@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright 2001-2009 Anne-Marie Mahfouf <annma@kde.org>                *
+ *   Copyright 2001-2010 Anne-Marie Mahfouf <annma@kde.org>                *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -21,7 +21,7 @@
 #ifndef _KHANGMAN_H_
 #define _KHANGMAN_H_
 
-#include <kxmlguiwindow.h>
+#include <KXmlGuiWindow>
 
 #include "khangmanview.h"
 #include "ui_generalui.h"
@@ -32,11 +32,10 @@ class KToggleAction;
 
 
 // Id tags for the status bar.
-const int IDS_LANG     = 100;
-const int IDS_LEVEL    = 101;
-const int IDS_ACCENTS  = 102;
-const int IDS_WINS     = 104;
-const int IDS_LOSSES   = 105;
+const int IDS_LEVEL    = 100;
+const int IDS_ACCENTS  = 101;
+const int IDS_WINS     = 103;
+const int IDS_LOSSES   = 104;
 
 
 /**
@@ -70,14 +69,16 @@ public:
     void setMessages();
 
 public slots:
-    ///When the langugae is changed in the Language menu
+    ///When the language is changed in the Language menu
     void slotChangeLanguage(int);
+    ///When the hint is set or unset
+    void slotSetHint(bool);
 
 private slots:
     // Slots for when the user changes level, setting, etc.
     void  slotChangeLevel(int);
     void  slotChangeMode(int);
-    void slotChangeHintAction(bool);
+
     ///In Settings menu, Configure KHangMan... menu item
     void optionsPreferences();
     ///access the KNewStuff class to install new data
@@ -93,6 +94,8 @@ private slots:
     void slotNewGame();
     ///open a local KVTML file
     void slotFileOpen();
+    ///upload a KVTML file
+    void slotUploadFile();
     /**
     Update the text in the Statusbar
     @param text the text that will be written in the statusbar
