@@ -1,5 +1,5 @@
 /*
- * Copyright 2001-2010 Anne-Marie Mahfouf <annma@kde.org>
+ * Copyright 2001-2009 Anne-Marie Mahfouf <annma@kde.org>
 
      This program is free software; you can redistribute it and/or modify  
      it under the terms of the GNU General Public License as published by  
@@ -251,8 +251,9 @@ void KHangManView::paintEvent( QPaintEvent * e )
     paintWord(p, e->rect());
     paintMisses(p, e->rect());
     paintHint(p, e->rect());
-    if(m_loser || m_winner) 
+    if (m_loser || m_winner) {
         paintGameOver(p, e->rect());
+    }
 }
 
 
@@ -332,13 +333,14 @@ void KHangManView::paintMisses(QPainter &p, const QRect& )
 
 void KHangManView::paintHint(QPainter &p, const QRect &rect)
 {
-    if(!Prefs::hint())
+    if (!Prefs::hint()) {
         return;
-
+    }
+    
     QRect myRect = m_theme->hintRect(size());
-    if(!myRect.intersects(rect))
+    if (!myRect.intersects(rect)) {
         return;
-
+    }
 /* Debug */  //p.fillRect(myRect, QBrush(Qt::cyan) );
 
     QColor letterColor = m_theme->letterColor();
@@ -383,7 +385,7 @@ void KHangManView::paintGameOver(QPainter &p, const QRect &rect)
     QRect rectangle=QRect(width()/10, height()*2/10, width()-width()*2/10, height()-height()*4/10);
     p.drawRoundRect(rectangle);
     QFont tFont("Domestic Manners");
-    tFont.setPixelSize( width()/26 );
+    tFont.setPixelSize( (int)   ((width()*42)    /   (26*title.length()))   );
     p.setPen(Qt::black);
     p.setFont(tFont);
     p.drawText(rectangle, Qt::AlignCenter, title);
@@ -391,8 +393,9 @@ void KHangManView::paintGameOver(QPainter &p, const QRect &rect)
     if(m_bgfill<100) {
         m_bgfill+=5;
         QTimer::singleShot(10, this, SLOT(update()));
-	return;
+        return;
     }
+    
     m_playAgainButton->setFocus();
     m_playAgainButton->setDefault(true);
     m_playAgainButton->move(width()/2 - m_playAgainButton->width()/2 , height()/2 + m_playAgainButton->height()/2);
@@ -422,7 +425,7 @@ void KHangManView::resizeEvent(QResizeEvent *)
 
     m_playAgainButton->setGeometry( width()/2 - m_playAgainButton->width()/2 , 
                                     height()/2 + m_playAgainButton->height()/2,
-				    m_playAgainButton->sizeHint().width(), height()/9 );
+    m_playAgainButton->sizeHint().width(), height()/9 );
 }
 
 // ----------------------------------------------------------------
@@ -436,6 +439,7 @@ void KHangManView::slotTry()
         m_letterInput->setFocus();
         return;
     }
+    
     kDebug() << "guess as entered: " << guess;
 
     // Handle the guess.
@@ -501,7 +505,7 @@ void KHangManView::slotTry()
         popup->setPopupStyle( KPassivePopup::Balloon );
         popup->setAutoDelete( true );
         popup->setTimeout( Prefs::missedTimer()*1000 );
-        popup->setView(i18n("This letter has already been guessed.") );
+        popup->setView("KHangMan", i18n("This letter has already been guessed.") );
 
         int  x = 0;
         int  y = 0;
@@ -704,7 +708,8 @@ void KHangManView::slotSetWordsSequence()
             hint = m_doc->lesson()->entries(KEduVocLesson::Recursive).value(j)->translation(1)->text();
         }
 
-        m_randomList.append(qMakePair(m_doc->lesson()->entries(KEduVocLesson::Recursive).value(j)->translation(0)->text(), hint));    }
+        m_randomList.append(qMakePair(m_doc->lesson()->entries(KEduVocLesson::Recursive).value(j)->translation(0)->text(), hint));
+    }
     //shuffle the list
     randomSequence.randomize(m_randomList);
 }
