@@ -46,8 +46,6 @@
 #include "starblockfactory.h"
 #include "skymesh.h"
 
-class SkyComponent;
-class KStarsData;
 class SkyMesh;
 class StarObject;
 class SkyLabeler;
@@ -60,87 +58,42 @@ class MeshIterator;
 
 class StarComponent: public ListComponent
 {
+protected:
 
- protected:
+    StarComponent( SkyComposite* );
 
-    StarComponent( SkyComponent* );
-
- public:
+public:
 
     virtual ~StarComponent();
 
-    /**@short Create an instance of StarComponent
-     */
-    static StarComponent *Create( SkyComponent* );
+    // TODO: Desingletonize StarComponent
+    /**@short Create an instance of StarComponent */
+    static StarComponent *Create( SkyComposite* );
 
-    /**@return the instance of StarComponent if already created, NULL otherwise
-     */    
+    /**@return the instance of StarComponent if already created, NULL otherwise */
     static StarComponent *Instance() { return pinstance; }
 
     //This function is empty; we need that so that the JiT update 
     //is the only one beiong used.
-    void update( KStarsData *data, KSNumbers *num );
+    virtual void update( KSNumbers *num );
 
     bool selected();
 
-    void reindex( KSNumbers *num );
-
     void draw( QPainter& psky );
 
     /* @short draw all the labels in the prioritized LabelLists and then
-     * clear the LabelLists.
-     */
+     * clear the LabelLists. */
     void drawLabels( QPainter& psky );
 
-    void init(KStarsData *data);
-
-    inline KStarsData *data() { return m_Data; }
-
-    /**@return the current setting of the color mode for stars (0=real colors,
-        *1=solid red, 2=solid white or 3=solid black).
-        */
-    int starColorMode( void ) const;
-
-    /**@short Retrieve the color-intensity value for stars.
-        *
-        *When using the "realistic colors" mode for stars, stars are rendered as 
-        *white circles with a colored border.  The "color intensity" setting modulates
-        *the relative thickness of this colored border, so it effectively adjusts
-        *the color-saturation level for star images.
-        *@return the current setting of the color intensity setting for stars.
-        */
-    int starColorIntensity( void ) const;
-
-    /**
-     *@return the magnitude of the faintest star
-     */
-    float faintMagnitude() const;
-
-    /**
-     *@short Return the size of the star as rendered by KStars
-     *@return the Size of the star's representation on the skymap in pixels
-     */
+    /**@short Return the size of the star as rendered by KStars
+     * @return the Size of the star's representation on the skymap in pixels */
     float starRenderingSize( float mag ) const;
 
-    float zoomMagnitudeLimit() const;
+    static float zoomMagnitudeLimit();
 
-    /**
-     *@short Read data for stars which will remain static in the memory
-     *
-     *This method reads data for named stars (stars having names,
-     *which are stored by default in "namedstars.dat") into
-     *memory. These stars are always kept in memory, as against 'deep'
-     *stars which are mostly loaded dynamically (KStars treats all
-     *unnamed stars as 'deep' stars) into memory when required,
-     *depending on region and magnitude limit. Once loading is
-     *successful, this method sets the starsLoaded flag to true
-     */
+    virtual SkyObject* objectNearest(SkyPoint *p, double &maxrad );
 
-    bool loadStaticData();
-
-    SkyObject* objectNearest(SkyPoint *p, double &maxrad );
-
-    SkyObject* findStarByGenetiveName( const QString name );
+    virtual SkyObject* findStarByGenetiveName( const QString name );
 
     /**
      *@short Find stars by name (including genetive name)
@@ -152,8 +105,18 @@ class StarComponent: public ListComponent
      *@return  Pointer to the star with the given name as a SkyObject, NULL if
      *         no match was found
      */
+    virtual SkyObject* findByName( const QString &name );
 
-    SkyObject* findByName( const QString &name );
+    /**
+     * @short Searches the region(s) and appends the SkyObjects found to the list of sky objects
+     *
+     * Look for a SkyObject that is in one of the regions 
+     * If found, then append to the list of sky objects
+     * @p list list of SkyObject to which matching list has to be appended to
+     * @p region defines the regions in which the search for SkyObject should be done within
+     * @return void
+     */
+    virtual void objectsInArea( QList<SkyObject*>& list, const SkyRegion& region );
 
     /**
      *@short Find stars by HD catalog index
@@ -162,40 +125,51 @@ class StarComponent: public ListComponent
      *        If it is a dynamic star, a fake copy will be created that survives till
      *        the next findByHDIndex() call. If no match was found, returns NULL.
      */
-
+    // FIXME: check whether return type should be StarObject*
     SkyObject* findByHDIndex( int HDnum );
 
-    // TODO: Make byteSwap a template method and put it in byteswap.h
+    // TODO: Make byteSwap a template method and put it in byteorder.h
     // It should ideally handle 32-bit, 16-bit fields and starData and
     // deepStarData fields
     static void byteSwap( starData *stardata );
 
 private:
+    /**@short Read data for stars which will remain static in the memory
+     *
+     * This method reads data for named stars (stars having names,
+     * which are stored by default in "namedstars.dat") into
+     * memory. These stars are always kept in memory, as against 'deep'
+     * stars which are mostly loaded dynamically (KStars treats all
+     * unnamed stars as 'deep' stars) into memory when required,
+     * depending on region and magnitude limit. Once loading is
+     * successful, this method sets the starsLoaded flag to true
+     */
+    bool loadStaticData();
+
+    /** @return the magnitude of the faintest star */
+    float faintMagnitude() const;
+
+    void reindex( KSNumbers *num );
+
+    
     SkyMesh*       m_skyMesh;
     StarIndex*     m_starIndex;
 
     KSNumbers      m_reindexNum;
     double         m_reindexInterval;
 
-    int            m_lineNumber[ MAX_LINENUMBER_MAG + 1 ];
     LabelList*     m_labelList[  MAX_LINENUMBER_MAG + 1 ];
-    int            m_lastLineNum;
-    bool           m_validLineNums;
     bool           m_hideLabels;
 
-    KStarsData*    m_Data;
     float          m_zoomMagLimit;
 
     float          m_FaintMagnitude; // Limiting magnitude of the catalog currently loaded
     bool           starsLoaded;
     float          magLim;           // Current limiting magnitude for visible stars
-    unsigned long  visibleStarCount;
-    quint16        MSpT;             // Maximum number of stars in any given trixel
 
     StarObject     m_starObject;
     StarObject     *focusStar;       // This object is always drawn
 
-    KStarsSplash*  m_reloadSplash;
     KStarsSplash*  m_reindexSplash;
 
     StarBlockFactory *m_StarBlockFactory;
