@@ -17,8 +17,6 @@
 
 //This file contains drawing functions SkyMap class.
 
-#include <stdlib.h> // abs
-#include <math.h> //log10()
 #include <iostream>
 
 #include <QPainter>
@@ -26,6 +24,7 @@
 
 #include "skymap.h"
 #include "Options.h"
+#include "fov.h"
 #include "kstars.h"
 #include "kstarsdata.h"
 #include "ksnumbers.h"
@@ -33,15 +32,11 @@
 #include "skyobjects/deepskyobject.h"
 #include "skyobjects/starobject.h"
 #include "skyobjects/ksplanetbase.h"
-#include "skyobjects/ksasteroid.h"
-#include "skyobjects/kscomet.h"
-#include "skyobjects/ksmoon.h"
-#include "skyobjects/jupitermoons.h"
-#include "infoboxes.h"
 #include "simclock.h"
 #include "observinglist.h"
-#include "skycomponents/constellationboundary.h"
+#include "skycomponents/constellationboundarylines.h"
 #include "skycomponents/skylabeler.h"
+#include "skycomponents/skymapcomposite.h"
 
 #include <config-kstars.h>
 
@@ -53,14 +48,16 @@
 #include "indi/indidevice.h"
 #endif
 
-void toXYZ(SkyPoint* p, double *x, double *y, double *z) {
-    double sinRa, sinDec, cosRa, cosDec;
+namespace {
+    void toXYZ(SkyPoint* p, double *x, double *y, double *z) {
+        double sinRa, sinDec, cosRa, cosDec;
 
-    p->ra()->SinCos( sinRa, cosRa );
-    p->dec()->SinCos( sinDec, cosDec );
-    *x = cosDec * cosRa;
-    *y = cosDec * sinRa;
-    *z = sinDec;
+        p->ra().SinCos(  sinRa,  cosRa );
+        p->dec().SinCos( sinDec, cosDec );
+        *x = cosDec * cosRa;
+        *y = cosDec * sinRa;
+        *z = sinDec;
+    }
 }
 
 QPointF SkyMap::clipLine( SkyPoint *p1, SkyPoint *p2 )
@@ -125,44 +122,41 @@ QPointF SkyMap::clipLine( SkyPoint *p1, SkyPoint *p2 )
 }
 
 QPoint SkyMap::clipLineI( SkyPoint *p1, SkyPoint *p2 ) {
-    QPointF qpf = clipLine( p1, p2 );
-    return QPoint( (int) qpf.x(), (int) qpf.y() );
+    return clipLine( p1, p2 ).toPoint();
 }
 
-
 void SkyMap::drawOverlays( QPixmap *pm ) {
-    if ( ks ) { //only if we are drawing in the GUI window
-        QPainter p;
-        p.begin( pm );
-        p.setRenderHint(QPainter::Antialiasing, Options::useAntialias() );
+    if( !KStars::Instance() )
+        return;
 
-        if ( Options::showHighlightedCBound() )
-            drawHighlightConstellation( p );
+    QPainter p;
+    p.begin( pm );
+    p.setRenderHint(QPainter::Antialiasing, Options::useAntialias() );
 
-        showFocusCoords( true );
-        drawBoxes( p );
+    if ( Options::showHighlightedCBound() )
+        drawHighlightConstellation( p );
 
-        //draw FOV symbol
-        ks->data()->fovSymbol.draw( p,  (float)(Options::fOVSizeX() * Options::zoomFactor()/57.3/60.0), (float)(Options::fOVSizeY() * Options::zoomFactor()/57.3/60.0));
-        drawTelescopeSymbols( p );
-        drawObservingList( p );
-        drawZoomBox( p );
+    //draw FOV symbol
+    foreach( FOV* fov, KStarsData::Instance()->visibleFOVs ) {
+        fov->draw(p, Options::zoomFactor());
+    }
+    drawTelescopeSymbols( p );
+    drawObservingList( p );
+    drawZoomBox( p );
 
-        if ( transientObject() ) drawTransientLabel( p );
-        if (isAngleMode()) {
-            updateAngleRuler();
-            drawAngleRuler( p );
-        }
+    if ( transientObject() )
+        drawTransientLabel( p );
+    if (isAngleMode()) {
+        updateAngleRuler();
+        drawAngleRuler( p );
     }
 }
 
 void SkyMap::drawAngleRuler( QPainter &p ) {
     p.setPen( QPen( data->colorScheme()->colorNamed( "AngularRuler" ), 3.0, Qt::DotLine ) );
-
-    QPointF startPoint = toScreen( AngularRuler.point(0) );
-    QPointF endPoint = toScreen( AngularRuler.point(1) );
-
-    p.drawLine( startPoint, endPoint );
+    p.drawLine(
+        toScreen( AngularRuler.point(0) ),
+        toScreen( AngularRuler.point(1) ) );
 }
 
 void SkyMap::drawZoomBox( QPainter &p ) {
@@ -175,11 +169,12 @@ void SkyMap::drawZoomBox( QPainter &p ) {
 
 void SkyMap::drawHighlightConstellation( QPainter &psky ) {
     const QPolygonF* cbound =
-        ConstellationBoundary::Instance()->constellationPoly( focus() );
+        KStarsData::Instance()->skyComposite()->getConstellationBoundary()->constellationPoly( focus() );
+    if ( ! cbound )
+        return;
 
-    if ( ! cbound ) return;
-
-    psky.setPen( QPen( QColor( ks->data()->colorScheme()->colorNamed( "CBoundHighColor" ) ), 3, Qt::SolidLine ) );
+    psky.setPen( QPen( QColor( KStarsData::Instance()->colorScheme()->colorNamed( "CBoundHighColor" ) ),
+                       3, Qt::SolidLine ) );
 
     QPolygonF clippedPoly;
     bool needClip( false );
@@ -191,8 +186,8 @@ void SkyMap::drawHighlightConstellation( QPainter &psky ) {
         for ( int i = 0; i < cbound->size(); i++ ) {
             QPointF node = cbound->at( i );
             SkyPoint sp( node.x(), node.y() );
-            sp.EquatorialToHorizontal( data->LST, data->geo()->lat() );
-            horizPolygon << QPointF( sp.az()->Degrees(), sp.alt()->Degrees() );
+            sp.EquatorialToHorizontal( data->lst(), data->geo()->lat() );
+            horizPolygon << QPointF( sp.az().Degrees(), sp.alt().Degrees() );
         }
 
         QRectF rBound = horizPolygon.boundingRect();
@@ -219,8 +214,8 @@ void SkyMap::drawHighlightConstellation( QPainter &psky ) {
                 SkyPoint sp;
                 sp.setAz( node.x() );
                 sp.setAlt( node.y() );
-                sp.HorizontalToEquatorial( data->LST, data->geo()->lat() );
-                clippedPoly << QPointF( sp.ra()->Hours(), sp.dec()->Degrees() );
+                sp.HorizontalToEquatorial( data->lst(), data->geo()->lat() );
+                clippedPoly << QPointF( sp.ra().Hours(), sp.dec().Degrees() );
             }
         }
     }
@@ -239,7 +234,7 @@ void SkyMap::drawHighlightConstellation( QPainter &psky ) {
     QPointF node = bound->at( 0 );
     pLast.set( node.x(), node.y() );
 
-    pLast.EquatorialToHorizontal( data->LST, data->geo()->lat() );
+    pLast.EquatorialToHorizontal( data->lst(), data->geo()->lat() );
     oLast = toScreen( &pLast, Options::useRefraction(), &isVisibleLast );
 
     int limit = bound->size();
@@ -247,7 +242,7 @@ void SkyMap::drawHighlightConstellation( QPainter &psky ) {
     for ( int i=1 ; i < limit ; i++ ) {
         node = bound->at( i );
         pThis.set( node.x(), node.y() );
-        pThis.EquatorialToHorizontal( data->LST, data->geo()->lat() );
+        pThis.EquatorialToHorizontal( data->lst(), data->geo()->lat() );
         oThis = toScreen( &pThis, Options::useRefraction(), &isVisible );
 
         if ( isVisible && isVisibleLast ) {
@@ -273,7 +268,7 @@ void SkyMap::drawHighlightConstellation( QPainter &psky ) {
         bool isVisible;
     	foreach ( const QPointF &node, cbound ) {
     		SkyPoint sp( node.x(), node.y() );
-    		sp.EquatorialToHorizontal( data->LST, data->geo()->lat() );
+    		sp.EquatorialToHorizontal( data->lst(), data->geo()->lat() );
     		QPointF v = toScreen( &sp, scale, Options::useRefraction() );
 
     		poly << v;
@@ -333,8 +328,10 @@ void SkyMap::drawObjectLabels( QList<SkyObject*>& labelObjects, QPainter &psky )
             if ( obj->name() == i18n( "Neptune" ) && ! Options::showNeptune() ) continue;
             if ( obj->name() == i18n( "Pluto" ) && ! Options::showPluto() ) continue;
         }
-        if ( obj->type() >= SkyObject::OPEN_CLUSTER && obj->type() <= SkyObject::GALAXY 
-             || obj->type() >= SkyObject::ASTERISM || obj->type() <= SkyObject::QUASAR ) {
+        if ( (obj->type() >= SkyObject::OPEN_CLUSTER && obj->type() <= SkyObject::GALAXY) ||
+             (obj->type() >= SkyObject::ASTERISM) ||
+             (obj->type() <= SkyObject::QUASAR) )
+        {
             if ( ((DeepSkyObject*)obj)->isCatalogM() && ! drawMessier ) continue;
             if ( ((DeepSkyObject*)obj)->isCatalogNGC() && ! drawNGC ) continue;
             if ( ((DeepSkyObject*)obj)->isCatalogIC() && ! drawIC ) continue;
@@ -354,31 +351,23 @@ void SkyMap::drawObjectLabels( QList<SkyObject*>& labelObjects, QPainter &psky )
 }
 
 void SkyMap::drawTransientLabel( QPainter &p ) {
-    if ( ! transientObject() || ! checkVisibility( transientObject() ) ) return;
-
+    if( !transientObject() || !checkVisibility( transientObject() ) )
+        return;
     QPointF o = toScreen( transientObject() );
-    if ( ! onScreen( o ) ) return;
+    if( !onScreen( o ) )
+        return;
 
     p.setPen( TransientColor );
     transientObject()->drawRudeNameLabel( p, o );
 }
 
-//FIXME: Implement Options::boxBGMode()
-void SkyMap::drawBoxes( QPainter &p ) {
-    if ( ks ) { //only if we are drawing in the GUI window
-        ks->infoBoxes()->drawBoxes( p,
-                                    data->colorScheme()->colorNamed( "BoxTextColor" ),
-                                    data->colorScheme()->colorNamed( "BoxGrabColor" ),
-                                    data->colorScheme()->colorNamed( "BoxBGColor" ), 1 /*Options::boxBGMode()*/ );
-    }
-}
-
 void SkyMap::drawObservingList( QPainter &psky ) {
     int penWidth = int(m_Scale);
     psky.setPen( QPen( QColor( data->colorScheme()->colorNamed( "ObsListColor" ) ), penWidth ) );
 
-    if ( ks && ks->observingList()->obsList().size() ) {
-        foreach ( SkyObject* obj, ks->observingList()->obsList() ) {
+    KStars* kstars = KStars::Instance();
+    if ( kstars && kstars->observingList()->sessionList().size() ) {
+        foreach ( SkyObject* obj, kstars->observingList()->sessionList() ) {
             if ( checkVisibility( obj ) ) {
                 QPointF o = toScreen( obj );
 
@@ -411,158 +400,135 @@ void SkyMap::drawObservingList( QPainter &psky ) {
 void SkyMap::drawTelescopeSymbols(QPainter &psky)
 {
 #ifdef HAVE_INDI_H
-    if ( ks ) { //ks doesn't exist in non-GUI mode!
-        INDI_P *eqNum;
-        INDI_P *portConnect;
-        INDI_E *lp;
-        INDIMenu *devMenu = ks->indiMenu();
-        bool useJ2000 (false), useAltAz(false);
-        SkyPoint indi_sp;
+    KStars* kstars = KStars::Instance();
+    if( !kstars )
+        return;
 
-        if (!Options::showTargetCrosshair() || devMenu == NULL)
-            return;
+    INDI_P *eqNum;
+    INDI_P *portConnect;
+    INDI_E *lp;
+    INDIMenu *devMenu = kstars->indiMenu();
+    SkyPoint indi_sp;
 
-        psky.setPen( QPen( QColor( data->colorScheme()->colorNamed("TargetColor" ) ) ) );
-        psky.setBrush( Qt::NoBrush );
-        float pxperdegree = Options::zoomFactor()/57.3;
+    if (!Options::showTargetCrosshair() || devMenu == NULL)
+        return;
 
-        //fprintf(stderr, "in draw telescope function with managerssize of %d\n", devMenu->managers.size());
-        for ( int i=0; i < devMenu->managers.size(); i++ )
-        {
-            for ( int j=0; j < devMenu->managers.at(i)->indi_dev.size(); j++ )
-            {
-                useAltAz = false;
-                useJ2000 = false;
+    psky.setPen( QPen( QColor( data->colorScheme()->colorNamed("TargetColor" ) ) ) );
+    psky.setBrush( Qt::NoBrush );
+    float pxperdegree = Options::zoomFactor()/57.3;
 
-                // make sure the dev is on first
-                if (devMenu->managers.at(i)->indi_dev.at(j)->isOn())
-                {
-                    portConnect = devMenu->managers.at(i)->indi_dev.at(j)->findProp("CONNECTION");
+    //fprintf(stderr, "in draw telescope function with managerssize of %d\n", devMenu->managers.size());
+    for ( int i=0; i < devMenu->managers.size(); i++ ) {
+        for ( int j=0; j < devMenu->managers.at(i)->indi_dev.size(); j++ ) {
+            bool useAltAz = false;
+            bool useJ2000 = false;
 
-                    if (!portConnect)
-                        return;
+            // make sure the dev is on first
+            if (devMenu->managers.at(i)->indi_dev.at(j)->isOn()) {
+                portConnect = devMenu->managers.at(i)->indi_dev.at(j)->findProp("CONNECTION");
+                if( !portConnect || portConnect->state == PS_BUSY )
+                    return;
 
-                    if (portConnect->state == PS_BUSY)
-                        return;
-
-                    eqNum = devMenu->managers.at(i)->indi_dev.at(j)->findProp("EQUATORIAL_EOD_COORD");
-
-                    if (eqNum == NULL)
-                    {
-                        eqNum = devMenu->managers.at(i)->indi_dev.at(j)->findProp("EQUATORIAL_COORD");
+                eqNum = devMenu->managers.at(i)->indi_dev.at(j)->findProp("EQUATORIAL_EOD_COORD");
+                if (eqNum == NULL) {
+                    eqNum = devMenu->managers.at(i)->indi_dev.at(j)->findProp("EQUATORIAL_COORD");
+                    if (eqNum == NULL) {
+                        eqNum = devMenu->managers.at(i)->indi_dev.at(j)->findProp("HORIZONTAL_COORD");
                         if (eqNum == NULL)
-                        {
-                            eqNum = devMenu->managers.at(i)->indi_dev.at(j)->findProp("HORIZONTAL_COORD");
-                            if (eqNum == NULL) continue;
-                            else
-                                useAltAz = true;
-                        }
+                            continue;
                         else
-                            useJ2000 = true;
+                            useAltAz = true;
+                    } else {
+                        useJ2000 = true;
+                    }
+                }
+
+                // make sure it has RA and DEC properties
+                if ( eqNum) {
+                    //fprintf(stderr, "Looking for RA label\n");
+                    if (useAltAz) {
+                        lp = eqNum->findElement("AZ");
+                        if (!lp)
+                            continue;
+
+                        dms azDMS(lp->value);
+
+                        lp = eqNum->findElement("ALT");
+                        if (!lp)
+                            continue;
+
+                        dms altDMS(lp->value);
+                        indi_sp.setAz(azDMS);
+                        indi_sp.setAlt(altDMS);
+                    } else {
+                        lp = eqNum->findElement("RA");
+                        if (!lp)
+                            continue;
+
+                        // express hours in degrees on the celestial sphere
+                        dms raDMS(lp->value);
+                        raDMS.setD ( raDMS.Degrees() * 15.0);
+
+                        lp = eqNum->findElement("DEC");
+                        if (!lp)
+                            continue;
+
+                        dms decDMS(lp->value);
+
+                        indi_sp.setRA(raDMS);
+                        indi_sp.setDec(decDMS);
+
+                        if (useJ2000) {
+                            indi_sp.setRA0(raDMS);
+                            indi_sp.setDec0(decDMS);
+                            indi_sp.apparentCoord( (double) J2000, data->ut().djd());
+                        }
+
+                        if ( Options::useAltAz() )
+                            indi_sp.EquatorialToHorizontal( data->lst(), data->geo()->lat() );
                     }
 
-                    // make sure it has RA and DEC properties
-                    if ( eqNum)
-                    {
-                        //fprintf(stderr, "Looking for RA label\n");
-                        if (useAltAz)
-                        {
-                            lp = eqNum->findElement("AZ");
-                            if (!lp)
-                                continue;
+                    QPointF P = toScreen( &indi_sp );
+                    if ( Options::useAntialias() ) {
+                        float s1 = 0.5*pxperdegree;
+                        float s2 = pxperdegree;
+                        float s3 = 2.0*pxperdegree;
 
-                            dms azDMS(lp->value);
+                        float x0 = P.x();        float y0 = P.y();
+                        float x1 = x0 - 0.5*s1;  float y1 = y0 - 0.5*s1;
+                        float x2 = x0 - 0.5*s2;  float y2 = y0 - 0.5*s2;
+                        float x3 = x0 - 0.5*s3;  float y3 = y0 - 0.5*s3;
 
-                            lp = eqNum->findElement("ALT");
-                            if (!lp)
-                                continue;
+                        //Draw radial lines
+                        psky.drawLine( QPointF(x1, y0), QPointF(x3, y0) );
+                        psky.drawLine( QPointF(x0+s2, y0), QPointF(x0+0.5*s1, y0) );
+                        psky.drawLine( QPointF(x0, y1), QPointF(x0, y3) );
+                        psky.drawLine( QPointF(x0, y0+0.5*s1), QPointF(x0, y0+s2) );
+                        //Draw circles at 0.5 & 1 degrees
+                        psky.drawEllipse( QRectF(x1, y1, s1, s1) );
+                        psky.drawEllipse( QRectF(x2, y2, s2, s2) );
 
-                            dms altDMS(lp->value);
+                        psky.drawText( QPointF(x0+s2+2., y0), QString(devMenu->managers.at(i)->indi_dev.at(j)->label) );
+                    } else {
+                        int s1 = int( 0.5*pxperdegree );
+                        int s2 = int( pxperdegree );
+                        int s3 = int( 2.0*pxperdegree );
 
-                            indi_sp.setAz(azDMS);
-                            indi_sp.setAlt(altDMS);
+                        int x0 = int(P.x());   int y0 = int(P.y());
+                        int x1 = x0 - s1/2;  int y1 = y0 - s1/2;
+                        int x2 = x0 - s2/2;  int y2 = y0 - s2/2;
+                        int x3 = x0 - s3/2;  int y3 = y0 - s3/2;
 
-                        }
-                        else
-                        {
+                        //Draw radial lines
+                        psky.drawLine( QPoint(x1, y0),      QPoint(x3, y0) );
+                        psky.drawLine( QPoint(x0+s2, y0),   QPoint(x0+s1/2, y0) );
+                        psky.drawLine( QPoint(x0, y1),      QPoint(x0, y3) );
+                        psky.drawLine( QPoint(x0, y0+s1/2), QPoint(x0, y0+s2) );
+                        //Draw circles at 0.5 & 1 degrees
+                        psky.drawEllipse( QRect(x1, y1, s1, s1) );
+                        psky.drawEllipse( QRect(x2, y2, s2, s2) );
 
-                            lp = eqNum->findElement("RA");
-                            if (!lp)
-                                continue;
-
-                            // express hours in degrees on the celestial sphere
-                            dms raDMS(lp->value);
-                            raDMS.setD ( raDMS.Degrees() * 15.0);
-
-                            lp = eqNum->findElement("DEC");
-                            if (!lp)
-                                continue;
-
-                            dms decDMS(lp->value);
-
-
-                            //kDebug() << "the KStars RA is " << raDMS.toHMSString();
-                            //kDebug() << "the KStars DEC is " << decDMS.toDMSString() << "\n****************";
-
-                            indi_sp.setRA(raDMS);
-                            indi_sp.setDec(decDMS);
-
-                            if (useJ2000)
-                            {
-                                indi_sp.setRA0(raDMS);
-                                indi_sp.setDec0(decDMS);
-                                indi_sp.apparentCoord( (double) J2000, ks->data()->ut().djd());
-                            }
-
-                            if ( Options::useAltAz() )
-                                indi_sp.EquatorialToHorizontal( ks->LST(), ks->geo()->lat() );
-
-                        }
-
-                        QPointF P = toScreen( &indi_sp );
-
-                        if ( Options::useAntialias() ) {
-                            float s1 = 0.5*pxperdegree;
-                            float s2 = pxperdegree;
-                            float s3 = 2.0*pxperdegree;
-
-                            float x0 = P.x();        float y0 = P.y();
-                            float x1 = x0 - 0.5*s1;  float y1 = y0 - 0.5*s1;
-                            float x2 = x0 - 0.5*s2;  float y2 = y0 - 0.5*s2;
-                            float x3 = x0 - 0.5*s3;  float y3 = y0 - 0.5*s3;
-
-                            //Draw radial lines
-                            psky.drawLine( QPointF(x1, y0), QPointF(x3, y0) );
-                            psky.drawLine( QPointF(x0+s2, y0), QPointF(x0+0.5*s1, y0) );
-                            psky.drawLine( QPointF(x0, y1), QPointF(x0, y3) );
-                            psky.drawLine( QPointF(x0, y0+0.5*s1), QPointF(x0, y0+s2) );
-                            //Draw circles at 0.5 & 1 degrees
-                            psky.drawEllipse( QRectF(x1, y1, s1, s1) );
-                            psky.drawEllipse( QRectF(x2, y2, s2, s2) );
-
-                            psky.drawText( QPointF(x0+s2+2., y0), QString(devMenu->managers.at(i)->indi_dev.at(j)->label) );
-
-                        } else {
-                            int s1 = int( 0.5*pxperdegree );
-                            int s2 = int( pxperdegree );
-                            int s3 = int( 2.0*pxperdegree );
-
-                            int x0 = int(P.x());   int y0 = int(P.y());
-                            int x1 = x0 - s1/2;  int y1 = y0 - s1/2;
-                            int x2 = x0 - s2/2;  int y2 = y0 - s2/2;
-                            int x3 = x0 - s3/2;  int y3 = y0 - s3/2;
-
-                            //Draw radial lines
-                            psky.drawLine( QPoint(x1, y0),      QPoint(x3, y0) );
-                            psky.drawLine( QPoint(x0+s2, y0),   QPoint(x0+s1/2, y0) );
-                            psky.drawLine( QPoint(x0, y1),      QPoint(x0, y3) );
-                            psky.drawLine( QPoint(x0, y0+s1/2), QPoint(x0, y0+s2) );
-                            //Draw circles at 0.5 & 1 degrees
-                            psky.drawEllipse( QRect(x1, y1, s1, s1) );
-                            psky.drawEllipse( QRect(x2, y2, s2, s2) );
-
-                            psky.drawText( QPoint(x0+s2+2, y0), QString(devMenu->managers.at(i)->indi_dev.at(j)->label) );
-                        }
+                        psky.drawText( QPoint(x0+s2+2, y0), QString(devMenu->managers.at(i)->indi_dev.at(j)->label) );
                     }
                 }
             }
@@ -609,20 +575,13 @@ void SkyMap::exportSkyImage( QPaintDevice *pd ) {
 }
 
 void SkyMap::setMapGeometry() {
-    m_Guidemax = Options::zoomFactor()/10.0;
-
-    isPoleVisible = false;
+    double Ymax;
     if ( Options::useAltAz() ) {
-        XRange = 1.2*fov()/cos( focus()->alt()->radians() );
-        Ymax = fabs( focus()->alt()->Degrees() ) + fov();
+        XRange = 1.2*fov()/cos( focus()->alt().radians() );
+        Ymax = fabs( focus()->alt().Degrees() ) + fov();
     } else {
-        XRange = 1.2*fov()/cos( focus()->dec()->radians() );
-        Ymax = fabs( focus()->dec()->Degrees() ) + fov();
+        XRange = 1.2*fov()/cos( focus()->dec().radians() );
+        Ymax = fabs( focus()->dec().Degrees() ) + fov();
     }
-    if ( Ymax >= 90. ) isPoleVisible = true;
-
-    //at high zoom, double FOV for guide lines so they don't disappear.
-    guideFOV = fov();
-    guideXRange = XRange;
-    if ( Options::zoomFactor() > 10.*MINZOOM ) { guideFOV *= 2.0; guideXRange *= 2.0; }
+    isPoleVisible = Ymax >= 90.0;
 }
