@@ -18,7 +18,7 @@
 #include "jupitermoons.h"
 
 #include "ksnumbers.h"
-#include "ksplanet.h"
+#include "ksplanetbase.h"
 #include "kssun.h"
 #include "trailobject.h"
 
@@ -39,7 +39,7 @@ JupiterMoons::JupiterMoons(){
 JupiterMoons::~JupiterMoons(){
 }
 
-void JupiterMoons::findPosition( const KSNumbers *num, const KSPlanet *Jupiter, const KSSun *Sun ) {
+void JupiterMoons::findPosition( const KSNumbers *num, const KSPlanetBase *Jupiter, const KSSun *Sun ) {
     double Xj, Yj, Zj, Rj;
     double sinJB, cosJB, sinJL, cosJL;
     double sinSB, cosSB, sinSL, cosSL;
@@ -479,8 +479,8 @@ void JupiterMoons::findPosition( const KSNumbers *num, const KSPlanet *Jupiter,
         YP[i] = A6[i] * sin( D ) + C6[i] * cos( D );
         ZP[i] = B6[i];
 
-        Moon[i]->setRA( Jupiter->ra()->Hours() - 0.011*( XP[i] * cos( pa ) - YP[i] * sin( pa ) )/15.0 );
-        Moon[i]->setDec( Jupiter->dec()->Degrees() - 0.011*( XP[i] * sin( pa ) + YP[i] * cos( pa ) ) );
+        Moon[i]->setRA( Jupiter->ra().Hours() - 0.011*( XP[i] * cos( pa ) - YP[i] * sin( pa ) )/15.0 );
+        Moon[i]->setDec( Jupiter->dec().Degrees() - 0.011*( XP[i] * sin( pa ) + YP[i] * cos( pa ) ) );
 
         if ( ZP[i] < 0.0 ) InFront[i] = true;
         else InFront[i] = false;
@@ -488,7 +488,8 @@ void JupiterMoons::findPosition( const KSNumbers *num, const KSPlanet *Jupiter,
         //Update Trails
         if ( Moon[i]->hasTrail() ) {
             Moon[i]->addToTrail();
-            if ( Moon[i]->trail().size() > MAXTRAIL ) Moon[i]->trail().takeFirst();
+            if ( Moon[i]->trail().size() > MAXTRAIL )
+                Moon[i]->clipTrail();
         }
     }
 }
