@@ -5,7 +5,7 @@
 // find a copy of this license in LICENSE.txt in the top directory of
 // the source code.
 //
-// Copyright 2006-2007 Torsten Rahn <tackat@kde.org>
+// Copyright 2006-2010 Torsten Rahn <tackat@kde.org>
 // Copyright 2007      Inge Wallin  <ingwa@kde.org>
 //
 
@@ -16,6 +16,7 @@
 #include <QtCore/QSettings>
 #include <QtCore/QUrl>
 #include <QtGui/QCloseEvent>
+#include <QtCore/QTimer>
 
 #include <QtGui/QAction>
 #include <QtGui/QLabel>
@@ -30,6 +31,7 @@
 #include <QtGui/QMessageBox>
 
 #include <QtGui/QPrintDialog>
+#include <QtGui/QPrintPreviewDialog>
 #include <QtGui/QPrinter>
 #include <QtGui/QPainter>
 
@@ -37,17 +39,19 @@
 
 #include <QtNetwork/QNetworkProxy>
 
-#include <MarbleDirs.h>
-#include "lib/MarbleAboutDialog.h"
-#include "lib/QtMarbleConfigDialog.h"
-#include "lib/SunControlWidget.h"
-#include "lib/MarbleLocale.h"
+#include "MarbleDirs.h"
+#include "MarbleAboutDialog.h"
+#include "QtMarbleConfigDialog.h"
+#include "SunControlWidget.h"
+#include "MarbleLocale.h"
+#include "DownloadRegionDialog.h"
+#include "ViewParams.h"
+#include "ViewportParams.h"
 #include "AbstractDataPlugin.h"
 #include "AbstractFloatItem.h"
-#include "HttpDownloadManager.h"
 #include "MarbleMap.h"
 #include "MarbleModel.h"
-
+#include "SunLocator.h"
 
 namespace
 {
@@ -57,8 +61,12 @@ namespace
 
 using namespace Marble;
 
-MainWindow::MainWindow(const QString& marbleDataPath, QWidget *parent) : QMainWindow(parent), m_sunControlDialog(0)
+MainWindow::MainWindow(const QString& marbleDataPath, QWidget *parent) :
+        QMainWindow(parent), m_sunControlDialog(0), m_downloadRegionAction( 0 ),
+        m_downloadRegionDialog( 0 )
 {
+    MarbleGlobal::getInstance()->setProfiles( MarbleGlobal::detectProfiles() );
+
     setUpdatesEnabled( false );
     
     QString selectedPath = marbleDataPath.isEmpty() ? readMarbleDataPath() : marbleDataPath;
@@ -114,12 +122,21 @@ void MainWindow::createActions()
      m_exportMapAct->setShortcut(tr("Ctrl+S"));
      m_exportMapAct->setStatusTip(tr("Save a screenshot of the map"));
      connect(m_exportMapAct, SIGNAL(triggered()), this, SLOT(exportMapScreenShot()));
+     
+     // Action: Download Region
+     m_downloadRegionAction = new QAction( tr( "Download Region..." ), this );
+     m_downloadRegionAction->setStatusTip( tr( "Download a map region in different zoom levels for offline usage" ) );
+     connect( m_downloadRegionAction, SIGNAL( triggered() ), SLOT( showDownloadRegionDialog() ) );
 
      m_printAct = new QAction( QIcon(":/icons/document-print.png"), tr("&Print..."), this);
      m_printAct->setShortcut(tr("Ctrl+P"));
      m_printAct->setStatusTip(tr("Print a screenshot of the map"));
      connect(m_printAct, SIGNAL(triggered()), this, SLOT(printMapScreenShot()));
 
+     m_printPreviewAct = new QAction( QIcon(":/icons/document-printpreview.png"), tr("Print Previe&w ..."), this);
+     m_printPreviewAct->setStatusTip(tr("Print a screenshot of the map"));
+     connect(m_printPreviewAct, SIGNAL(triggered()), this, SLOT(printPreview()));
+     
      m_quitAct = new QAction( QIcon(":/icons/application-exit.png"), tr("&Quit"), this);
      m_quitAct->setShortcut(tr("Ctrl+Q"));
      m_quitAct->setStatusTip(tr("Quit the Application"));
@@ -130,7 +147,7 @@ void MainWindow::createActions()
      m_copyMapAct->setStatusTip(tr("Copy a screenshot of the map"));
      connect(m_copyMapAct, SIGNAL(triggered()), this, SLOT(copyMap()));
      
-     m_configDialogAct = new QAction( tr("&Settings"), this);
+     m_configDialogAct = new QAction( tr("&Configure Marble"), this);
      m_configDialogAct->setStatusTip(tr("Show the configuration dialog"));
      connect(m_configDialogAct, SIGNAL(triggered()), this, SLOT(editSettings()));
 
@@ -160,7 +177,7 @@ void MainWindow::createActions()
      m_lockFloatItemsAct = new QAction( tr("Lock Position"),this);
      m_lockFloatItemsAct->setCheckable( true );
      m_lockFloatItemsAct->setStatusTip(tr("Lock Position of Floating Items"));
-     connect(m_lockFloatItemsAct, SIGNAL(toggled( bool )), this, SLOT( lockPosition( bool )));
+     connect(m_lockFloatItemsAct, SIGNAL(triggered( bool )), this, SLOT( lockPosition( bool )));
 
      m_showCloudsAct = new QAction( tr("&Clouds"), this);
      m_showCloudsAct->setCheckable( true );
@@ -180,6 +197,11 @@ void MainWindow::createActions()
      m_controlSunAct->setStatusTip(tr("Configure Sun Control"));
      connect(m_controlSunAct, SIGNAL(triggered()), this, SLOT( controlSun()));
 
+     m_reloadAct = new QAction( tr("&Redisplay"), this);
+     m_reloadAct->setShortcut(tr("F5"));
+     m_reloadAct->setStatusTip(tr("Reload Current Map"));
+     connect(m_reloadAct, SIGNAL(triggered()), this, SLOT(reload()));
+     
      m_handbookAct = new QAction( QIcon(":/icons/help-contents.png"), tr("Marble Desktop Globe &Handbook"), this);
      m_handbookAct->setShortcut(tr("F1"));
      m_handbookAct->setStatusTip(tr("Show the Handbook for Marble Desktop Globe"));
@@ -202,11 +224,25 @@ void MainWindow::createActions()
 
 void MainWindow::createMenus()
 {
+    // Do not create too many menu entries on a MID
+    if( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
+        menuBar()->addAction( m_workOfflineAct );
+        menuBar()->addAction( m_sideBarAct );
+        /** @todo: Full screen cannot be left on Maemo currently (shortcuts not working) */
+        //menuBar()->addAction( m_fullScreenAct );
+        menuBar()->addAction( m_downloadRegionAction );
+        menuBar()->addAction( m_aboutMarbleAct );
+        return;
+    }
+    
     m_fileMenu = menuBar()->addMenu(tr("&File"));
     m_fileMenu->addAction(m_openAct);
     m_fileMenu->addAction(m_downloadAct);
+    m_fileMenu->addAction( m_downloadRegionAction );
     m_fileMenu->addAction(m_exportMapAct);
+    m_fileMenu->addSeparator();
     m_fileMenu->addAction(m_printAct);
+    m_fileMenu->addAction(m_printPreviewAct);
     m_fileMenu->addSeparator();
     m_fileMenu->addAction(m_workOfflineAct);
     m_fileMenu->addAction(m_quitAct);
@@ -225,6 +261,7 @@ void MainWindow::createMenus()
             m_fileMenu->addAction( (*i)->action() );
         }
     }
+    m_fileMenu->addAction(m_reloadAct); 
 
     m_fileMenu->addSeparator();
     m_infoBoxesMenu = m_fileMenu->addMenu("&Info Boxes");
@@ -268,8 +305,15 @@ void MainWindow::createMenus()
 }
 
 void MainWindow::createInfoBoxesMenu()
-{
+{    
     m_infoBoxesMenu->clear();
+    
+    // Do not create too many menu entries on a MID
+    // FIXME: Set up another way of switching the plugins on and off.
+    if( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
+        return;
+    }
+    
     m_infoBoxesMenu->addAction(m_lockFloatItemsAct);
     m_infoBoxesMenu->addSeparator();
 
@@ -287,6 +331,12 @@ void MainWindow::createOnlineServicesMenu()
 {
     m_onlineServicesMenu->clear();
     
+    // Do not create too many menu entries on a MID
+    // FIXME: Set up another way of switching the plugins on and off.
+    if( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
+        return;
+    }
+    
     QList<RenderPlugin *> renderPluginList = m_controlView->marbleWidget()->renderPlugins();
     
     QList<RenderPlugin *>::const_iterator i = renderPluginList.constBegin();
@@ -302,13 +352,19 @@ void MainWindow::createOnlineServicesMenu()
 }
 
 void MainWindow::createPluginMenus()
-{
-    //Remove and delete toolbars if they exist
+{    
+    // Remove and delete toolbars if they exist
     while( !m_pluginToolbars.isEmpty() ) {
         QToolBar* tb = m_pluginToolbars.takeFirst();
         this->removeToolBar(tb);
         delete tb;
     }
+    
+    // Do not create too many menu entries on a MID
+    // FIXME: Set up another way of switching the plugins on and off.
+    if( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
+        return;
+    }
 
     //remove and delete old menus if they exist
     while(  !m_pluginMenus.isEmpty() ) {
@@ -384,33 +440,60 @@ void MainWindow::exportMapScreenShot()
 void MainWindow::printMapScreenShot()
 {
 #ifndef QT_NO_PRINTER
-    QPixmap mapPixmap = m_controlView->mapScreenShot();
-
-    QSize printSize = mapPixmap.size();
-
-    QPrinter printer;
-
-    QPrintDialog printDialog( &printer, this );
-
-    if (printDialog.exec() == QDialog::Accepted) {
-
-        QRect mapPageRect = printer.pageRect();
-
-        printSize.scale( ( printer.pageRect() ).size(), Qt::KeepAspectRatio );
-
-        QPoint printTopLeft( mapPageRect.width() /2  - printSize.width() /2 ,
-                             mapPageRect.height()/2  - printSize.height()/2 );
-
-        QRect mapPrintRect( printTopLeft, printSize );
-
-        QPainter painter( &printer );
-
-        painter.drawPixmap( mapPrintRect, mapPixmap, mapPixmap.rect() );
+    QPrinter printer( QPrinter::HighResolution );
+    QPointer<QPrintDialog> printDialog = new QPrintDialog( &printer, this );
 
+    if (printDialog->exec() == QDialog::Accepted) {
+        QPixmap mapPixmap = m_controlView->mapScreenShot();
+        printPixmap( &printer, mapPixmap );
     }
+    delete printDialog;
 #endif    
 }
 
+void MainWindow::printPixmap( QPrinter * printer, const QPixmap& pixmap  )
+{
+#ifndef QT_NO_PRINTER
+    QSize printSize = pixmap.size();
+    
+    QRect mapPageRect = printer->pageRect();
+
+    printSize.scale( printer->pageRect().size(), Qt::KeepAspectRatio );
+
+    QPoint printTopLeft( ( mapPageRect.width() - printSize.width() ) / 2 ,
+                         ( mapPageRect.height() - printSize.height() ) / 2 );
+
+    QRect mapPrintRect( printTopLeft, printSize );
+
+    QPainter painter;
+    if (!painter.begin(printer))
+        return;
+    painter.drawPixmap( mapPrintRect, pixmap, pixmap.rect() );
+    painter.end();
+#endif
+}
+
+void MainWindow::printPreview()
+{
+#ifndef QT_NO_PRINTER
+    QPrinter printer( QPrinter::HighResolution );
+
+    QPointer<QPrintPreviewDialog> preview = new QPrintPreviewDialog( &printer, this );
+    preview->setWindowFlags ( Qt::Window );
+    connect( preview, SIGNAL( paintRequested( QPrinter * ) ), SLOT( paintPrintPreview( QPrinter * ) ) );
+    preview->exec();
+    delete preview;
+#endif
+}
+
+void MainWindow::paintPrintPreview( QPrinter * printer )
+{
+#ifndef QT_NO_PRINTER
+    QPixmap mapPixmap = m_controlView->mapScreenShot();
+    printPixmap( printer, mapPixmap );
+#endif
+}
+
 void MainWindow::showFullScreen( bool isChecked )
 {
     if ( isChecked ) {
@@ -472,9 +555,7 @@ void MainWindow::showClouds( bool isChecked )
 
 void MainWindow::workOffline( bool offline )
 {
-    HttpDownloadManager * const downloadManager =
-        m_controlView->marbleWidget()->map()->model()->downloadManager();
-    downloadManager->setDownloadEnabled( !offline );
+    m_controlView->marbleControl()->setWorkOffline( offline );
 }
 
 void MainWindow::showAtmosphere( bool isChecked )
@@ -516,6 +597,11 @@ void MainWindow::showSun( bool active )
     m_controlView->marbleWidget()->sunLocator()->setShow( active ); 
 }
 
+void MainWindow::reload()
+{
+    m_controlView->marbleWidget()->map()->reload();
+}
+
 void MainWindow::enterWhatsThis()
 {
     QWhatsThis::enterWhatsThisMode();
@@ -565,18 +651,17 @@ void MainWindow::updateStatusBar()
 
 void MainWindow::openFile()
 {
-    QString fileName;
-    fileName = QFileDialog::getOpenFileName(this, tr("Open File"),
+    QStringList fileNames = QFileDialog::getOpenFileNames(this, tr("Open File"),
                             QString(), 
                             tr("All Supported Files (*.gpx *.kml);;GPS Data (*.gpx);;Google Earth KML (*.kml)"));
 
-    if ( ! fileName.isNull() ) {
+    foreach( const QString &fileName, fileNames ) {
         QString extension = fileName.section( '.', -1 );
 
         if ( extension.compare( "gpx", Qt::CaseInsensitive ) == 0 ) {
             m_controlView->marbleWidget()->openGpxFile( fileName );
         }
-        else if ( extension.compare( "kml", Qt::CaseInsensitive ) 
+        else if ( extension.compare( "kml", Qt::CaseInsensitive )
                   == 0 ) 
         {
             m_controlView->marbleWidget()->addPlacemarkFile( fileName );
@@ -640,7 +725,12 @@ void MainWindow::readSettings()
          resize(settings.value("size", QSize(640, 480)).toSize());
          move(settings.value("pos", QPoint(200, 200)).toPoint());
          showFullScreen(settings.value("fullScreen", false ).toBool());
-         showSideBar(settings.value("sideBar", true ).toBool());
+         if( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
+             showSideBar(settings.value("sideBar", false ).toBool());
+         }
+         else {
+             showSideBar(settings.value("sideBar", true ).toBool());
+         }
          showStatusBar(settings.value("statusBar", false ).toBool());
          show();
          showClouds(settings.value("showClouds", true ).toBool());
@@ -651,7 +741,7 @@ void MainWindow::readSettings()
      setUpdatesEnabled(false);
 
      settings.beginGroup("MarbleWidget");
-         QString mapThemeId = settings.value("mapTheme", "" ).toString();
+         QString mapThemeId = settings.value("mapTheme", m_controlView->defaultMapThemeId() ).toString();
          qDebug() << "ReadSettings: mapThemeId: " << mapThemeId;
          m_controlView->marbleWidget()->setMapThemeId( mapThemeId );
          m_controlView->marbleWidget()->setProjection(
@@ -676,7 +766,9 @@ void MainWindow::readSettings()
 	    m_controlView->marbleWidget()->goHome();
 	 }
          
-         m_lockFloatItemsAct->setChecked( settings.value( "lockFloatItemPositions", false ).toBool() );
+         bool isLocked = settings.value( "lockFloatItemPositions", false ).toBool();
+         m_lockFloatItemsAct->setChecked( isLocked );
+         lockPosition(isLocked);
      settings.endGroup();
      
      settings.beginGroup( "Sun" );
@@ -774,7 +866,66 @@ void MainWindow::updateSettings()
     m_controlView->marbleWidget()->setPersistentTileCacheLimit( m_configDialog->persistentTileCacheLimit() * 1024 );
     m_controlView->marbleWidget()->setVolatileTileCacheLimit( m_configDialog->volatileTileCacheLimit() * 1024 );
     
+    /*
+    m_controlView->marbleWidget()->setProxy( m_configDialog->proxyUrl(), m_configDialog->proxyPort(), m_configDialog->user(), m_configDialog->password() );
+    */
+
     m_controlView->marbleWidget()->updateChangedMap();
 }
 
+void MainWindow::showDownloadRegionDialog()
+{
+    MarbleModel *const model = m_controlView->marbleWidget()->map()->model();
+    if ( !m_downloadRegionDialog ) {
+        m_downloadRegionDialog = new DownloadRegionDialog( model, m_controlView );
+        // it might be tempting to move the connects to DownloadRegionDialog's "accepted" and
+        // "applied" signals, be aware that the "hidden" signal might be come before the "accepted"
+        // signal, leading to a too early disconnect.
+        connect( m_downloadRegionDialog, SIGNAL( accepted() ), SLOT( downloadRegion() ));
+        connect( m_downloadRegionDialog, SIGNAL( applied() ), SLOT( downloadRegion() ));
+        connect( m_downloadRegionDialog, SIGNAL( shown() ), SLOT( connectDownloadRegionDialog() ));
+        connect( m_downloadRegionDialog, SIGNAL( hidden() ),
+                 SLOT( disconnectDownloadRegionDialog() ));
+    }
+    // FIXME: get allowed range from current map theme
+    m_downloadRegionDialog->setAllowedTileLevelRange( 0, 18 );
+    m_downloadRegionDialog->setSelectionMethod( DownloadRegionDialog::VisibleRegionMethod );
+    ViewportParams const * const viewport =
+        m_controlView->marbleWidget()->map()->viewParams()->viewport();
+    m_downloadRegionDialog->setSpecifiedLatLonAltBox( viewport->viewLatLonAltBox() );
+    m_downloadRegionDialog->setVisibleLatLonAltBox( viewport->viewLatLonAltBox() );
+
+    m_downloadRegionDialog->show();
+    m_downloadRegionDialog->raise();
+    m_downloadRegionDialog->activateWindow();
+}
+
+// connect to expensive slots, only needed when the non modal dialog is show
+void MainWindow::connectDownloadRegionDialog()
+{
+    connect( m_controlView->marbleWidget(), SIGNAL( visibleLatLonAltBoxChanged( GeoDataLatLonAltBox )),
+             m_downloadRegionDialog, SLOT( setVisibleLatLonAltBox( GeoDataLatLonAltBox )));
+    connect( m_controlView->marbleWidget(), SIGNAL( themeChanged( QString )),
+             m_downloadRegionDialog, SLOT( updateTextureLayer() ));
+}
+
+// disconnect from expensive slots, not needed when dialog is hidden
+void MainWindow::disconnectDownloadRegionDialog()
+{
+    disconnect( m_controlView->marbleWidget(), SIGNAL( visibleLatLonAltBoxChanged( GeoDataLatLonAltBox )),
+                m_downloadRegionDialog, SLOT( setVisibleLatLonAltBox( GeoDataLatLonAltBox )));
+    disconnect( m_controlView->marbleWidget(), SIGNAL( themeChanged( QString )),
+                m_downloadRegionDialog, SLOT( updateTextureLayer() ));
+}
+
+void MainWindow::downloadRegion()
+{
+    Q_ASSERT( m_downloadRegionDialog );
+    QString const mapThemeId = m_controlView->marbleWidget()->mapThemeId();
+    QString const sourceDir = mapThemeId.left( mapThemeId.lastIndexOf( '/' ));
+    mDebug() << "downloadRegion mapThemeId:" << mapThemeId << sourceDir;
+    TileCoordsPyramid const pyramid = m_downloadRegionDialog->region();
+    m_controlView->marbleWidget()->map()->model()->downloadRegion( sourceDir, pyramid );
+}
+
 #include "QtMainWindow.moc"
