@@ -17,15 +17,12 @@
 #define MARBLE_ABSTRACTTILE_H
 
 #include <QtCore/QObject>
-#include <QtCore/QDateTime>
-
-#include "TileId.h"
-
 
 namespace Marble
 {
 
 class AbstractTilePrivate;
+class TileId;
 
 class AbstractTile : public QObject
 {
@@ -56,10 +53,6 @@ class AbstractTile : public QObject
     void setUsed( bool used );
 
     TileState state() const;
-    void setState( TileState state );
-
-    const QDateTime & created() const;
-    void setCreated( QDateTime created );
 
 protected:
     AbstractTilePrivate * const d_ptr;
@@ -68,7 +61,6 @@ protected:
 private:
     Q_DECLARE_PRIVATE( AbstractTile )
     Q_DISABLE_COPY( AbstractTile )
-
 };
 
 }
