@@ -27,10 +27,10 @@ using namespace Marble;
 
 // Only remove 20 files without checking
 // changed cacheLimits and changed themes etc.
-static const quint8 maxFilesDelete = 20;
+static const int maxFilesDelete = 20;
 // Delete only files that are older than 120 Seconds
-static const quint8 deleteOnlyFilesOlderThan = 120;
-static const quint8 softLimitPercent = 5;
+static const int deleteOnlyFilesOlderThan = 120;
+static const int softLimitPercent = 5;
 
 
 // Methods of FileStorageWatcherThread
@@ -84,7 +84,7 @@ void FileStorageWatcherThread::resetCurrentSize()
     emit variableChanged();
 }
 
-void FileStorageWatcherThread::updateTheme( QString mapTheme )
+void FileStorageWatcherThread::updateTheme( const QString &mapTheme )
 {
     mDebug() << "Theme changed to " << mapTheme;
     m_themeMutex.lock();
@@ -199,7 +199,8 @@ void FileStorageWatcherThread::ensureCacheSize()
     }
 }
 
-void FileStorageWatcherThread::ensureSizePerPlanet( QString planetDirectory, QString currentTheme )
+void FileStorageWatcherThread::ensureSizePerPlanet( const QString &planetDirectory,
+                                                    const QString &currentTheme )
 {
     mDebug() << "Deleting from folder: " << planetDirectory;
     
@@ -238,7 +239,7 @@ static bool greaterThanByNumber( const QString &s1, const QString &s2)
     return s1.toInt() > s2.toInt();
 }
 
-void FileStorageWatcherThread::ensureSizePerTheme( QString themeDirectory )
+void FileStorageWatcherThread::ensureSizePerTheme( const QString &themeDirectory )
 {
     mDebug() << "Deleting from folder: " << themeDirectory;
 
@@ -366,7 +367,7 @@ void FileStorageWatcher::resetCurrentSize()
     emit cleared();
 }
 
-void FileStorageWatcher::updateTheme( QString mapTheme )
+void FileStorageWatcher::updateTheme( const QString &mapTheme )
 {
     QMutexLocker locker( m_themeLimitMutex );
     if( m_started )
