@@ -8,38 +8,34 @@
 // Copyright 2007      Murad Tagirov <tmurad@gmail.com>
 //
 
+#ifndef MARBLE_KMLFILEVIEWITEM_H
+#define MARBLE_KMLFILEVIEWITEM_H
 
-#ifndef KMLFILEVIEWITEM_H
-#define KMLFILEVIEWITEM_H
-
-#include <QtCore/QVariant>
 #include "AbstractFileViewItem.h"
 #include "GeoDataDocument.h"
 
 namespace Marble
 {
 
-class PlacemarkManager;
+class FileManager;
 
 class KmlFileViewItem : public AbstractFileViewItem
 {
   public:
-    KmlFileViewItem( PlacemarkManager& manager, const GeoDataDocument& document );
+    KmlFileViewItem( FileManager &manager, const GeoDataDocument& document );
     ~KmlFileViewItem();
 
     /*
      * AbstractFileViewItem methods
      */
     virtual void saveFile();
-    virtual void closeFile( int start, bool finalize = true );
-    virtual QVariant data( int role = Qt::DisplayRole ) const;
     virtual bool isShown() const;
     virtual void setShown( bool value );
-    virtual int size() const;
+    virtual QString name() const;
 
     GeoDataDocument* document();
   private:
-    PlacemarkManager&   m_placemarkManager;
+    FileManager  &m_fileManager;
     GeoDataDocument    m_document;
 };
 
