@@ -23,40 +23,34 @@
 
 #include "MarbleControlBox.h"
 
-#include <QtCore/QAbstractItemModel>
-#include <QtCore/QBuffer>
-#include <QtCore/QByteArray>
-#include <QtCore/QDataStream>
-#include <QtCore/QtAlgorithms>
 #include <QtCore/QTime>
 #include <QtCore/QTimer>
 #include <QtGui/QSortFilterProxyModel>
 #include <QtGui/QStandardItemModel>
-#include <QtGui/QTextFrame>
 
-#include <global.h>
+#include "global.h"
 
-#include <MarbleWidget.h>
-#include <MarbleMap.h>
-#include <ViewParams.h>
-#include <MarbleModel.h>
-#include <MarbleDirs.h>
-#include <FileViewModel.h>
+#include "MarbleWidget.h"
+#include "MarbleMap.h"
+#include "MarbleModel.h"
+#include "FileViewModel.h"
 #include "gps/GpxFileModel.h"
 #include "gps/GpsLayer.h"
 #include "gps/PositionTracking.h"
 #include "MarbleLocale.h"
-#include "MarbleDebug.h"
 #include "MarblePlacemarkModel.h"
+#include "RoutingWidget.h"
 #include "MarbleRunnerManager.h"
 #include "MathHelper.h"
 #include "MapThemeSortFilterProxyModel.h"
 #include "GeoSceneDocument.h"
 #include "GeoSceneHead.h"
 #include "Planet.h"
+#include "HttpDownloadManager.h"
+#include "PositionProviderPlugin.h"
+#include "PluginManager.h"
 
-#include "GeoOnfParser.h"
-#include "GeoDataDocument.h"
+#include "GeoDataCoordinates.h"
 
 using namespace Marble;
 
@@ -84,14 +78,17 @@ class MarbleControlBoxPrivate
     QStandardItemModel     *m_mapThemeModel;
     QSortFilterProxyModel  *m_sortproxy;
     MapThemeSortFilterProxyModel *m_mapSortProxy;
-    
+
     MarbleRunnerManager  *m_runnerManager;
+    RoutingWidget  *m_routingWidget;
     GeoSceneDocument      *mapTheme;
 
     MarbleLocale* m_locale;
+    QList<PositionProviderPlugin*> m_positionProviderPlugins;
+    GeoDataCoordinates m_currentPosition;
 };
 
-MarbleControlBoxPrivate::MarbleControlBoxPrivate()
+MarbleControlBoxPrivate::MarbleControlBoxPrivate() : m_routingWidget(0)
 {
 }
 
@@ -105,7 +102,7 @@ MarbleControlBox::MarbleControlBox(QWidget *parent)
     d->m_widget = 0;
     d->m_searchTerm.clear();
     d->m_searchTriggered = false;
-    d->uiWidget.setupUi( this );    
+    d->uiWidget.setupUi( this );
 
     setFocusPolicy( Qt::NoFocus );
 //    setFocusProxy( d->uiWidget.searchLineEdit );
@@ -175,11 +172,11 @@ MarbleControlBox::MarbleControlBox(QWidget *parent)
              this,                      SLOT( adjustForStill() ) );
 
     d->uiWidget.projectionComboBox->setEnabled( true );
-    
+
     d->m_runnerManager = new MarbleRunnerManager( this );
 
     connect( d->m_runnerManager, SIGNAL( modelChanged(  MarblePlacemarkModel* ) ),
-             this,               SLOT( setLocations( MarblePlacemarkModel* ) ) );    
+             this,               SLOT( setLocations( MarblePlacemarkModel* ) ) );
 
     connect( d->uiWidget.searchLineEdit,  SIGNAL( textChanged( const QString& ) ),
              this,                        SLOT( searchLineChanged( const QString& ) ) );
@@ -237,7 +234,7 @@ void MarbleControlBox::updateCelestialModel()
                                 << new QStandardItem( celestialBodyName )
                                 << new QStandardItem( celestialBodyId ) );
         }
-    }   
+    }
 }
 
 void MarbleControlBox::updateButtons( int value )
@@ -258,12 +255,16 @@ void MarbleControlBox::updateButtons( int value )
 void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
 {
     d->m_widget = widget;
+    d->m_runnerManager->setMap( d->m_widget->map() );
+
+    d->m_routingWidget = new RoutingWidget( widget, this );
+    d->uiWidget.toolBox->addItem( d->m_routingWidget, tr( "Routing" ) );
 
     // Make us aware of all the Placemarks in the MarbleModel so that
     // we can search them.
     setLocations( static_cast<MarblePlacemarkModel*>(d->m_widget->placemarkModel()) );
 
-//    FIXME: Why does this fail: "selection model works on a different model than the view..." ? 
+//    FIXME: Why does this fail: "selection model works on a different model than the view..." ?
 //    d->uiWidget.locationListView->setSelectionModel( d->m_widget->placemarkSelectionModel() );
 
     //set up everything for the FileModel
@@ -294,7 +295,7 @@ void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
     connect( this, SIGNAL(moveUp()),    d->m_widget, SLOT(moveUp()) );
     connect( this, SIGNAL(moveDown()),  d->m_widget, SLOT(moveDown()) );
 
-    connect( this,        SIGNAL( projectionSelected( Projection ) ),  
+    connect( this,        SIGNAL( projectionSelected( Projection ) ),
              d->m_widget, SLOT( setProjection( Projection ) ) );
 
     connect( d->m_widget, SIGNAL( themeChanged( QString ) ),
@@ -317,6 +318,18 @@ void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
     connect( d->uiWidget.marbleLegendBrowser, SIGNAL( toggledShowProperty( QString, bool ) ),
              d->m_widget,                     SLOT( setPropertyValue( QString, bool ) ) );
 
+    PluginManager* pluginManager = d->m_widget->model()->pluginManager();
+    d->m_positionProviderPlugins = pluginManager->createPositionProviderPlugins();
+    foreach( const PositionProviderPlugin *plugin, d->m_positionProviderPlugins ) {
+       d->uiWidget.positionTrackingComboBox->addItem( plugin->guiString() );
+    }
+    if ( d->m_positionProviderPlugins.isEmpty() ) {
+        d->uiWidget.positionTrackingComboBox->setEnabled( false );
+        QString html = "<p>No Position Tracking Plugin installed.</p>";
+        d->uiWidget.locationLabel->setText( html );
+        d->uiWidget.locationLabel->setEnabled ( true );
+    }
+
     //connect CurrentLoctaion signals
     connect( this, SIGNAL( gpsInputDisabled( bool ) ),
              d->m_widget, SLOT( setShowGps( bool ) ) );
@@ -325,11 +338,13 @@ void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
     connect( d->m_widget->model()->gpsLayer()->getPositionTracking(),
              SIGNAL( gpsLocation( GeoDataCoordinates, qreal ) ),
              this, SLOT( receiveGpsCoordinates( GeoDataCoordinates, qreal ) ) );
-    connect( d->uiWidget.navigationCheckBox, SIGNAL(clicked(bool) ),
-             d->m_widget->model()->gpsLayer(), SLOT( setVisible(bool) ) );
-
-    connect( d->m_widget, SIGNAL( timeout() ),
-             this,        SIGNAL( updateGps() ) );
+    connect( d->uiWidget.positionTrackingComboBox, SIGNAL( currentIndexChanged( QString ) ),
+             this, SLOT( changePositionProvider( QString ) ) );
+    connect( d->uiWidget.locationLabel, SIGNAL( linkActivated( QString ) ),
+             this, SLOT( centerOnCurrentLocation() ) );
+    connect( d->m_widget->model()->gpsLayer()->getPositionTracking(),
+             SIGNAL( statusChanged( PositionProviderStatus) ), this,
+             SLOT( adjustPositionTrackingStatus( PositionProviderStatus) ) );
 }
 
 void MarbleControlBox::setWidgetTabShown( QWidget * widget,
@@ -377,61 +392,105 @@ int MarbleControlBox::minimumZoom() const
 void MarbleControlBox::updateMapThemeView()
 {
     updateCelestialModel();
-    
+
     if ( d->m_widget ) {
         QString mapThemeId = d->m_widget->mapThemeId();
-        if ( !mapThemeId.isEmpty() ) 
+        if ( !mapThemeId.isEmpty() )
             selectTheme( mapThemeId );
     }
 }
 
 void MarbleControlBox::changeZoom(int zoom)
 {
-    // No infinite loops here
-    // if (zoomSlider->value() != zoom)
+    // There exists a circular signal/slot connection between MarbleWidget and this widget's
+    // zoom slider. MarbleWidget prevents recursion, but it still loops one time unless
+    // blocked here. Note that it would be possible to only connect the sliders
+    // sliderMoved signal instead of its valueChanged signal above to break up the loop.
+    // This however means that the slider cannot be operated with the mouse wheel, as this
+    // does not emit the sliderMoved signal for some reason. Therefore the signal is blocked
+    // below before calling setValue on the slider to avoid that it calls back to MarbleWidget,
+    // and then un-blocked again to make user interaction possible.
+
+    d->uiWidget.zoomSlider->blockSignals( true );
+
     d->uiWidget.zoomSlider->setValue( zoom );
     d->uiWidget.zoomSlider->setMinimum( minimumZoom() );
+
+    d->uiWidget.zoomSlider->blockSignals( false );
 }
 
-
-void MarbleControlBox::receiveGpsCoordinates( GeoDataCoordinates in, qreal speed )
+void MarbleControlBox::adjustPositionTrackingStatus( PositionProviderStatus status )
 {
-    Q_UNUSED( speed );
-    if ( d->uiWidget.navigationCheckBox->isChecked() ) {
-        QString unitString;
-        QString speedString;
-        QString distanceUnitString;
-        QString distanceString;
-        qreal unitSpeed = 0.0;
-        qreal distance = 0.0;
-
-        switch ( d->m_locale->measureSystem() ) {
-            case Marble::Metric:
-            //kilometers per hour
-            unitString = tr("km/h");
-            unitSpeed = speed / 1000;
-            distanceUnitString = tr("m");
-            distance = in.altitude();
-            break;
-
-            case Marble::Imperial:
-            //miles per hour
-            unitString = tr("m/h");
-            unitSpeed = (speed/1000) * KM2MI;
-            distanceUnitString = tr("ft");
-            distance = in.altitude() * M2FT;
-            break;
-        }
-        // TODO read this value from the incoming signal
-        speedString = QLocale::system().toString( unitSpeed, 'f', 1);
-        distanceString = QString( "%1 %2" ).arg( distance, 0, 'f', 1, QChar(' ') ).arg( distanceUnitString );
-
-        d->uiWidget.speedLabel->setText( speedString.rightJustified(5) + ' ' + unitString );
-
-        d->uiWidget.longitudeValue->setText( in.lonToString() );
-        d->uiWidget.latitudeValue->setText( in.latToString() );
-        d->uiWidget.altitudeValue->setText( distanceString );
+    if ( status == PositionProviderStatusAvailable ) {
+        return;
     }
+
+    QString html = "<html><body><p>";
+
+    switch ( status ) {
+        case PositionProviderStatusUnavailable:
+            html += tr( "Waiting for current location information..." );
+            break;
+        case PositionProviderStatusAcquiring:
+            html += tr( "Initializing current location service..." );
+            break;
+        case PositionProviderStatusAvailable:
+            Q_ASSERT( false );
+            break;
+        case PositionProviderStatusError:
+            html += tr( "Error when determining current location: " );
+            html += d->m_widget->model()->gpsLayer()->getPositionTracking()->error();
+            break;
+    }
+
+    html += "</p></body></html>";
+    d->uiWidget.locationLabel->setEnabled( true );
+    d->uiWidget.locationLabel->setText( html );
+}
+
+void MarbleControlBox::receiveGpsCoordinates( const GeoDataCoordinates &position, qreal speed )
+{
+    d->m_currentPosition = position;
+    QString unitString;
+    QString speedString;
+    QString distanceUnitString;
+    QString distanceString;
+    qreal unitSpeed = 0.0;
+    qreal distance = 0.0;
+
+    QString html = "<html><body>";
+    html += "<table cellspacing=\"2\" cellpadding=\"2\">";
+    html += "<tr><td>Longitude</td><td><a href=\"http://edu.kde.org/marble\">%1</a></td></tr>";
+    html += "<tr><td>Latitude</td><td><a href=\"http://edu.kde.org/marble\">%2</a></td></tr>";
+    html += "<tr><td>Altitude</td><td>%3</td></tr>";
+    html += "<tr><td>Speed</td><td>%4</td></tr>";
+    html += "</table>";
+    html += "</body></html>";
+
+    switch ( d->m_locale->measureSystem() ) {
+        case Metric:
+        //kilometers per hour
+        unitString = tr("km/h");
+        unitSpeed = speed / 1000;
+        distanceUnitString = tr("m");
+        distance = position.altitude();
+        break;
+
+        case Imperial:
+        //miles per hour
+        unitString = tr("m/h");
+        unitSpeed = (speed/1000) * KM2MI;
+        distanceUnitString = tr("ft");
+        distance = position.altitude() * M2FT;
+        break;
+    }
+    // TODO read this value from the incoming signal
+    speedString = QLocale::system().toString( unitSpeed, 'f', 1);
+    distanceString = QString( "%1 %2" ).arg( distance, 0, 'f', 1, QChar(' ') ).arg( distanceUnitString );
+
+    html = html.arg( position.lonToString() ).arg( position.latToString() );
+    html = html.arg( distanceString ).arg( speedString + ' ' + unitString );
+    d->uiWidget.locationLabel->setText( html );
 }
 
 
@@ -442,18 +501,11 @@ void MarbleControlBox::enableFileViewActions()
     d->uiWidget.m_saveButton->setEnabled( tmp );
     d->uiWidget.m_closeButton->setEnabled( tmp );
 
-#ifndef KML_GSOC
     if ( tmp ) {
         QModelIndex tmpIndex =
             d->uiWidget.m_fileView->selectionModel()->currentIndex();
         d->m_widget->gpxFileModel()->setSelectedIndex( tmpIndex );
     }
-#else
-    if ( tmp ) {
-        FileViewModel& model = ( FileViewModel& ) *d->uiWidget.m_fileView->model();
-        model.setSelectedIndex( d->uiWidget.m_fileView->selectionModel()->currentIndex() );
-    }
-#endif
 }
 
 void MarbleControlBox::setNavigationTabShown( bool show )
@@ -484,6 +536,17 @@ void MarbleControlBox::setCurrentLocationTabShown( bool show )
 {
     QString  title = tr( "Current Location" );
     setWidgetTabShown( d->m_currentLocation2Widget, 4, show, title );
+    if ( d->m_widget ) {
+        bool enabled = d->m_widget->mapTheme()->head()->target() == "earth";
+        int locationIndex = d->uiWidget.toolBox->indexOf( d->m_currentLocation2Widget );
+        if ( locationIndex >= 0 ) {
+            d->uiWidget.toolBox->setItemEnabled( locationIndex, enabled );
+        }
+
+        if ( !enabled ) {
+            d->m_widget->map()->model()->gpsLayer()->setVisible( false );
+        }
+    }
 }
 
 
@@ -524,11 +587,8 @@ void MarbleControlBox::searchLineChanged(const QString &search)
 void MarbleControlBox::searchReturnPressed()
 {
     // do nothing if search term empty
-    if ( d->m_searchTerm.isEmpty() ) {
-        return;
-    } else {
+    if ( !d->m_searchTerm.isEmpty() ) {
         d->m_runnerManager->newText( d->m_searchTerm );
-        return;
     }
 }
 
@@ -548,7 +608,7 @@ void MarbleControlBox::selectTheme( const QString &theme )
         return;
     // Check if the new selected theme is different from the current one
     QModelIndex currentIndex = d->uiWidget.marbleThemeSelectView->currentIndex();
-    QString indexTheme = d->m_mapSortProxy->data( d->m_mapSortProxy->index( 
+    QString indexTheme = d->m_mapSortProxy->data( d->m_mapSortProxy->index(
                          currentIndex.row(), 1, QModelIndex() ) ).toString();
 
 
@@ -556,8 +616,8 @@ void MarbleControlBox::selectTheme( const QString &theme )
     updateButtons( d->uiWidget.zoomSlider->value() );
 
     if ( theme != indexTheme ) {
-        /* indexTheme would be empty if the chosen map has not been set yet. As 
-        this needs to be done after the mapThemeId has been set, check if that is 
+        /* indexTheme would be empty if the chosen map has not been set yet. As
+        this needs to be done after the mapThemeId has been set, check if that is
         not empty first. The behaviour differs between Linux and Windows: on
         Windows the reading of the settings is not delayed, thus the mapThemeId
         is available earlier than on Linux.
@@ -567,7 +627,7 @@ void MarbleControlBox::selectTheme( const QString &theme )
             if( items.size() >= 1 ) {
                 QModelIndex iterIndex = items.first()->index();
                 QModelIndex iterIndexName = d->m_mapSortProxy->mapFromSource( iterIndex.sibling( iterIndex.row(), 0 ) );
-		
+
                 d->uiWidget.marbleThemeSelectView->setCurrentIndex( iterIndexName );
 
                 d->uiWidget.marbleThemeSelectView->scrollTo( iterIndexName );
@@ -576,7 +636,13 @@ void MarbleControlBox::selectTheme( const QString &theme )
 
         QString selectedId = d->m_widget->mapTheme()->head()->target();
         d->m_runnerManager->setCelestialBodyId( selectedId );
-        
+        int routingIndex = d->uiWidget.toolBox->indexOf( d->m_routingWidget );
+        d->uiWidget.toolBox->setItemEnabled( routingIndex, selectedId == "earth" );
+        int locationIndex = d->uiWidget.toolBox->indexOf( d->m_currentLocation2Widget );
+        if ( locationIndex >= 0 ) {
+            d->uiWidget.toolBox->setItemEnabled( locationIndex, selectedId == "earth" );
+        }
+
         QList<QStandardItem*> itemList = d->m_celestialList->findItems( selectedId, Qt::MatchExactly, 1 );
 
         if ( !itemList.isEmpty() ) {
@@ -608,7 +674,7 @@ void MarbleControlBox::selectCurrentMapTheme( const QString& celestialBodyId )
     bool foundMapTheme = false;
 
     QString currentMapThemeId = d->m_widget->mapThemeId();
-    
+
     int row = d->m_mapSortProxy->rowCount();
 
     for ( int i = 0; i < row; ++i )
@@ -641,11 +707,11 @@ void MarbleControlBox::mapCenterOnSignal( const QModelIndex &index )
 
 void MarbleControlBox::adjustForAnimation()
 {
-    // TODO: use signals here as well 
+    // TODO: use signals here as well
     if ( !d->m_widget )
         return;
 
-    d->m_widget->setViewContext( Marble::Animation );
+    d->m_widget->setViewContext( Animation );
 }
 
 void MarbleControlBox::adjustForStill()
@@ -654,15 +720,54 @@ void MarbleControlBox::adjustForStill()
     if ( !d->m_widget )
         return;
 
-    d->m_widget->setViewContext( Marble::Still );
+    d->m_widget->setViewContext( Still );
 
-    if ( d->m_widget->mapQuality( Marble::Still )
-        != d->m_widget->mapQuality( Marble::Animation ) )
+    if ( d->m_widget->mapQuality( Still )
+        != d->m_widget->mapQuality( Animation ) )
     {
         d->m_widget->updateChangedMap();
     }
 }
 
+void MarbleControlBox::setWorkOffline(bool offline)
+{
+    HttpDownloadManager * const downloadManager =
+        d->m_widget->map()->model()->downloadManager();
+    downloadManager->setDownloadEnabled( !offline );
+    d->m_runnerManager->setWorkOffline( offline );
+    if ( d->m_routingWidget ) {
+        d->m_routingWidget->setWorkOffline( offline );
+    }
+}
+
+void MarbleControlBox::changePositionProvider( const QString &provider )
+{
+    if ( provider == tr("Disabled") ) {
+        d->uiWidget.locationLabel->setEnabled( false );
+        d->m_widget->map()->model()->gpsLayer()->setVisible( false );
+        d->m_widget->model()->gpsLayer()->getPositionTracking()->setPositionProviderPlugin( 0 );
+        d->m_widget->update();
+    }
+    else {
+        foreach( PositionProviderPlugin* plugin, d->m_positionProviderPlugins ) {
+            if ( plugin->guiString() == provider ) {
+               d->uiWidget.locationLabel->setEnabled( true );
+               PositionProviderPlugin* instance = plugin->newInstance();
+               PositionTracking *tracking = d->m_widget->model()->gpsLayer()->getPositionTracking();
+               tracking->setPositionProviderPlugin( instance );
+               d->m_widget->map()->model()->gpsLayer()->setVisible( true );
+               d->m_widget->update();
+               return;
+            }
+        }
+    }
+}
+
+void MarbleControlBox::centerOnCurrentLocation()
+{
+    d->m_widget->centerOn(d->m_currentPosition, true);
+}
+
 }
 
 #include "MarbleControlBox.moc"
