@@ -5,9 +5,9 @@
 // find a copy of this license in LICENSE.txt in the top directory of
 // the source code.
 //
-// Copyright 2006-2007 Torsten Rahn <tackat@kde.org>"
-// Copyright 2007      Inge Wallin  <ingwa@kde.org>"
-// Copyright 2008      Jens-Michael Hoffmann <jensmh@gmx.de>
+// Copyright 2006-2007 Torsten Rahn <tackat@kde.org>
+// Copyright 2007      Inge Wallin  <ingwa@kde.org>
+// Copyright 2008, 2009, 2010 Jens-Michael Hoffmann <jmho@c-xx.com>
 // Copyright 2008-2009      Patrick Spendrin <ps_ml@gmx.de>
 //
 
@@ -16,20 +16,25 @@
 #include <cmath>
 
 #include <QtCore/QAtomicInt>
-#include <QtCore/QDebug>
+#include <QtCore/QPointer>
 #include <QtCore/QTime>
 #include <QtCore/QTimer>
 #include <QtCore/QAbstractItemModel>
 #include <QtGui/QItemSelectionModel>
 #include <QtGui/QSortFilterProxyModel>
 
+#include "MapThemeManager.h"
 #include "global.h"
+#include "MarbleDebug.h"
 #include "gps/GpsLayer.h"
 
 #include "GeoSceneDocument.h"
+#include "GeoSceneFilter.h"
 #include "GeoSceneHead.h"
 #include "GeoSceneLayer.h"
 #include "GeoSceneMap.h"
+#include "GeoScenePalette.h"
+#include "GeoSceneSettings.h"
 #include "GeoSceneTexture.h"
 #include "GeoSceneVector.h"
 #include "GeoSceneXmlDataSource.h"
@@ -38,7 +43,7 @@
 #include "GeoDataStyle.h"
 
 #include "DgmlAuxillaryDictionary.h"
-
+#include "ExtDateTime.h"
 #include "FileStoragePolicy.h"
 #include "GeoPainter.h"
 #include "FileViewModel.h"
@@ -46,6 +51,7 @@
 #include "EquirectScanlineTextureMapper.h"
 #include "MercatorScanlineTextureMapper.h"
 #include "gps/GpxFileModel.h"
+#include "gps/GpxFile.h"
 #include "HttpDownloadManager.h"
 #include "KmlFileViewItem.h"
 #include "LayerManager.h"
@@ -54,17 +60,20 @@
 #include "MarblePlacemarkModel.h"
 #include "MarbleGeometryModel.h"
 #include "MergedLayerDecorator.h"
+#include "FileManager.h"
 #include "PlacemarkManager.h"
 #include "PlacemarkLayout.h"
 #include "PlacemarkPainter.h"
 #include "Planet.h"
+#include "PluginManager.h"
 #include "StoragePolicy.h"
 #include "SunLocator.h"
 #include "TextureColorizer.h"
-#include "TextureTile.h"
+#include "StackedTile.h"
+#include "TileCoordsPyramid.h"
 #include "TileCreator.h"
 #include "TileCreatorDialog.h"
-#include "TileLoader.h"
+#include "StackedTileLoader.h"
 #include "VectorComposer.h"
 #include "ViewParams.h"
 #include "ViewportParams.h"
@@ -78,10 +87,15 @@ class MarbleModelPrivate
     MarbleModelPrivate( MarbleModel *parent )
         : m_parent( parent ),
           m_dataFacade( 0 ),
+          m_pluginManager( new PluginManager( parent ) ),
+          m_mapThemeManager( new MapThemeManager( parent )),
           m_mapTheme( 0 ),
           m_layerManager( 0 ),
-          m_downloadManager( new HttpDownloadManager( QUrl(), new FileStoragePolicy(
-                                                                   MarbleDirs::localPath() )))
+          m_downloadManager( new HttpDownloadManager( new FileStoragePolicy(
+                                                                   MarbleDirs::localPath() ),
+                                                      m_pluginManager ) ),
+          m_fileManager( 0 ),
+          m_placemarkmanager( 0 )
     {
     }
 
@@ -92,12 +106,17 @@ class MarbleModelPrivate
 
     void resize( int width, int height );
     void notifyModelChanged();
-    void geoDataDocumentAdded( const GeoDataDocument& document );
+    GeoSceneGroup * textureLayerProperties() const;
+
+    void drawFog( QPainter *painter, ViewParams *viewParams, int width, int height );
 
     static QAtomicInt       refCounter;
     MarbleModel             *m_parent;
     MarbleDataFacade        *m_dataFacade;
 
+    PluginManager           *m_pluginManager;
+    MapThemeManager         *m_mapThemeManager;
+
     // View and paint stuff
     GeoSceneDocument        *m_mapTheme;
     LayerManager            *m_layerManager;
@@ -105,16 +124,15 @@ class MarbleModelPrivate
 
     HttpDownloadManager     *m_downloadManager;
 
-    TileLoader              *m_tileLoader;
+    StackedTileLoader       *m_tileLoader;
     AbstractScanlineTextureMapper   *m_texmapper;
 
     static VectorComposer   *m_veccomposer; // FIXME: Make not a pointer.
 
     // Places on the map
+    FileManager             *m_fileManager;
     PlacemarkManager        *m_placemarkmanager;
-    MarblePlacemarkModel    *m_placemarkmodel;
     PlacemarkLayout         *m_placemarkLayout;
-    MarbleGeometryModel     *m_geometrymodel;
     QSortFilterProxyModel   *m_popSortModel;
 
     // Misc stuff.
@@ -139,46 +157,41 @@ VectorComposer      *MarbleModelPrivate::m_veccomposer = 0;
 TextureColorizer    *MarbleModelPrivate::m_texcolorizer = 0;
 QAtomicInt           MarbleModelPrivate::refCounter(0);
 
+GeoSceneGroup * MarbleModelPrivate::textureLayerProperties() const
+{
+    if ( !m_mapTheme )
+        return 0;
+
+    GeoSceneSettings * const settings = m_mapTheme->settings();
+    if ( !settings )
+        return 0;
+
+    return settings->group( "Texture Layers" );
+}
+
 MarbleModel::MarbleModel( QObject *parent )
     : QObject( parent ),
       d( new MarbleModelPrivate( this ) )
 {
+    QTime t;
+    t.start();
     MarbleModelPrivate::refCounter.ref();
     d->m_dataFacade = new MarbleDataFacade( this );
-    d->m_layerManager = new LayerManager( d->m_dataFacade, this );
-
-    // FIXME: more on the spot update names and API
-    connect ( d->m_layerManager,      SIGNAL( floatItemsChanged() ),
-              this,                   SIGNAL( modelChanged() ) );
-
-    d->m_timer = new QTimer( this );
-    d->m_timer->start( 200 );
-
-    connect( d->m_timer, SIGNAL( timeout() ),
-             this,       SIGNAL( timeout() ) );
-
-    d->m_tileLoader = new TileLoader( d->m_downloadManager, this );
 
+    d->m_tileLoader = new StackedTileLoader( d->m_mapThemeManager, d->textureLayerProperties(),
+                                             d->m_downloadManager, this );
     d->m_texmapper = 0;
     
-    if( MarbleModelPrivate::refCounter == 1 ) {
-        d->m_veccomposer = new VectorComposer();
-        d->m_texcolorizer = 0;
-        /* d->m_texcolorizer is not initialized here since it takes a long time
-           to create the palette and it might not even be used. Instead it's created
-           in setMapTheme if the theme being loaded does need it. If the theme
-           doesn't need it, it's left as is. */
-    }
+    d->m_fileManager = new FileManager();
+    d->m_fileManager->setDataFacade(d->m_dataFacade);
 
     d->m_placemarkmanager = new PlacemarkManager();
+    d->m_placemarkmanager->setDataFacade(d->m_dataFacade);
+    d->m_placemarkmanager->setFileManager(d->m_fileManager);
 
-    connect( d->m_placemarkmanager, SIGNAL( geoDataDocumentAdded( const GeoDataDocument& ) ),
-             this,                  SLOT( geoDataDocumentAdded( const GeoDataDocument& ) ) );
-
-    d->m_placemarkmodel = new MarblePlacemarkModel( d->m_placemarkmanager, this );
     d->m_popSortModel = new QSortFilterProxyModel( this );
 
-    d->m_popSortModel->setSourceModel( d->m_placemarkmodel );
+    d->m_popSortModel->setSourceModel( d->m_dataFacade->placemarkModel() );
 //    d->m_popSortModel->setSortLocaleAware( true );
 //    d->m_popSortModel->setDynamicSortFilter( true );
     d->m_popSortModel->setSortRole( MarblePlacemarkModel::PopularityIndexRole );
@@ -186,35 +199,49 @@ MarbleModel::MarbleModel( QObject *parent )
     
     d->m_placemarkselectionmodel = new QItemSelectionModel( d->m_popSortModel );
 
-    d->m_geometrymodel = new MarbleGeometryModel();
-    d->m_placemarkmanager->setGeoModel( d->m_geometrymodel );
-
     d->m_placemarkLayout = new PlacemarkLayout( this );
-    connect( d->m_placemarkmanager,         SIGNAL( finalize() ),
-             d->m_placemarkLayout,          SLOT( requestStyleReset() ) );
     connect( d->m_placemarkselectionmodel,  SIGNAL( selectionChanged( QItemSelection,
                                                                       QItemSelection) ),
              d->m_placemarkLayout,          SLOT( requestStyleReset() ) );
-    connect( d->m_placemarkmodel,           SIGNAL( layoutChanged() ),
+    connect( d->m_dataFacade->placemarkModel(),           SIGNAL( layoutChanged() ),
              d->m_placemarkLayout,          SLOT( requestStyleReset() ) );
 
     /*
      * Create FileViewModel
      */
-    connect( fileViewModel(), SIGNAL( modelChanged() ),
+    connect( d->m_dataFacade->fileViewModel(), SIGNAL( modelChanged() ),
              this,            SIGNAL( modelChanged() ) );
 
     d->m_gpxFileModel = new GpxFileModel( this );
     d->m_gpsLayer = new GpsLayer( d->m_gpxFileModel );
 
-    connect( fileViewModel(), SIGNAL(layoutChanged()),
+    connect( d->m_dataFacade->fileViewModel(), SIGNAL(layoutChanged()),
              d->m_gpsLayer, SLOT(clearModel() ) );
 
+    d->m_layerManager = new LayerManager( d->m_dataFacade, d->m_pluginManager, this );
+
+    // FIXME: more on the spot update names and API
+    connect ( d->m_layerManager,      SIGNAL( floatItemsChanged() ),
+              this,                   SIGNAL( modelChanged() ) );
+
+    connect ( d->m_layerManager, SIGNAL( pluginSettingsChanged() ),
+              this,              SIGNAL( pluginSettingsChanged() ) );
+    connect ( d->m_layerManager, SIGNAL( repaintNeeded( QRegion ) ),
+              this,              SIGNAL( repaintNeeded( QRegion ) ) );
+    connect ( d->m_layerManager, SIGNAL( renderPluginInitialized( RenderPlugin * ) ),
+              this,              SIGNAL( renderPluginInitialized( RenderPlugin * ) ) );
+
+    d->m_timer = new QTimer( this );
+    d->m_timer->start( 200 );
+
+    connect( d->m_timer, SIGNAL( timeout() ),
+             this,       SIGNAL( timeout() ) );
+
     d->m_dateTime       = new ExtDateTime();
     /* Assume we are dealing with the earth */
     d->m_planet = new Planet( "earth" );
     d->m_sunLocator     = new SunLocator( d->m_dateTime, d->m_planet );
-    d->m_layerDecorator = new MergedLayerDecorator( d->m_sunLocator );
+    d->m_layerDecorator = new MergedLayerDecorator( d->m_tileLoader, d->m_sunLocator );
 
     connect(d->m_dateTime,   SIGNAL( timeChanged() ),
             d->m_sunLocator, SLOT( update() ) );
@@ -224,7 +251,7 @@ MarbleModel::MarbleModel( QObject *parent )
 
 MarbleModel::~MarbleModel()
 {
-//    qDebug() << "MarbleModel::~MarbleModel";
+//    mDebug() << "MarbleModel::~MarbleModel";
     
     delete d->m_texmapper;
 
@@ -234,10 +261,9 @@ MarbleModel::~MarbleModel()
         delete d->m_veccomposer;
         delete d->m_texcolorizer;
     }
-    delete d->m_geometrymodel;
-    delete d->m_placemarkmodel;
     delete d->m_popSortModel;
     delete d->m_placemarkmanager;
+    delete d->m_fileManager;
     delete d->m_gpsLayer;
     delete d->m_mapTheme;
     delete d->m_timer;
@@ -249,7 +275,7 @@ MarbleModel::~MarbleModel()
     delete d->m_planet;
     delete d;
     MarbleModelPrivate::refCounter.deref();
-    qDebug() << "Model deleted:" << this;
+    mDebug() << "Model deleted:" << this;
 }
 
 bool MarbleModel::showGps() const
@@ -300,27 +326,29 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
                                Projection currentProjection )
 {
     d->m_mapTheme = mapTheme;
+    addDownloadPolicies( d->m_mapTheme );
+    d->m_tileLoader->setTextureLayerSettings( d->textureLayerProperties() );
 
     // Some output to show how to use this stuff ...
-    qDebug() << "DGML2 Name       : " << d->m_mapTheme->head()->name(); 
+    mDebug() << "DGML2 Name       : " << d->m_mapTheme->head()->name(); 
 /*
-    qDebug() << "DGML2 Description: " << d->m_mapTheme->head()->description(); 
+    mDebug() << "DGML2 Description: " << d->m_mapTheme->head()->description(); 
 
     if ( d->m_mapTheme->map()->hasTextureLayers() )
-        qDebug() << "Contains texture layers! ";
+        mDebug() << "Contains texture layers! ";
     else
-        qDebug() << "Does not contain any texture layers! ";
+        mDebug() << "Does not contain any texture layers! ";
 
-    qDebug() << "Number of SRTM textures: " << d->m_mapTheme->map()->layer("srtm")->datasets().count();
+    mDebug() << "Number of SRTM textures: " << d->m_mapTheme->map()->layer("srtm")->datasets().count();
 
     if ( d->m_mapTheme->map()->hasVectorLayers() )
-        qDebug() << "Contains vector layers! ";
+        mDebug() << "Contains vector layers! ";
     else
-        qDebug() << "Does not contain any vector layers! ";
+        mDebug() << "Does not contain any vector layers! ";
 */
     //Don't change the planet unless we have to...
     if( d->m_mapTheme->head()->target().toLower() != d->m_planet->id() ) {
-        qDebug() << "Changing Planet";
+        mDebug() << "Changing Planet";
         *(d->m_planet) = Planet( d->m_mapTheme->head()->target().toLower() );
         sunLocator()->setPlanet(d->m_planet);
     }
@@ -342,10 +370,10 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
         QString installMap = texture->installMap();
         QString role = d->m_mapTheme->map()->layer( themeID )->role();
 
-        if ( !TileLoader::baseTilesAvailable( layer )
+        if ( !StackedTileLoader::baseTilesAvailable( layer )
             && !installMap.isEmpty() )
         {
-            qDebug() << "Base tiles not available. Creating Tiles ... \n"
+            mDebug() << "Base tiles not available. Creating Tiles ... \n"
                      << "SourceDir: " << sourceDir << "InstallMap:" << installMap;
             MarbleDirs::debug();
 
@@ -354,13 +382,13 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
                                      installMap,
                                      (role == "dem") ? "true" : "false" );
 
-            TileCreatorDialog  tileCreatorDlg( tileCreator, 0 );
-            tileCreatorDlg.setSummary( d->m_mapTheme->head()->name(),
-                                       d->m_mapTheme->head()->description() );
-            tileCreatorDlg.exec();
+            QPointer<TileCreatorDialog> tileCreatorDlg = new TileCreatorDialog( tileCreator, 0 );
+            tileCreatorDlg->setSummary( d->m_mapTheme->head()->name(),
+                                        d->m_mapTheme->head()->description() );
+            tileCreatorDlg->exec();
             qDebug("Tile creation completed");
+            delete tileCreatorDlg;
         }
-        d->m_tileLoader->setLayer( layer );
     }
     else {
         d->m_tileLoader->flush();
@@ -369,9 +397,9 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
     setupTextureMapper( currentProjection );
 
     // Set all the colors for the vector layers
-    if ( d->m_mapTheme->map()->hasVectorLayers() ) {
+    if ( d->m_mapTheme->map()->hasVectorLayers() && d->m_veccomposer ) {
         d->m_veccomposer->setOceanColor( d->m_mapTheme->map()->backgroundColor() );
-
+        
         // Just as with textures, this is a workaround for DGML2 to
         // emulate the old behaviour.
 
@@ -403,9 +431,8 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
             }
         }
     }
-
-    d->m_geometrymodel->setGeoDataRoot( 0 );
-    QStringList loadedContainers = d->m_placemarkmanager->containers();
+    d->m_dataFacade->geometryModel()->setGeoDataRoot( 0 );
+    QStringList loadedContainers = d->m_fileManager->containers();
     QStringList loadList;
     const QVector<GeoSceneLayer*> & layers = d->m_mapTheme->map()->layers();
     QVector<GeoSceneLayer*>::const_iterator it = layers.constBegin();
@@ -423,7 +450,7 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
                     QString containername = reinterpret_cast<GeoSceneXmlDataSource*>(dataset)->filename();
                     if( containername.endsWith(".kml") ) containername.remove(".kml");
                     loadedContainers.removeOne( containername );
-                    loadList << containername;          
+                    loadList << containername;
                 }
             }
         }
@@ -431,14 +458,15 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
     // unload old standard Placemarks which are not part of the new map
     foreach(const QString& container, loadedContainers) {
         loadedContainers.pop_front();
-        d->m_placemarkmanager->removePlacemarkKey( container );
+        d->m_fileManager->removeFile( container );
     }
     // load new standard Placemarks
     foreach(const QString& container, loadList) {
         loadList.pop_front();
-        d->m_placemarkmanager->addPlacemarkFile( container, loadList.isEmpty() );
+        d->m_fileManager->addFile( container );
     }
     d->notifyModelChanged();
+    d->m_placemarkLayout->requestStyleReset();
 
 
     // FIXME: Still needs to get fixed for the DGML2 refactoring
@@ -474,60 +502,67 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme,
             }
         }
     }
-    qDebug() << "THEME CHANGED: ***" << mapTheme->head()->mapThemeId();
+    mDebug() << "THEME CHANGED: ***" << mapTheme->head()->mapThemeId();
     emit themeChanged( mapTheme->head()->mapThemeId() );
 
     d->m_layerManager->syncViewParamsAndPlugins( mapTheme );
 
     d->notifyModelChanged();
-    d->m_placemarkLayout->requestStyleReset();
 }
 
 void MarbleModel::setupTextureMapper( Projection projection )
 {
   // FIXME: replace this with an approach based on the factory method pattern.
-
     delete d->m_texmapper;
 
     switch( projection ) {
         case Spherical:
-            d->m_texmapper = new SphericalScanlineTextureMapper( d->m_tileLoader, this );
+            d->m_texmapper = new SphericalScanlineTextureMapper( textureLayer(), d->m_tileLoader,
+                                                                 this );
             break;
         case Equirectangular:
-            d->m_texmapper = new EquirectScanlineTextureMapper( d->m_tileLoader, this );
+            d->m_texmapper = new EquirectScanlineTextureMapper( textureLayer(), d->m_tileLoader,
+                                                                this );
             break;
         case Mercator:
-            d->m_texmapper = new MercatorScanlineTextureMapper( d->m_tileLoader, this );
+            d->m_texmapper = new MercatorScanlineTextureMapper( textureLayer(), d->m_tileLoader,
+                                                                this );
             break;
+        default:
+            d->m_texmapper = 0;
     }
-
+    Q_ASSERT( d->m_texmapper );
+    connect( d->m_texmapper, SIGNAL( tileLevelChanged( int )), SIGNAL( tileLevelChanged( int )));
     connect( d->m_texmapper, SIGNAL( mapChanged() ),
              this,           SLOT( notifyModelChanged() ) );
 }
 
+void MarbleModel::setupVectorComposer()
+{
+    if( MarbleModelPrivate::refCounter == 1 ) {
+        d->m_veccomposer = new VectorComposer();
+        connect( d->m_veccomposer, SIGNAL( datasetLoaded() ), SIGNAL( modelChanged() ) );
+        d->m_texcolorizer = 0;
+        /* d->m_texcolorizer is not initialized here since it takes a long time
+           to create the palette and it might not even be used. Instead it's created
+           in setMapTheme if the theme being loaded does need it. If the theme
+           doesn't need it, it's left as is. */
+    }    
+}
+
 HttpDownloadManager* MarbleModel::downloadManager() const
 {
     return d->m_downloadManager;
 }
 
-
-void MarbleModel::setDownloadManager( HttpDownloadManager *downloadManager )
-{
-    HttpDownloadManager *previousDownloadManager = d->m_downloadManager;
-
-    d->m_tileLoader->setDownloadManager( downloadManager );
-    d->m_downloadManager = downloadManager;
-
-    delete previousDownloadManager;
-}
-
-
 void MarbleModelPrivate::resize( int width, int height )
 {
     if ( m_mapTheme->map()->hasTextureLayers() ) {
         m_texmapper->resizeMap( width, height );
     }
-    m_veccomposer->resizeMap( width, height );
+    if ( m_veccomposer ) {
+        m_veccomposer->resizeMap( width, height );
+    }
 }
 
 
@@ -557,7 +592,8 @@ void MarbleModel::paintGlobe( GeoPainter *painter,
             // Create the height map image a.k.a viewParams->m_canvasImage.
             d->m_texmapper->mapTexture( viewParams );
 
-            if ( !viewParams->showElevationModel()
+            if ( d->m_veccomposer
+                && !viewParams->showElevationModel()
                 && layer->role() == "dem"
                 && !d->m_mapTheme->map()->filters().isEmpty() ) {
 
@@ -571,7 +607,7 @@ void MarbleModel::paintGlobe( GeoPainter *painter,
                 if( filter->type() == "colorize" ) {
                     d->m_texcolorizer->colorize( viewParams );
                 }
-            } //else { qDebug() << "No filters to act on..."; }
+            } //else { mDebug() << "No filters to act on..."; }
         }
     }
 
@@ -598,7 +634,7 @@ void MarbleModel::paintGlobe( GeoPainter *painter,
     renderPositions << "SURFACE";
 
     // Paint the vector layer.
-    if ( d->m_mapTheme->map()->hasVectorLayers() ) {
+    if ( d->m_veccomposer && d->m_mapTheme->map()->hasVectorLayers() ) {
 
         if ( !d->m_mapTheme->map()->hasTextureLayers() ) {
             d->m_veccomposer->paintBaseVectorMap( painter, viewParams );
@@ -637,8 +673,20 @@ void MarbleModel::paintGlobe( GeoPainter *painter,
                                viewParams );
 
     renderPositions.clear();
-    renderPositions << "HOVERS_ABOVE_SURFACE" << "ATMOSPHERE"
-                    << "ORBIT" << "ALWAYS_ON_TOP" << "FLOAT_ITEM";
+    renderPositions << "HOVERS_ABOVE_SURFACE";
+    d->m_layerManager->renderLayers( painter, viewParams, renderPositions );
+
+    // FIXME: This is really slow. That's why we defer this to
+    //        PrintQuality. Either cache on a pixmap - or maybe
+    //        better: Add to GlobeScanlineTextureMapper.
+
+    if ( viewParams->mapQuality() == PrintQuality )
+        d->drawFog( painter, viewParams,
+                    width, height );
+
+    renderPositions.clear();
+    renderPositions << "ATMOSPHERE"
+                    << "ORBIT" << "ALWAYS_ON_TOP" << "FLOAT_ITEM" << "USER_TOOLS";
                                
     d->m_layerManager->renderLayers( painter, viewParams, renderPositions );
 }
@@ -646,7 +694,7 @@ void MarbleModel::paintGlobe( GeoPainter *painter,
 
 QAbstractItemModel *MarbleModel::placemarkModel() const
 {
-    return d->m_placemarkmodel;
+    return d->m_dataFacade->placemarkModel();
 }
 
 QItemSelectionModel *MarbleModel::placemarkSelectionModel() const
@@ -654,11 +702,6 @@ QItemSelectionModel *MarbleModel::placemarkSelectionModel() const
     return d->m_placemarkselectionmodel;
 }
 
-QAbstractItemModel *MarbleModel::geometryModel() const
-{
-    return d->m_geometrymodel;
-}
-
 VectorComposer *MarbleModel::vectorComposer() const
 {
     return d->m_veccomposer;
@@ -691,26 +734,33 @@ GpxFileModel *MarbleModel::gpxFileModel() const
 
 FileViewModel *MarbleModel::fileViewModel() const
 {
-    return d->m_placemarkmanager->fileViewModel();
+    return d->m_dataFacade->fileViewModel();
+}
+
+void MarbleModel::openGpxFile( const QString& filename )
+{
+    GpxFile* gpxFile = new GpxFile( filename );
+    d->m_fileManager->addFile( filename );
+    d->m_gpxFileModel->addFile( gpxFile );
 }
 
 void MarbleModel::addPlacemarkFile( const QString& filename )
 {
-    d->m_placemarkmanager->loadKml( filename, false );
+    d->m_fileManager->addFile( filename );
 
     d->notifyModelChanged();
 }
 
 void MarbleModel::addPlacemarkData( const QString& data, const QString& key )
 {
-    d->m_placemarkmanager->addPlacemarkData( data, key );
+    d->m_fileManager->addData( key, data );
 
     d->notifyModelChanged();
 }
 
-void MarbleModel::removePlacemarkKey( const QString& key )
+void MarbleModel::removePlacemarkKey( const QString& fileName )
 {
-    d->m_placemarkmanager->removePlacemarkKey( key );
+    d->m_fileManager->removeFile( fileName );
 
     d->notifyModelChanged();
 }
@@ -725,22 +775,51 @@ void MarbleModelPrivate::notifyModelChanged()
     emit m_parent->modelChanged();
 }
 
-void MarbleModelPrivate::geoDataDocumentAdded( const GeoDataDocument& document )
+void MarbleModelPrivate::drawFog( QPainter *painter,
+                                  ViewParams *viewParams,
+                                  int width,
+                                  int height )
 {
-/*    QVector<GeoDataFeature>::Iterator end = document->end();
-    QVector<GeoDataFeature>::Iterator itr = document->begin();
-    for ( ; itr != end; ++itr ) {
-        // use *itr (or itr.value()) here
-        QString styleUrl = itr->styleUrl().remove('#');
-        itr->setStyle( &document->style( styleUrl ) );
-    }
+    if ( viewParams->projection() != Spherical)
+        return;
 
-    m_geometrymodel->setGeoDataRoot( document );*/
+    // No use to draw the fog if it's not visible in the area.
+    if ( viewParams->viewport()->mapCoversViewport() )
+        return;
+
+    int imgWidth2  = width / 2;
+    int imgHeight2 = height / 2;
+
+    int radius = viewParams->radius();
+
+    // Recalculate the atmosphere effect and paint it to canvasImage.
+    QRadialGradient grad1( QPointF( imgWidth2, imgHeight2 ), radius );
+
+    // FIXME: Add a cosine relationship
+    grad1.setColorAt( 0.85, QColor( 255, 255, 255, 0 ) );
+    grad1.setColorAt( 1.00, QColor( 255, 255, 255, 64 ) );
+
+    QBrush    brush1( grad1 );
+    QPen      pen1( Qt::NoPen );
+
+    painter->save();
+
+    painter->setBrush( brush1 );
+    painter->setPen( pen1 );
+    painter->setRenderHint( QPainter::Antialiasing, false );
+
+    // FIXME: Cut out what's really needed
+    painter->drawEllipse( imgWidth2  - radius,
+                         imgHeight2 - radius,
+                         2 * radius,
+                         2 * radius );
+
+    painter->restore();
 }
 
 void MarbleModel::update()
 {
-    qDebug() << "MarbleModel::update()";
+    mDebug() << "MarbleModel::update()";
     QTimer::singleShot( 0, d->m_tileLoader, SLOT( update() ) );
 }
 
@@ -756,8 +835,8 @@ QString MarbleModel::planetName()   const
 
 ExtDateTime* MarbleModel::dateTime() const
 {
-//    qDebug() << "In dateTime, model:" << this;
-//    qDebug() << d << ":" << d->m_dateTime;
+//    mDebug() << "In dateTime, model:" << this;
+//    mDebug() << d << ":" << d->m_dateTime;
     return d->m_dateTime;
 }
 
@@ -774,7 +853,7 @@ MergedLayerDecorator* MarbleModel::layerDecorator() const
 void MarbleModel::clearVolatileTileCache()
 {
     d->m_tileLoader->update();
-    qDebug() << "Cleared Volatile Cache!";
+    mDebug() << "Cleared Volatile Cache!";
 }
 
 quint64 MarbleModel::volatileTileCacheLimit() const
@@ -809,10 +888,10 @@ void MarbleModel::clearPersistentTileCache()
         QString installMap = texture->installMap();
         QString role = d->m_mapTheme->map()->layer( themeID )->role();
 
-        if ( !TileLoader::baseTilesAvailable( layer )
+        if ( !StackedTileLoader::baseTilesAvailable( layer )
             && !installMap.isEmpty() )
         {
-            qDebug() << "Base tiles not available. Creating Tiles ... \n"
+            mDebug() << "Base tiles not available. Creating Tiles ... \n"
                      << "SourceDir: " << sourceDir << "InstallMap:" << installMap;
             MarbleDirs::debug();
 
@@ -821,31 +900,30 @@ void MarbleModel::clearPersistentTileCache()
                                      installMap,
                                      (role == "dem") ? "true" : "false" );
 
-            TileCreatorDialog  tileCreatorDlg( tileCreator, 0 );
-            tileCreatorDlg.setSummary( d->m_mapTheme->head()->name(),
-                       d->m_mapTheme->head()->description() );
-            tileCreatorDlg.exec();
+            QPointer<TileCreatorDialog> tileCreatorDlg = new TileCreatorDialog( tileCreator, 0 );
+            tileCreatorDlg->setSummary( d->m_mapTheme->head()->name(),
+                                        d->m_mapTheme->head()->description() );
+            tileCreatorDlg->exec();
             qDebug("Tile creation completed");
+            delete tileCreatorDlg;
         }
     }
 }
 
-void MarbleModel::paintTile( TextureTile* tile, int x, int y, int level,
-                             GeoSceneTexture *textureLayer, bool requestTileUpdate )
+void MarbleModel::paintTile( StackedTile* tile, GeoSceneTexture *textureLayer )
 {
-//    qDebug() << "MarbleModel::paintTile: " << "x: " << x << "y:" << y << "level: " << level
+//    mDebug() << "MarbleModel::paintTile: " << "x: " << x << "y:" << y << "level: " << level
 //             << "requestTileUpdate" << requestTileUpdate;
-    
-    if ( d->m_downloadManager != 0 ) {
-        connect( d->m_layerDecorator, SIGNAL( downloadTile( QUrl, QString, QString ) ),
-                 d->m_downloadManager, SLOT( addJob( QUrl, QString, QString ) ) );
-    }
 
-    d->m_layerDecorator->setInfo( x, y, level, tile->id() );
-    d->m_layerDecorator->setTile( tile->tile() );
+    d->m_layerDecorator->setInfo( tile->id() );
+    d->m_layerDecorator->setTile( tile->resultTile() );
 
     d->m_layerDecorator->paint( "maps/" + textureLayer->sourceDir(), mapTheme() );
-    tile->initJumpTables( requestTileUpdate );
+}
+
+PluginManager* MarbleModel::pluginManager() const
+{
+    return d->m_pluginManager;
 }
 
 QList<RenderPlugin *> MarbleModel::renderPlugins() const
@@ -868,6 +946,16 @@ QList<AbstractDataPluginItem *> MarbleModel::whichItemAt( const QPoint& curpos )
     return d->m_layerManager->whichItemAt( curpos );
 }
 
+void MarbleModel::addLayer( LayerInterface *layer )
+{
+    d->m_layerManager->addLayer(layer);
+}
+
+void MarbleModel::removeLayer( LayerInterface *layer )
+{
+    d->m_layerManager->removeLayer(layer);
+}
+
 Planet* MarbleModel::planet() const
 {
     return d->m_planet;
@@ -881,6 +969,94 @@ int MarbleModel::tileZoomLevel() const
     return d->m_texmapper->tileZoomLevel();
 }
 
+void MarbleModel::reloadMap() const
+{
+    Q_ASSERT( d->m_tileLoader );
+    QList<TileId> displayed = d->m_tileLoader->tilesOnDisplay();
+    QList<TileId>::const_iterator pos = displayed.constBegin();
+    QList<TileId>::const_iterator const end = displayed.constEnd();
+    for (; pos != end; ++pos ) {
+        // it's debatable here, whether DownloadBulk or DownloadBrowse should be used
+        // but since "reload" or "refresh" seems to be a common action of a browser and it
+        // allows for more connections (in our model), use "DownloadBrowse"
+        d->m_tileLoader->reloadTile( *pos, DownloadBrowse );
+    }
+}
+
+void MarbleModel::downloadRegion( QString const & mapThemeId,
+                                  TileCoordsPyramid const & pyramid ) const
+{
+    Q_ASSERT( d->m_tileLoader );
+    QTime t;
+    t.start();
+    // When downloading a region (the author of these lines thinks) most users probably expect
+    // the download to begin with the low resolution tiles and then procede level-wise to
+    // higher resolution tiles. In order to achieve this, we start requesting downloads of
+    // high resolution tiles and request the low resolution tiles at the end because
+    // DownloadQueueSet (silly name) is implemented as stack.
+    for ( int level = pyramid.bottomLevel(); level >= pyramid.topLevel(); --level ) {
+        QRect const coords = pyramid.coords( level );
+        mDebug() << "MarbleModel::downloadRegion level:" << level << "tile coords:" << coords;
+        int x1, y1, x2, y2;
+        coords.getCoords( &x1, &y1, &x2, &y2 );
+        for ( int x = x1; x <= x2; ++x )
+            for ( int y = y1; y <= y2; ++y ) {
+                TileId const tileId( mapThemeId, level, x, y );
+                // FIXME: use lazy evaluation to not generate up to 100k tiles in one go
+                // this can take considerable time even on very fast systems
+                // in contrast generating the TileIds on the fly when they are needed
+                // does not seem to affect download speed.
+                d->m_tileLoader->downloadTile( tileId );
+            }
+    }
+    int const elapsedMs = t.elapsed();
+    mDebug() << "MarbleModel::downloadRegion:" << pyramid.tilesCount() << "tiles, " << elapsedMs << "ms";
+}
+
+void MarbleModel::addDownloadPolicies( GeoSceneDocument *mapTheme )
+{
+    if ( !mapTheme )
+        return;
+    if ( !mapTheme->map()->hasTextureLayers() )
+        return;
+
+    // As long as we don't have an Layer Management Class we just lookup 
+    // the name of the layer that has the same name as the theme ID
+    const QString themeId = mapTheme->head()->theme();
+    GeoSceneLayer * const layer = static_cast<GeoSceneLayer*>( mapTheme->map()->layer( themeId ));
+    if ( !layer )
+        return;
+
+    GeoSceneTexture * const texture = static_cast<GeoSceneTexture*>( layer->groundDataset() );
+    if ( !texture )
+        return;
+
+    QList<DownloadPolicy *> policies = texture->downloadPolicies();
+    QList<DownloadPolicy *>::const_iterator pos = policies.constBegin();
+    QList<DownloadPolicy *>::const_iterator const end = policies.constEnd();
+    for (; pos != end; ++pos ) {
+        d->m_downloadManager->addDownloadPolicy( **pos );
+    }
+}
+
+// this method will only temporarily "pollute" the MarbleModel class
+GeoSceneTexture * MarbleModel::textureLayer() const
+{
+    if ( !d->m_mapTheme )
+        return 0;
+    if ( !d->m_mapTheme->map()->hasTextureLayers() )
+        return 0;
+
+    // As long as we don't have an Layer Management Class we just lookup
+    // the name of the layer that has the same name as the theme Id
+    const QString themeId = d->m_mapTheme->head()->theme();
+    GeoSceneLayer * const layer = static_cast<GeoSceneLayer*>( d->m_mapTheme->map()->layer( themeId ));
+    if ( !layer )
+        return 0;
+
+    return static_cast<GeoSceneTexture*>( layer->groundDataset() );
+}
+
 }
 
 #include "MarbleModel.moc"
