@@ -5,17 +5,16 @@
 // find a copy of this license in LICENSE.txt in the top directory of
 // the source code.
 //
-// Copyright 2006-2007 Torsten Rahn <tackat@kde.org>"
-// Copyright 2007      Inge Wallin  <ingwa@kde.org>"
+// Copyright 2006-2007 Torsten Rahn <tackat@kde.org>
+// Copyright 2007      Inge Wallin  <ingwa@kde.org>
 //
 
 //
 // MarbleThemeSelectView lets the user choose a map theme
 //
 
-
-#ifndef MARBLETHEMESELECTVIEW_H
-#define MARBLETHEMESELECTVIEW_H
+#ifndef MARBLE_MARBLETHEMESELECTVIEW_H
+#define MARBLE_MARBLETHEMESELECTVIEW_H
 
 
 #include <QtCore/QModelIndex>
@@ -54,4 +53,4 @@ class MARBLE_EXPORT MarbleThemeSelectView : public QListView
 
 }
 
-#endif // KATLASTHEMESELECTVIEW_H
+#endif
