@@ -13,19 +13,15 @@
 
 #include <cmath>
 
-#include <QtCore/QCoreApplication>
+#include <QtCore/QAbstractItemModel>
 #include <QtCore/QHash>
 #include <QtCore/QSettings>
 #include <QtCore/QTime>
 #include <QtCore/QTimer>
-#include <QtCore/QAbstractItemModel>
 #include <QtGui/QItemSelectionModel>
-#include <QtGui/QSizePolicy>
 #include <QtGui/QPaintEvent>
 #include <QtGui/QRegion>
-#include <QtGui/QStyleOptionGraphicsItem>
-#include <QtGui/QMainWindow>
-#include <QtGui/QAction>
+#include <QtGui/QSizePolicy>
 #include <QtNetwork/QNetworkProxy>
 
 #ifdef MARBLE_DBUS
@@ -33,33 +29,33 @@
 #endif
 
 #include "AbstractProjection.h"
-#include "MarbleDirs.h"
-#include "MarbleDebug.h"
-#include "MarbleMap.h"
-#include "MarbleModel.h"
-#include "Quaternion.h"
-#include "ViewParams.h"
-#include "GeoPainter.h"
+#include "DataMigration.h"
 #include "FileViewModel.h"
-#include "GeoDataCoordinates.h"
+#include "GeoDataLatLonAltBox.h"
+#include "GeoPainter.h"
+#include "MarbleDebug.h"
+#include "MarbleDirs.h"
+#include "MarbleMap.h"
+#include "MarbleMap_p.h" // FIXME: remove this
+#include "MarbleModel.h"
 #include "MarblePhysics.h"
 #include "MarblePlacemarkModel.h"
 #include "MarbleWidgetInputHandler.h"
-#include "TileCreatorDialog.h"
-#include "gps/GpsLayer.h"
+#include "MergedLayerDecorator.h"
 #include "RenderPlugin.h"
 #include "SunLocator.h"
-#include "MergedLayerDecorator.h"
+#include "TileCreatorDialog.h"
+#include "ViewParams.h"
 #include "ViewportParams.h"
+#include "gps/GpsLayer.h"
 
-#include "MarbleMap_p.h"
 
 namespace Marble
 {
 
 #ifdef Q_CC_MSVC
 # ifndef KDEWIN_MATH_H
-   static long double sqrt(int a) { return sqrt((long double)a); }
+   static long double sqrt( int a ) { return sqrt( (long double)a ); }
 # endif
 #endif
 
@@ -73,9 +69,9 @@ class MarbleWidgetPrivate
         : m_widget( parent ),
           m_map( map ),
           m_model( map->model() ),
-          m_viewContext( Marble::Still ),
-          m_stillQuality( Marble::HighQuality ),
-          m_animationQuality( Marble::LowQuality ),
+          m_viewContext( Still ),
+          m_stillQuality( HighQuality ),
+          m_animationQuality( LowQuality ),
           m_animationsEnabled( false ),
           m_inputhandler( 0 ),
           m_physics( new MarblePhysics( parent ) ),
@@ -94,6 +90,22 @@ class MarbleWidgetPrivate
 
     void  construct();
 
+    /**
+      * @brief Rotate the globe in the given direction in discrete steps
+      * @param stepsRight Number of steps to go right. Negative values go left.
+      * @param stepsDown Number of steps to go down. Negative values go up.
+      * @param mode Interpolation mode to use when traveling to the target
+      */
+    void moveByStep( MarbleWidget* widget, int stepsRight, int stepsDown, FlyToMode mode );
+
+    /**
+      * @brief Update widget flags and cause a full repaint
+      *
+      * The background of the widget only needs to be redrawn in certain cases. This
+      * method sets the widget flags accordingly and triggers a repaint.
+      */
+    void repaint();
+
     MarbleWidget    *m_widget;
     // The model we are showing.
     MarbleMap       *m_map;
@@ -154,9 +166,15 @@ MarbleWidget::~MarbleWidget()
 
 void MarbleWidgetPrivate::construct()
 {
+    QPointer<DataMigration> dataMigration = new DataMigration( m_widget );
+    dataMigration->exec();
+    delete dataMigration;
+
 #ifdef MARBLE_DBUS
-    QDBusConnection::sessionBus().registerObject("/MarbleWidget", m_widget, 
-                    QDBusConnection::ExportAllSlots | QDBusConnection::ExportAllSignals | QDBusConnection::ExportAllProperties);
+    QDBusConnection::sessionBus().registerObject( "/MarbleWidget", m_widget,
+                                                  QDBusConnection::ExportAllSlots
+                                                  | QDBusConnection::ExportAllSignals
+                                                  | QDBusConnection::ExportAllProperties );
 #endif
 
     // Widget settings
@@ -193,6 +211,10 @@ void MarbleWidgetPrivate::construct()
     // this in the view, i.e. here.
     m_widget->connect( m_map,    SIGNAL( zoomChanged( int ) ),
                        m_widget, SIGNAL( zoomChanged( int ) ) );
+    m_widget->connect( m_map,    SIGNAL( distanceChanged( QString ) ),
+                       m_widget, SIGNAL( distanceChanged( QString ) ) );
+    m_widget->connect( m_map,    SIGNAL( visibleLatLonAltBoxChanged( GeoDataLatLonAltBox )),
+                       m_widget, SIGNAL( visibleLatLonAltBoxChanged( GeoDataLatLonAltBox )));
 
     // Set background: black.
     m_widget->setPalette( QPalette ( Qt::black ) );
@@ -213,14 +235,17 @@ void MarbleWidgetPrivate::construct()
 
     m_logZoom  = 0;
 
-    m_widget->connect( m_model->sunLocator(), SIGNAL( enableWidgetInput(bool) ),
+    m_widget->connect( m_model->sunLocator(), SIGNAL( enableWidgetInput( bool ) ),
                        m_widget, SLOT( setInputEnabled( bool ) ) );
 
     m_widget->connect( m_model->sunLocator(), SIGNAL( updateStars() ),
                        m_widget, SLOT( update() ) );
 
-    m_widget->connect( m_physics, SIGNAL( valueChanged( qreal ) ),
-                       m_widget, SLOT( updateAnimation( qreal ) ) );
+    m_widget->connect( m_physics, SIGNAL( positionReached( GeoDataLookAt ) ),
+                       m_widget, SLOT( updateAnimation( GeoDataLookAt ) ) );
+
+    m_widget->connect( m_physics, SIGNAL( finished() ),
+                       m_widget, SLOT( startStillMode() ) );
 
     m_widget->connect( m_model->sunLocator(), SIGNAL( centerSun() ),
                        m_widget, SLOT( centerSun() ) );
@@ -235,6 +260,24 @@ void MarbleWidgetPrivate::construct()
                        m_widget, SIGNAL( renderPluginInitialized( RenderPlugin * ) ) );
 }
 
+void MarbleWidgetPrivate::moveByStep( MarbleWidget* widget, int stepsRight, int stepsDown, FlyToMode mode )
+{
+    int polarity = m_map->viewParams()->viewport()->polarity();
+    qreal left = polarity * stepsRight * m_map->moveStep();
+    qreal down = stepsDown * m_map->moveStep();
+    widget->rotateBy( left, down, mode );
+}
+
+void MarbleWidgetPrivate::repaint()
+{
+    // We only have to repaint the background every time if the earth
+    // doesn't cover the whole image.
+    m_widget->setAttribute( Qt::WA_NoSystemBackground,
+                  m_map->mapCoversViewport() && !m_model->mapThemeId().isEmpty() );
+
+    m_widget->repaint();
+}
+
 // ----------------------------------------------------------------
 
 
@@ -249,7 +292,7 @@ MarbleModel *MarbleWidget::model() const
 }
 
 
-void MarbleWidget::setInputHandler(MarbleWidgetInputHandler *handler)
+void MarbleWidget::setInputHandler( MarbleWidgetInputHandler *handler )
 {
     delete d->m_inputhandler;
     d->m_inputhandler = handler;
@@ -258,7 +301,7 @@ void MarbleWidget::setInputHandler(MarbleWidgetInputHandler *handler)
         d->m_inputhandler->init( this );
         installEventFilter( d->m_inputhandler );
 
-        connect( d->m_inputhandler, SIGNAL( mouseClickScreenPosition( int, int) ),
+        connect( d->m_inputhandler, SIGNAL( mouseClickScreenPosition( int, int ) ),
                  this,              SLOT( notifyMouseClick( int, int ) ) );
 
         connect( d->m_inputhandler, SIGNAL( mouseMoveGeoPosition( QString ) ),
@@ -282,22 +325,14 @@ int MarbleWidget::radius() const
     return d->m_map->radius();
 }
 
-void MarbleWidget::setRadius(int radius)
+void MarbleWidget::setRadius( int radius )
 {
     if ( radius == d->m_map->radius() ) {
         return;
     }
         
     d->m_map->setRadius( radius );
-
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    emit distanceChanged( distanceString() );
-
-    repaint();
+    d->repaint();
 }
 
 
@@ -469,168 +504,141 @@ quint64 MarbleWidget::volatileTileCacheLimit() const
 }
 
 
-void MarbleWidget::zoomView(int newZoom)
+void MarbleWidget::zoomView( int newZoom, FlyToMode mode )
 {
-    // This function is tricky since it needs to be possible to call
-    // both from above as an ordinary function, and "from below",
-    // i.e. as a slot.  That's why we need to save m_logZoom from when
-    // we repainted last time.
+    if ( mode == Instant || !d->m_animationsEnabled ) {
+        // This function is tricky since it needs to be possible to call
+        // both from above as an ordinary function, and "from below",
+        // i.e. as a slot.  That's why we need to save m_logZoom from when
+        // we repainted last time.
 
-    // Make all the internal changes to the map.
-    d->m_map->zoomView( newZoom );
+        // Make all the internal changes to the map.
+        d->m_map->zoomView( newZoom );
 
-    // If no change, we don't need to repainting or anything else.
-    if ( d->m_logZoom == newZoom )
-	return;
+        // If no change, we don't need to repainting or anything else.
+        if ( d->m_logZoom == newZoom )
+            return;
 
-    d->m_logZoom = newZoom;
-
-    // We only have to repaint the background every time if the globe
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    emit distanceChanged( distanceString() );
-
-    repaint();
+        d->m_logZoom = newZoom;
+        d->repaint();
+    }
+    else {
+        GeoDataLookAt target = d->m_map->lookAt();
+        int radius = d->m_map->d->radius( newZoom );
+        target.setRange( 1000 * d->m_map->distanceFromRadius( radius ) );
+        flyTo( target, mode == Automatic ? Instant : mode );
+    }
 }
 
 
-void MarbleWidget::zoomViewBy( int zoomStep )
+void MarbleWidget::zoomViewBy( int zoomStep, FlyToMode mode )
 {
-    zoomView( MarbleMapPrivate::toLogScale( radius() ) + zoomStep );
+    zoomView( zoom() + zoomStep, mode );
 }
 
 
-void MarbleWidget::zoomIn()
+void MarbleWidget::zoomIn( FlyToMode mode )
 {
-    d->m_map->zoomIn();
+    if ( mode == Instant || !d->m_animationsEnabled ) {
+        d->m_map->zoomIn();
+        d->repaint();
+    }
+    else {
+        GeoDataLookAt target = d->m_map->lookAt();
+        MarbleMap *map = d->m_map;
+        int newRadius = map->d->radius( map->zoom() + map->d->m_zoomStep );
+        target.setRange( 1000 * d->m_map->distanceFromRadius( newRadius ) );
+        flyTo( target, mode );
+    }
 }
 
-void MarbleWidget::zoomOut()
+void MarbleWidget::zoomOut( FlyToMode mode )
 {
-    d->m_map->zoomOut();
+    if ( mode == Instant || !d->m_animationsEnabled ) {
+        d->m_map->zoomOut();
+        d->repaint();
+    }
+    else {
+        GeoDataLookAt target = d->m_map->lookAt();
+        MarbleMap *map = d->m_map;
+        int newRadius = map->d->radius( map->zoom() - map->d->m_zoomStep );
+        target.setRange( 1000 * d->m_map->distanceFromRadius( newRadius ) );
+        flyTo( target, mode );
+    }
 }
 
-void MarbleWidget::rotateBy(const Quaternion& incRot)
+void MarbleWidget::rotateBy( const Quaternion& incRot )
 {
     d->m_map->rotateBy( incRot );
-
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    repaint();
+    d->repaint();
 }
 
-void MarbleWidget::rotateBy( const qreal& deltaLon, const qreal& deltaLat)
+void MarbleWidget::rotateBy( const qreal deltaLon, const qreal deltaLat, FlyToMode mode )
 {
-    d->m_map->rotateBy( deltaLon, deltaLat );
+    Quaternion  rotPhi( 1.0, deltaLat / 180.0, 0.0, 0.0 );
+    Quaternion  rotTheta( 1.0, 0.0, deltaLon / 180.0, 0.0 );
 
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    repaint();
+    Quaternion  axis = d->m_map->viewParams()->planetAxis();
+    qreal lon( 0.0 ), lat( 0.0 );
+    axis.getSpherical( lon, lat );
+    axis = rotTheta * axis;
+    axis *= rotPhi;
+    axis.normalize();
+    lat = -axis.pitch();
+    lon = axis.yaw();
+    
+    GeoDataLookAt target = d->m_map->lookAt();
+    target.setLongitude( lon );
+    target.setLatitude( lat );
+    flyTo( target, mode );
 }
 
 
-void MarbleWidget::centerOn( const qreal& lon, const qreal& lat, bool animated )
+void MarbleWidget::centerOn( const qreal lon, const qreal lat, bool animated )
 {
-    if ( d->m_animationsEnabled && animated ) {
-        d->m_physics->setCurrentPosition( GeoDataCoordinates( centerLongitude(), centerLatitude(),
-                                                              distance(),
-                                                              GeoDataCoordinates::Degree ) );
-        d->m_physics->jumpTo( GeoDataCoordinates( lon, lat, distance(),
-                                                  GeoDataCoordinates::Degree ) );
-    } else {
-        d->m_map->centerOn( lon, lat );
-    }
-
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    repaint();
+    GeoDataCoordinates target( lon, lat, 0.0, GeoDataCoordinates::Degree );
+    centerOn( target, animated );
 }
 
 void MarbleWidget::centerOn( const QModelIndex& index, bool animated )
 {
-    if ( d->m_animationsEnabled && animated ) {
-        QItemSelectionModel *selectionModel = d->m_map->model()->placemarkSelectionModel();
-        Q_ASSERT( selectionModel );
-    
-        selectionModel->clear();
-    
-        if ( index.isValid() ) {
-            const GeoDataCoordinates targetPosition =
-                index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
+    QItemSelectionModel *selectionModel = d->m_map->model()->placemarkSelectionModel();
+    Q_ASSERT( selectionModel );
 
-            d->m_physics->setCurrentPosition( GeoDataCoordinates( centerLongitude(),
-                                                                  centerLatitude(), distance(),
-                                                                  GeoDataCoordinates::Degree ) );
-            d->m_physics->jumpTo( targetPosition );
+    selectionModel->clear();
 
-            selectionModel->select( index, QItemSelectionModel::SelectCurrent );
-        }
-    } else {
-        d->m_map->centerOn( index );
+    if ( index.isValid() ) {
+        const GeoDataCoordinates targetPosition =
+            index.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
+
+        GeoDataLookAt target = d->m_map->lookAt();
+        target.setLongitude( targetPosition.longitude() );
+        target.setLatitude( targetPosition.latitude() );
+        flyTo( target, animated ? Automatic : Instant );
+
+        selectionModel->select( index, QItemSelectionModel::SelectCurrent );
     }
-
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    repaint();
 }
 
 void MarbleWidget::centerOn( const GeoDataCoordinates &position, bool animated )
 {
-    if ( d->m_animationsEnabled && animated ) {
-        GeoDataCoordinates targetPosition = position;
-        targetPosition.setAltitude( distance() );
-
-        // Avoid zero distance
-        qreal minDistance = 0.001;
-        
-        if ( targetPosition.altitude() <= minDistance ) {
-            targetPosition.setAltitude( minDistance );
-        }
-        
-        d->m_physics->jumpTo( targetPosition );
-    } else {
-        qreal  lon, lat;
-        position.geoCoordinates( lon, lat, GeoDataCoordinates::Degree );
-        d->m_map->setDistance( position.altitude() );
-        d->m_map->centerOn( lon, lat );
-    }
-
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    repaint();
+    GeoDataLookAt target = d->m_map->lookAt();
+    target.setLongitude( position.longitude() );
+    target.setLatitude( position.latitude() );
+    flyTo( target, animated ? Automatic : Instant );
 }
 
 void MarbleWidget::centerOn( const GeoDataLatLonBox &box, bool animated )
 {
-    Q_UNUSED(animated);
-    //work out the needed zoom level
+    Q_UNUSED( animated );
+
     ViewportParams* viewparams = d->m_map->viewParams()->viewport();
-    int maxScreenSize = viewparams->height() > viewparams->width() ?
-                     viewparams->height() : viewparams->width();
-
-    qreal maxBoundingSize = box.height() > box.width() ? box.height() : box.width();
-
     //prevent divide by zero
-    if( maxBoundingSize ) {
-        int zoomRadius = ( 0.25 * M_PI ) * ( maxScreenSize / maxBoundingSize );
-        setRadius( zoomRadius );
+    if( box.height() && box.width() ) {
+        //work out the needed zoom level
+        int horizontalRadius = ( 0.25 * M_PI ) * ( viewparams->height() / box.height() );
+        int verticalRadius = ( 0.25 * M_PI ) * ( viewparams->width() / box.width() );
+        setRadius( qMin<int>( horizontalRadius, verticalRadius ) );
     }
 
     //move the map
@@ -640,38 +648,14 @@ void MarbleWidget::centerOn( const GeoDataLatLonBox &box, bool animated )
     repaint();
 }
 
-void MarbleWidget::updateAnimation( qreal updateValue )
+void MarbleWidget::setCenterLatitude( qreal lat, FlyToMode mode )
 {
-    GeoDataCoordinates position = d->m_physics->suggestedPosition();
-
-    if ( updateValue < 1.0 ) {
-        setViewContext( Marble::Animation );
-        centerOn( position );
-        setViewContext( Marble::Still );
-        return;
-    }
-
-    centerOn( position );
+    centerOn( centerLongitude(), lat, mode );
 }
 
-void MarbleWidget::setCenterLatitude( qreal lat )
+void MarbleWidget::setCenterLongitude( qreal lon, FlyToMode mode )
 {
-    centerOn( centerLongitude(), lat );
-
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-}
-
-void MarbleWidget::setCenterLongitude( qreal lon )
-{
-    centerOn( lon, centerLatitude() );
-
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+    centerOn( lon, centerLatitude(), mode );
 }
 
 Projection MarbleWidget::projection() const
@@ -682,11 +666,7 @@ Projection MarbleWidget::projection() const
 void MarbleWidget::setProjection( Projection projection )
 {
     d->m_map->setProjection( projection );
-
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    repaint();
+    d->repaint();
 }
 
 void MarbleWidget::setProjection( int projection )
@@ -704,79 +684,47 @@ void MarbleWidget::setHome( qreal lon, qreal lat, int zoom )
     d->m_map->setHome( lon, lat, zoom );
 }
 
-void MarbleWidget::setHome(const GeoDataCoordinates& homePoint, int zoom)
+void MarbleWidget::setHome( const GeoDataCoordinates& homePoint, int zoom )
 {
     d->m_map->setHome( homePoint, zoom );
 }
 
 
-void MarbleWidget::moveLeft()
+void MarbleWidget::moveLeft( FlyToMode mode )
 {
-    d->m_map->moveLeft();
-
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    repaint();
+    d->moveByStep( this, -1, 0, mode );
 }
 
-void MarbleWidget::moveRight()
+void MarbleWidget::moveRight( FlyToMode mode )
 {
-    d->m_map->moveRight();
-
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    repaint();
+    d->moveByStep( this, 1, 0, mode );
 }
 
 
-void MarbleWidget::moveUp()
+void MarbleWidget::moveUp( FlyToMode mode )
 {
-    d->m_map->moveUp();
-
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    repaint();
+    d->moveByStep( this, 0, -1, mode );
 }
 
-void MarbleWidget::moveDown()
+void MarbleWidget::moveDown( FlyToMode mode )
 {
-    d->m_map->moveDown();
-
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    repaint();
+    d->moveByStep( this, 0, 1, mode );
 }
 
-void MarbleWidget::leaveEvent (QEvent*)
+void MarbleWidget::leaveEvent( QEvent* )
 {
     emit mouseMoveGeoPosition( tr( NOT_AVAILABLE ) );
 }
 
-void MarbleWidget::resizeEvent (QResizeEvent*)
+void MarbleWidget::resizeEvent( QResizeEvent* )
 {
     setUpdatesEnabled( false );
     d->m_map->setSize( width(), height() );
-
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
-
-    repaint();
+    d->repaint();
     setUpdatesEnabled( true );
 }
 
-void MarbleWidget::connectNotify ( const char * signal )
+void MarbleWidget::connectNotify( const char * signal )
 {
     if ( QByteArray( signal ) == 
          QMetaObject::normalizedSignature ( SIGNAL( mouseMoveGeoPosition( QString ) ) ) )
@@ -784,7 +732,7 @@ void MarbleWidget::connectNotify ( const char * signal )
             d->m_inputhandler->setPositionSignalConnected( true );
 }
 
-void MarbleWidget::disconnectNotify ( const char * signal )
+void MarbleWidget::disconnectNotify( const char * signal )
 {
     if ( QByteArray( signal ) == 
          QMetaObject::normalizedSignature ( SIGNAL( mouseMoveGeoPosition( QString ) ) ) )
@@ -803,9 +751,9 @@ bool MarbleWidget::screenCoordinates( qreal lon, qreal lat,
     return d->m_map->screenCoordinates( lon, lat, x, y );
 }
 
-bool MarbleWidget::geoCoordinates(int x, int y,
-                                  qreal& lon, qreal& lat,
-                                  GeoDataCoordinates::Unit unit )
+bool MarbleWidget::geoCoordinates( int x, int y,
+                                   qreal& lon, qreal& lat,
+                                   GeoDataCoordinates::Unit unit )
 {
     return d->m_map->geoCoordinates( x, y, lon, lat, unit );
 }
@@ -820,17 +768,17 @@ qreal MarbleWidget::centerLongitude() const
     return d->m_map->centerLongitude();
 }
 
-const QRegion MarbleWidget::activeRegion()
+QRegion MarbleWidget::activeRegion()
 {
     return d->m_map->viewParams()->viewport()->activeRegion();
 }
 
-const QRegion MarbleWidget::mapRegion()
+QRegion MarbleWidget::mapRegion()
 {
     return d->m_map->viewParams()->currentProjection()->mapRegion( d->m_map->viewParams()->viewport() );
 }
 
-void MarbleWidget::paintEvent(QPaintEvent *evt)
+void MarbleWidget::paintEvent( QPaintEvent *evt )
 {
     // Stop repaint timer if it is already running
     d->m_repaintTimer.stop();
@@ -853,7 +801,7 @@ void MarbleWidget::paintEvent(QPaintEvent *evt)
                                      ? QImage::Format_RGB32
                                      : QImage::Format_ARGB32_Premultiplied;
         // Paint to an intermediate image
-        image = QImage(rect().size(), imageFormat);
+        image = QImage( rect().size(), imageFormat );
         paintDevice = &image;
     }
 
@@ -869,21 +817,21 @@ void MarbleWidget::paintEvent(QPaintEvent *evt)
     customPaint( &painter );
     d->m_map->d->paintOverlay( painter, dirtyRect );
 
-    if (!isEnabled())
+    if ( !isEnabled() )
     {
         // Draw a grayscale version of the intermediate image
-        QRgb* pixel = reinterpret_cast<QRgb*>(image.scanLine(0));
+        QRgb* pixel = reinterpret_cast<QRgb*>( image.scanLine( 0 ));
         for (int i=0; i<image.width()*image.height(); ++i, ++pixel) {
-            int gray = qGray(*pixel);
-            *pixel = qRgb(gray,gray,gray);
+            int gray = qGray( *pixel );
+            *pixel = qRgb( gray, gray, gray );
         }
 
         GeoPainter widgetPainter( this, map()->viewParams()->viewport(),
                             map()->viewParams()->mapQuality(), doClip );
-        widgetPainter.drawImage(rect(), image);
+        widgetPainter.drawImage( rect(), image );
     }
 
-    if (d->m_map->showFrameRate())
+    if ( d->m_map->showFrameRate() )
     {
         qreal fps = 1000.0 / (qreal)( t.elapsed() + 1 );
         d->m_map->d->paintFps( painter, dirtyRect, fps );
@@ -891,23 +839,36 @@ void MarbleWidget::paintEvent(QPaintEvent *evt)
     }
 }
 
-void MarbleWidget::customPaint(GeoPainter *painter)
+void MarbleWidget::customPaint( GeoPainter *painter )
 {
     Q_UNUSED( painter );
     /* This is a NOOP in the base class*/
 }
 
 
-void MarbleWidget::goHome()
+void MarbleWidget::goHome( FlyToMode mode )
 {
-    d->m_map->goHome();
+    if ( !d->m_animationsEnabled || mode == Instant )
+    {
+        d->m_map->goHome();
 
-    // We only have to repaint the background every time if the earth
-    // doesn't cover the whole image.
-    setAttribute( Qt::WA_NoSystemBackground,
-                  d->m_map->mapCoversViewport() && !mapThemeId().isEmpty() );
+         // not obsolete in case the zoomlevel stays unaltered
+        d->repaint();
+    }
+    else {
+        qreal  homeLon = 0;
+        qreal  homeLat = 0;
+        int homeZoom = 0;
+        d->m_map->home( homeLon, homeLat, homeZoom );
 
-    repaint(); // not obsolete in case the zoomlevel stays unaltered
+        GeoDataLookAt target;
+        target.setLongitude( homeLon, GeoDataCoordinates::Degree );
+        target.setLatitude( homeLat, GeoDataCoordinates::Degree );
+        int radius = d->m_map->d->radius( homeZoom );
+        target.setRange( 1000 * d->m_map->distanceFromRadius( radius ) );
+
+        flyTo( target, mode );
+    }
 }
 
 QString MarbleWidget::mapThemeId() const
@@ -1101,7 +1062,7 @@ void MarbleWidget::notifyMouseClick( int x, int y)
     bool const valid = geoCoordinates( x, y, lon, lat, GeoDataCoordinates::Radian );
 
     if ( valid ) {
-        emit mouseClickGeoPosition( lon, lat, GeoDataCoordinates::Radian);
+        emit mouseClickGeoPosition( lon, lat, GeoDataCoordinates::Radian );
     }
 }
 
@@ -1117,9 +1078,9 @@ void MarbleWidget::updateGps()
     }
 }
 
-void MarbleWidget::openGpxFile(QString &filename)
+void MarbleWidget::openGpxFile( const QString &filename )
 {
-    d->m_map->openGpxFile(filename);
+    d->m_map->openGpxFile( filename );
 }
 
 GpxFileModel *MarbleWidget::gpxFileModel()
@@ -1171,9 +1132,9 @@ void MarbleWidget::updateChangedMap()
     update();
 }
 
-void MarbleWidget::scheduleRepaint( QRegion dirtyRegion )
+void MarbleWidget::scheduleRepaint( const QRegion& dirtyRegion )
 {
-    Q_UNUSED(dirtyRegion);
+    Q_UNUSED( dirtyRegion );
     if ( !d->m_repaintTimer.isActive() ) {
         d->m_repaintTimer.start();
     }
@@ -1212,7 +1173,7 @@ ViewContext MarbleWidget::viewContext() const
     return d->m_viewContext;
 }
 
-void MarbleWidget::setViewContext( Marble::ViewContext viewContext )
+void MarbleWidget::setViewContext( ViewContext viewContext )
 {
     d->m_viewContext = viewContext;
 
@@ -1232,12 +1193,12 @@ void MarbleWidget::setAnimationsEnabled( bool enabled )
     d->m_animationsEnabled = enabled;
 }
 
-Marble::AngleUnit MarbleWidget::defaultAngleUnit() const
+AngleUnit MarbleWidget::defaultAngleUnit() const
 {
     return map()->defaultAngleUnit();
 }
 
-void MarbleWidget::setDefaultAngleUnit( Marble::AngleUnit angleUnit )
+void MarbleWidget::setDefaultAngleUnit( AngleUnit angleUnit )
 {
     map()->setDefaultAngleUnit( angleUnit );
 }
@@ -1252,7 +1213,7 @@ void MarbleWidget::setDefaultFont( const QFont& font )
     map()->setDefaultFont( font );
 }
 
-void MarbleWidget::setSelection(const QRect& region)
+void MarbleWidget::setSelection( const QRect& region )
 {
     QPoint tl = region.topLeft();
     QPoint br = region.bottomRight();
@@ -1260,18 +1221,18 @@ void MarbleWidget::setSelection(const QRect& region)
              << br.x() << ", " << br.y() << ")" << endl;
 
     AbstractProjection *proj = d->m_map->viewParams()->currentProjection();
-    GeoDataLatLonAltBox box  = proj->latLonAltBox(region, d->m_map->viewParams()->viewport());
+    GeoDataLatLonAltBox box  = proj->latLonAltBox( region, d->m_map->viewParams()->viewport() );
 
     // NOTE: coordinates as lon1, lat1, lon2, lat2 (or West, North, East, South)
     // as left/top, right/bottom rectangle.
     QList<double> coordinates;
-    coordinates << box.west(GeoDataPoint::Degree) << box.north(GeoDataPoint::Degree)
-                << box.east(GeoDataPoint::Degree) << box.south(GeoDataPoint::Degree);
+    coordinates << box.west( GeoDataPoint::Degree ) << box.north( GeoDataPoint::Degree )
+                << box.east( GeoDataPoint::Degree ) << box.south( GeoDataPoint::Degree );
 
     mDebug() << "West: " << coordinates[0] << " North: " <<  coordinates[1]
              << " East: " << coordinates[2] << " South: " << coordinates[3] << endl;
 
-    emit regionSelected(coordinates);
+    emit regionSelected( coordinates );
 }
 
 qreal MarbleWidget::distance() const
@@ -1341,39 +1302,40 @@ void MarbleWidget::setInputEnabled( bool enabled )
 
 void MarbleWidget::setProxy( const QString& proxyHost, const quint16 proxyPort, const QString& user, const QString& password  )
 {
+    d->m_proxyHost = proxyHost;
+    d->m_proxyPort = proxyPort;
+    d->m_user = user;
+    d->m_password = password;
+
     QNetworkProxy::ProxyType type = QNetworkProxy::HttpProxy;
 
     // Make sure that no proxy is used for an empty string or the default value: 
     if ( proxyHost.isEmpty() || proxyHost == "http://" )
         type = QNetworkProxy::NoProxy;
 
-    QString hostName = proxyHost;
-    if ( hostName.startsWith("http://" ) ) {
-        hostName = hostName.mid( QString( "http://" ).size() );
-    }
-    
-    QNetworkProxy proxy( type, hostName, proxyPort, user, password );
+    QNetworkProxy proxy( type, d->m_proxyHost, d->m_proxyPort, d->m_user, d->m_password );
     QNetworkProxy::setApplicationProxy( proxy );
+    mDebug() << "MarbleWidget::setProxy" << type << d->m_proxyHost << d->m_proxyPort << d->m_user << d->m_password;
 }
 
 QString MarbleWidget::proxyHost() const
 {
-    return QNetworkProxy::applicationProxy().hostName();
+    return d->m_proxyHost;
 }
 
 quint16 MarbleWidget::proxyPort() const
 {
-    return QNetworkProxy::applicationProxy().port();
+    return d->m_proxyPort;
 }
 
 QString MarbleWidget::user() const
 {
-    return QNetworkProxy::applicationProxy().user();
+    return d->m_user;
 }
 
 QString MarbleWidget::password() const
 {
-    return QNetworkProxy::applicationProxy().password();
+    return d->m_password;
 }
 
 QList<RenderPlugin *> MarbleWidget::renderPlugins() const
@@ -1420,9 +1382,9 @@ QList<AbstractFloatItem *> MarbleWidget::floatItems() const
     return d->m_model->floatItems();
 }
 
-void MarbleWidget::changeEvent ( QEvent * event )
+void MarbleWidget::changeEvent( QEvent * event )
 {
-    if (event->type() == QEvent::EnabledChange)
+    if ( event->type() == QEvent::EnabledChange )
     {
         setInputEnabled(isEnabled());
     }
@@ -1430,6 +1392,34 @@ void MarbleWidget::changeEvent ( QEvent * event )
     QWidget::changeEvent(event);
 }
 
+void MarbleWidget::flyTo( const GeoDataLookAt &lookAt, FlyToMode mode )
+{
+    if ( !d->m_animationsEnabled || mode == Instant ) {
+        d->m_map->flyTo( lookAt );
+        d->repaint();
+    }
+    else {
+        GeoDataLookAt source = d->m_map->lookAt();
+        setViewContext( Marble::Animation );
+        ViewportParams *viewport = d->m_map->viewParams()->viewport();
+        d->m_physics->flyTo( source, lookAt, viewport, mode );
+    }
+}
+
+void MarbleWidget::updateAnimation( const GeoDataLookAt &lookAt )
+{
+    setViewContext( Marble::Animation );
+    d->m_map->flyTo( lookAt );
+    d->repaint();
+}
+
+void MarbleWidget::startStillMode()
+{
+    setViewContext( Marble::Still );
+    setNeedsUpdate();
+    d->repaint();
+}
+
 }
 
 #include "MarbleWidget.moc"
