@@ -17,6 +17,7 @@
 #include <QtCore/QPointer>
 #include <QtCore/QTime>
 #include <QtCore/QTimer>
+#include <QtGui/QAction>
 #include <QtGui/QCursor>
 #include <QtGui/QMouseEvent>
 #include <QtGui/QPixmap>
@@ -40,6 +41,8 @@
 #include "AbstractDataPluginItem.h"
 #include "MeasureTool.h"
 #include "MarbleWidgetPopupMenu.h"
+#include "AbstractProjection.h"
+#include "Planet.h"
 
 using namespace Marble;
 
@@ -55,6 +58,7 @@ public:
     bool m_positionSignalConnected;
     QTimer *m_mouseWheelTimer;
     Qt::MouseButtons m_disabledMouseButtons;
+    qreal m_wheelZoomTargetDistance;
 };
 
 MarbleWidgetInputHandler::Protected::Protected()
@@ -62,7 +66,8 @@ MarbleWidgetInputHandler::Protected::Protected()
       m_model( 0 ),
       m_positionSignalConnected( false ),
       m_mouseWheelTimer( 0 ),
-      m_disabledMouseButtons( Qt::NoButton )
+      m_disabledMouseButtons( Qt::NoButton ),
+      m_wheelZoomTargetDistance( 0.0 )
 {
 }
 
@@ -127,6 +132,23 @@ class MarbleWidgetDefaultInputHandler::Private
     Private();
     ~Private();
 
+    /**
+      * @brief Change to a different distance, keeping the given point still
+      * @param widget The marble widget to work on
+      * @param pos A screen position. The underlying geopoint should stay at the
+      * same position while zooming
+      * @param distance New camera distance to the ground
+      */
+    void ZoomAt(MarbleWidget* widget, const QPoint &pos, qreal distance);
+
+    /**
+      * @brief Change zoom value by the given factor, making the given point the new center
+      * @param widget The marble widget to work on
+      * @param pos A screen position. The underlying geopoint will become the new center point
+      * @param factor Zoom factor, 1.0 means no change
+      */
+    void MoveTo(MarbleWidget* marbleWidget, const QPoint &pos, qreal zoomFactor);
+
     QPixmap m_curpmtl;
     QPixmap m_curpmtc;
     QPixmap m_curpmtr;
@@ -139,6 +161,7 @@ class MarbleWidgetDefaultInputHandler::Private
     QCursor m_arrowcur[3][3];
 
     bool m_leftpressed;
+    int m_leftpresseddirection;
     bool m_midpressed;
     int m_leftpressedx;
     int m_leftpressedy;
@@ -147,7 +170,7 @@ class MarbleWidgetDefaultInputHandler::Private
     qreal m_leftpressedb;
 
     int m_dragThreshold;
-    QTime m_dragtimer;
+    QTimer m_lmbTimer;
 
     QPoint m_selectionOrigin;
     QRubberBand *m_selectionRubber;
@@ -157,6 +180,10 @@ class MarbleWidgetDefaultInputHandler::Private
     QPoint m_toolTipPosition;
 
     MarbleWidgetPopupMenu *m_popupmenu;
+
+    QAction  *m_addMeasurePointAction;
+    QAction  *m_removeLastMeasurePointAction;
+    QAction  *m_removeMeasurePointsAction;
 };
 
 MarbleWidgetDefaultInputHandler::Private::Private()
@@ -189,6 +216,73 @@ MarbleWidgetDefaultInputHandler::Private::~Private()
 {
 }
 
+void MarbleWidgetDefaultInputHandler::Private::ZoomAt(MarbleWidget* marbleWidget, const QPoint &pos, qreal newDistance)
+{
+    Q_ASSERT(newDistance > 0.0);
+
+    qreal  destLat;
+    qreal  destLon;
+    if (!marbleWidget->geoCoordinates(pos.x(), pos.y(), 
+          destLon, destLat, GeoDataCoordinates::Radian )) {
+        return;
+    }
+
+    ViewportParams* now = marbleWidget->map()->viewParams()->viewport();
+
+    qreal x(0), y(0);
+    if (!now->currentProjection()->screenCoordinates(destLon, destLat, now, x, y)) {
+        return;
+    }
+
+    ViewportParams soon;
+    soon.setProjection(now->projection());
+    soon.setPlanetAxis(now->planetAxis());
+    soon.setSize(now->size());
+
+    qreal newRadius = marbleWidget->map()->radiusFromDistance(newDistance);
+    soon.setRadius( newRadius );
+
+    qreal mouseLon, mouseLat;
+    if (!soon.currentProjection()->geoCoordinates(int(x), int(y), &soon, mouseLon, mouseLat, GeoDataCoordinates::Radian )) {
+        return;
+    }
+
+    qreal centerLat = DEG2RAD * marbleWidget->centerLatitude();
+    qreal centerLon = DEG2RAD * marbleWidget->centerLongitude();
+
+    qreal lon = destLon - ( mouseLon - centerLon );
+    qreal lat = destLat - ( mouseLat - centerLat );
+
+    GeoDataLookAt lookAt;
+    lookAt.setLongitude( lon );
+    lookAt.setLatitude( lat );
+    lookAt.setAltitude( 0.0 );
+    lookAt.setRange( newDistance * KM2METER );
+    
+    marbleWidget->map()->viewParams()->viewport()->setFocusPoint( GeoDataCoordinates( destLon,
+                                                                                      destLat ) );
+    marbleWidget->flyTo( lookAt, Linear );
+}
+
+void MarbleWidgetDefaultInputHandler::Private::MoveTo( MarbleWidget* marbleWidget,
+                                                       const QPoint &pos,
+                                                       qreal factor )
+{
+    Q_ASSERT( factor > 0.0 );
+
+    qreal  destLat;
+    qreal  destLon;
+    qreal distance = marbleWidget->map()->distance();
+    marbleWidget->geoCoordinates( pos.x(), pos.y(), destLon, destLat, GeoDataCoordinates::Radian );
+    
+    GeoDataLookAt lookAt;
+    lookAt.setLongitude( destLon );
+    lookAt.setLatitude( destLat );
+    lookAt.setAltitude( 0.0 );
+    lookAt.setRange( distance * factor * KM2METER );
+    
+    marbleWidget->flyTo( lookAt );
+}
 
 MarbleWidgetDefaultInputHandler::MarbleWidgetDefaultInputHandler()
     : MarbleWidgetInputHandler(), d( new Private )
@@ -200,6 +294,8 @@ MarbleWidgetDefaultInputHandler::MarbleWidgetDefaultInputHandler()
     d->m_toolTipTimer.setSingleShot( true );
     d->m_toolTipTimer.setInterval( TOOLTIP_START_INTERVAL );
     connect( &d->m_toolTipTimer, SIGNAL( timeout() ), this, SLOT( openItemToolTip() ) );
+    d->m_lmbTimer.setSingleShot(true);
+    connect( &d->m_lmbTimer, SIGNAL(timeout()), this, SLOT(lmbTimeout()));
 }
 
 MarbleWidgetDefaultInputHandler::~MarbleWidgetDefaultInputHandler()
@@ -209,20 +305,28 @@ MarbleWidgetDefaultInputHandler::~MarbleWidgetDefaultInputHandler()
     delete d;
 }
 
+void MarbleWidgetDefaultInputHandler::lmbTimeout()
+{
+    if (!d->m_selectionRubber->isVisible()) {
+        emit lmbRequest( d->m_leftpressedx, d->m_leftpressedy );
+    }
+}
+
 void MarbleWidgetInputHandler::restoreViewContext()
 {
     // Needs to stop the timer since it repeats otherwise.
     d->m_mouseWheelTimer->stop();
 
-    // Redraw the map with the quality set for Marble::Still (if necessary).
-    d->m_widget->setViewContext( Marble::Still );
-    if ( d->m_widget->mapQuality( Marble::Still )
-        != d->m_widget->mapQuality( Marble::Animation ) )
+    // Redraw the map with the quality set for Still (if necessary).
+    d->m_widget->setViewContext( Still );
+    if ( d->m_widget->mapQuality( Still )
+        != d->m_widget->mapQuality( Animation ) )
     {
         d->m_widget->updateChangedMap();
     }
 
     d->m_widget->map()->viewParams()->viewport()->resetFocusPoint();
+    d->m_wheelZoomTargetDistance = 0.0;
 }
 
 void MarbleWidgetInputHandler::installPluginEventFilter( RenderPlugin *renderPlugin )
@@ -243,16 +347,29 @@ void MarbleWidgetDefaultInputHandler::init( MarbleWidget *w )
         d->m_popupmenu = new MarbleWidgetPopupMenu( MarbleWidgetInputHandler::d->m_widget,
                                                     MarbleWidgetInputHandler::d->m_model );
     }
+
+    d->m_addMeasurePointAction = new QAction( tr( "Add &Measure Point" ), this);
+    d->m_popupmenu->addAction( Qt::RightButton, d->m_addMeasurePointAction );
+    d->m_removeLastMeasurePointAction = new QAction( tr( "Remove &Last Measure Point" ),
+                                                  this);
+    d->m_removeLastMeasurePointAction->setEnabled(false);
+    d->m_popupmenu->addAction( Qt::RightButton, d->m_removeLastMeasurePointAction );
+    d->m_removeMeasurePointsAction = new QAction( tr( "&Remove Measure Points" ),
+                                                this);
+    d->m_removeMeasurePointsAction->setEnabled(false);
+    d->m_popupmenu->addAction( Qt::RightButton, d->m_removeMeasurePointsAction );
+
+    connect( d->m_addMeasurePointAction, SIGNAL( triggered() ),
+             SLOT( addMeasurePoint() ) );
+    connect( d->m_removeLastMeasurePointAction, SIGNAL(triggered() ),
+             measureTool, SLOT( removeLastMeasurePoint() ) );
+    connect( d->m_removeMeasurePointsAction, SIGNAL( triggered() ),
+             measureTool, SLOT( removeMeasurePoints() ) );
+
     connect( this, SIGNAL( rmbRequest( int, int ) ),
              this, SLOT( showRmbMenu( int, int ) ) );
-    connect( d->m_popupmenu, SIGNAL( addMeasurePoint( qreal, qreal ) ),
-             measureTool, SLOT( addMeasurePoint( qreal, qreal ) ) );
-    connect( d->m_popupmenu, SIGNAL( removeLastMeasurePoint() ),
-             measureTool, SLOT( removeLastMeasurePoint() ) );
-    connect( d->m_popupmenu, SIGNAL( removeMeasurePoints() ),
-             measureTool, SLOT( removeMeasurePoints( ) ) );
     connect( measureTool, SIGNAL( numberOfMeasurePointsChanged( int ) ),
-             d->m_popupmenu, SLOT( slotNumberOfMeasurePointsChanged( int ) ) );
+             this, SLOT( setNumberOfMeasurePoints( int ) ) );
     connect( this, SIGNAL( lmbRequest( int, int ) ),
              this, SLOT( showLmbMenu( int, int ) ) );
 }
@@ -267,6 +384,7 @@ void MarbleWidgetDefaultInputHandler::showLmbMenu( int x, int y )
 void MarbleWidgetDefaultInputHandler::showRmbMenu( int x, int y )
 {
     if ( isMouseButtonPopupEnabled( Qt::RightButton ) ) {
+        d->m_addMeasurePointAction->setData( QPoint( x, y ) );
         d->m_popupmenu->showRmbMenu( x, y );
     }
 }
@@ -285,8 +403,24 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
 {
     Q_UNUSED( o );
 
-    if ( keyEvent( MarbleWidgetInputHandler::d->m_widget->map(), e ) ) {
-        MarbleWidgetInputHandler::d->m_widget->repaint();
+    if (d->m_selectionRubber->isVisible() && e->type() == QEvent::MouseMove)
+    {
+        QMouseEvent *event = static_cast<QMouseEvent*>( e );
+        if (!(event->modifiers() & Qt::ControlModifier))
+        {
+            d->m_selectionRubber->hide();
+        }
+    }
+
+    if (e->type() == QEvent::MouseButtonDblClick)
+    {
+        d->m_lmbTimer.stop();
+        QMouseEvent *event = static_cast<QMouseEvent*>( e );
+        d->MoveTo(MarbleWidgetInputHandler::d->m_widget, event->pos(), 0.67);
+        MarbleWidgetInputHandler::d->m_mouseWheelTimer->start( 400 );
+    }
+
+    if ( keyEvent( MarbleWidgetInputHandler::d->m_widget, e ) ) {
         return true;
     }
 
@@ -296,7 +430,6 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
          || e->type() == QEvent::MouseButtonPress
          || e->type() == QEvent::MouseButtonRelease )
     {
-
         QMouseEvent *event = static_cast<QMouseEvent*>( e );
         QRegion activeRegion = MarbleWidgetInputHandler::d->m_widget->activeRegion();
 
@@ -318,12 +451,16 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
         // Do not handle (and therefore eat) mouse press and release events
         // that occur above visible float items. Mouse motion events are still
         // handled, however.
-        foreach ( AbstractFloatItem *floatItem, MarbleWidgetInputHandler::d->m_widget->floatItems() ) {
-            if ( floatItem->enabled() && floatItem->visible()
-                 && floatItem->contains( event->pos() )
-                 && e->type() != QEvent::MouseMove )
-            {
-                return false;
+        if ( e->type() != QEvent::MouseMove
+             && !d->m_selectionRubber->isVisible() )
+        {
+            foreach ( AbstractFloatItem *floatItem, MarbleWidgetInputHandler::d->m_widget->floatItems() ) {
+                if ( floatItem->enabled() && floatItem->visible()
+                     && floatItem->contains( event->pos() ) )
+                {
+                    d->m_lmbTimer.stop();
+                    return false;
+                }
             }
         }
 
@@ -347,15 +484,12 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
 
         if ( activeRegion.contains( event->pos() ) || d->m_selectionRubber->isVisible() ) {
 
-            if ( e->type() == QEvent::MouseButtonDblClick) {
-                qDebug("check");
-            }
-
             // Regarding mouse button presses:
             if ( e->type() == QEvent::MouseButtonPress
                  && event->button() == Qt::LeftButton ) {
 
-                d->m_dragtimer.restart();
+                if (isMouseButtonPopupEnabled(Qt::LeftButton))
+                    d->m_lmbTimer.start(400);
 
                 d->m_leftpressed = true;
                 d->m_midpressed = false;
@@ -372,7 +506,26 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 d->m_leftpresseda = MarbleWidgetInputHandler::d->m_widget->centerLongitude() * DEG2RAD;
                 d->m_leftpressedb = MarbleWidgetInputHandler::d->m_widget->centerLatitude() * DEG2RAD;
 
-                MarbleWidgetInputHandler::d->m_widget->setViewContext( Marble::Animation );
+                d->m_leftpresseddirection = 1;
+
+                // Choose spin direction by taking into account whether we
+                // drag above or below the visible pole.
+                if ( MarbleWidgetInputHandler::d->m_widget->projection() == Spherical ) {
+                    if ( d->m_leftpressedb >= 0 ) {
+                        // The visible pole is the north pole
+                        if ( event->y() < ( - MarbleWidgetInputHandler::d->m_widget->northPoleY()
+                                            + MarbleWidgetInputHandler::d->m_widget->height() / 2 ) )
+                            d->m_leftpresseddirection = -1;
+                    }
+                    else {
+                        // The visible pole is the south pole
+                        if ( event->y() > ( + MarbleWidgetInputHandler::d->m_widget->northPoleY()
+                                            + MarbleWidgetInputHandler::d->m_widget->height() / 2 ) )
+                            d->m_leftpresseddirection = -1;
+                    }
+                }
+
+                MarbleWidgetInputHandler::d->m_widget->setViewContext( Animation );
             }
 
             if ( e->type() == QEvent::MouseButtonPress
@@ -382,7 +535,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 d->m_midpressedy = event->y();
 
                 d->m_selectionRubber->hide();
-                MarbleWidgetInputHandler::d->m_widget->setViewContext( Marble::Animation );
+                MarbleWidgetInputHandler::d->m_widget->setViewContext( Animation );
             }
 
             if ( e->type() == QEvent::MouseButtonPress
@@ -395,6 +548,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                  && ( event->modifiers() & Qt::ControlModifier ) )
             {
                 qDebug("Marble: Starting selection");
+                d->m_lmbTimer.stop();                
                 d->m_selectionOrigin = event->globalPos();
                 d->m_selectionRubber->setGeometry( QRect( d->m_selectionOrigin, QSize() ));
                 d->m_selectionRubber->show();
@@ -409,17 +563,9 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 //as requested
                 emit mouseClickScreenPosition( d->m_leftpressedx, d->m_leftpressedy );
 
-                // Show menu if mouse cursor position remains unchanged
-                // the click takes less than 250 ms
-                if ( d->m_dragtimer.elapsed() <= 250
-		     || ( d->m_leftpressedx == event->x()
-			  && d->m_leftpressedy == event->y() ) ) {
-                    emit lmbRequest( d->m_leftpressedx, d->m_leftpressedy );
-                }
-
-                MarbleWidgetInputHandler::d->m_widget->setViewContext( Marble::Still );
-                if ( MarbleWidgetInputHandler::d->m_widget->mapQuality( Marble::Still )
-                    != MarbleWidgetInputHandler::d->m_widget->mapQuality( Marble::Animation ) )
+                MarbleWidgetInputHandler::d->m_widget->setViewContext( Still );
+                if ( MarbleWidgetInputHandler::d->m_widget->mapQuality( Still )
+                    != MarbleWidgetInputHandler::d->m_widget->mapQuality( Animation ) )
                 {
                     MarbleWidgetInputHandler::d->m_widget->updateChangedMap();
                 }
@@ -431,9 +577,9 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                  && event->button() == Qt::MidButton ) {
                 d->m_midpressed = false;
 
-                MarbleWidgetInputHandler::d->m_widget->setViewContext( Marble::Still );
-                if ( MarbleWidgetInputHandler::d->m_widget->mapQuality( Marble::Still )
-                    != MarbleWidgetInputHandler::d->m_widget->mapQuality( Marble::Animation ) )
+                MarbleWidgetInputHandler::d->m_widget->setViewContext( Still );
+                if ( MarbleWidgetInputHandler::d->m_widget->mapQuality( Still )
+                    != MarbleWidgetInputHandler::d->m_widget->mapQuality( Animation ) )
                 {
                     MarbleWidgetInputHandler::d->m_widget->updateChangedMap();
                 }
@@ -460,30 +606,15 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 int deltax = event->x() - d->m_leftpressedx;
                 int deltay = event->y() - d->m_leftpressedy;
 
-                if ( abs( deltax ) <= d->m_dragThreshold
-                     && abs( deltay ) <= d->m_dragThreshold )
-                    return true;
+                if ( abs( deltax ) > d->m_dragThreshold
+                     || abs( deltay ) > d->m_dragThreshold ) {
 
-                qreal direction = 1;
-                // Choose spin direction by taking into account whether we
-                // drag above or below the visible pole.
-                if ( MarbleWidgetInputHandler::d->m_widget->projection() == Spherical ) {
-                    if ( polarity > 0 ) {
-
-                        if ( event->y() < ( - MarbleWidgetInputHandler::d->m_widget->northPoleY()
-                                            + MarbleWidgetInputHandler::d->m_widget->height() / 2 ) )
-                            direction = -1;
-                    }
-                    else {
-                        if ( event->y() > ( + MarbleWidgetInputHandler::d->m_widget->northPoleY()
-                                            + MarbleWidgetInputHandler::d->m_widget->height() / 2 ) )
-                            direction = -1;
-                    }
+                    d->m_lmbTimer.stop();
+                    MarbleWidgetInputHandler::d->m_widget->centerOn( RAD2DEG * ( qreal )( d->m_leftpresseda )
+                                                                     - 90.0 * d->m_leftpresseddirection * deltax / radius,
+                                                                     RAD2DEG * ( qreal )( d->m_leftpressedb )
+                                                                     + 90.0 * deltay / radius );
                 }
-                MarbleWidgetInputHandler::d->m_widget->centerOn( RAD2DEG * ( qreal )( d->m_leftpresseda )
-                                                                 - 90.0 * direction * deltax / radius,
-                                                                 RAD2DEG * ( qreal )( d->m_leftpressedb )
-                                                                 + 90.0 * deltay / radius );
             }
 
 
@@ -492,7 +623,6 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 int dy = d->m_midpressedy - eventy;
                 d->m_midpressed = eventy;
                 MarbleWidgetInputHandler::d->m_widget->zoomViewBy( (int)( 2 * dy / 3 ) );
-                MarbleWidgetInputHandler::d->m_widget->repaint();
             }
 
             if ( d->m_selectionRubber->isVisible() ) 
@@ -528,6 +658,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
             if ( event->button() == Qt::LeftButton
                  && e->type() == QEvent::MouseButtonPress ) {
 
+                d->m_lmbTimer.stop();                
                 if ( polarity < 0 )
                     MarbleWidgetInputHandler::d->m_widget->rotateBy( -MarbleWidgetInputHandler::d->m_widget->moveStep() * (qreal)(+dirX),
                                                                      MarbleWidgetInputHandler::d->m_widget->moveStep() * (qreal)(+dirY) );
@@ -589,39 +720,29 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
 
         MarbleWidgetInputHandler::d->m_widget->setCursor( d->m_arrowcur[dirX+1][dirY+1] );
 
-        return false; // let others, especially float items, still process the event
+        // let others, especially float items, still process the event
+        // Mouse move events need to be eaten to avoid the default oxygen behavior of
+        // moving the window around when we don't handle the event. See bug 242414.
+        return event->type() != QEvent::MouseMove;
     }
     else {
         if ( e->type() == QEvent::Wheel ) {
-            MarbleWidget *marbleWidget = MarbleWidgetInputHandler::d->m_widget;
-            // FIXME: disable animation quality after some time
-            marbleWidget->setViewContext( Marble::Animation );
 
             QWheelEvent *wheelevt = static_cast<QWheelEvent*>( e );
 
-            qreal  destLat;
-            qreal  destLon;
-            bool isValid = marbleWidget->geoCoordinates(wheelevt->x(), wheelevt->y(),
-                             destLon, destLat, GeoDataCoordinates::Radian );
+            MarbleWidget *marbleWidget = MarbleWidgetInputHandler::d->m_widget;
+            marbleWidget->setViewContext( Animation );
 
-            marbleWidget->setUpdatesEnabled( false );
-            marbleWidget->zoomViewBy( (int)(wheelevt->delta() / 3) );
-
-            qreal  mouseLat;
-            qreal  mouseLon;
-            isValid = isValid && marbleWidget->geoCoordinates(wheelevt->x(), wheelevt->y(),
-                        mouseLon, mouseLat, GeoDataCoordinates::Radian );
-
-            qreal centerLat = DEG2RAD * marbleWidget->centerLatitude();
-            qreal centerLon = DEG2RAD * marbleWidget->centerLongitude();
-
-            if ( isValid ) {
-                qreal lon = destLon - (mouseLon - centerLon);
-                qreal lat = destLat - (mouseLat - centerLat);
-                marbleWidget->centerOn( RAD2DEG * lon, RAD2DEG * lat );
+            int steps = wheelevt->delta() / 3;
+            qreal zoom = marbleWidget->map()->zoom();
+            qreal target = MarbleWidgetInputHandler::d->m_wheelZoomTargetDistance;
+            if ( marbleWidget->animationsEnabled() && target > 0.0 ) {
+                // Do not use intermediate (interpolated) distance values caused by animations
+                zoom = marbleWidget->map()->zoomFromDistance( target );
             }
-            marbleWidget->map()->viewParams()->viewport()->setFocusPoint(GeoDataCoordinates(destLon, destLat));
-            marbleWidget->setUpdatesEnabled( true );
+            qreal newDistance = marbleWidget->map()->distanceFromZoom( zoom + steps );
+            MarbleWidgetInputHandler::d->m_wheelZoomTargetDistance = newDistance;
+            d->ZoomAt(MarbleWidgetInputHandler::d->m_widget, wheelevt->pos(), newDistance);
 
             MarbleWidgetInputHandler::d->m_mouseWheelTimer->start( 400 );
             return true;
@@ -645,10 +766,22 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                     scaleFactor = line1.length() / line0.length();
                 }
 
-                MarbleWidgetInputHandler::d->m_widget->setViewContext( Marble::Animation );
+                MarbleWidget *marbleWidget = MarbleWidgetInputHandler::d->m_widget;
+
+                marbleWidget->setViewContext( Animation );
+
+                qreal  destLat;
+                qreal  destLon;
+                QPointF center = line1.pointAt(0.5);
+                bool isValid = marbleWidget->geoCoordinates(center.x(), center.y(),
+                             destLon, destLat, GeoDataCoordinates::Radian );
+
+                if (isValid) {
+                    marbleWidget->map()->viewParams()->viewport()->setFocusPoint(GeoDataCoordinates(destLon, destLat));
+                }
 
                 //convert the scaleFactor to be 0: the same: < 0: smaller, > 0: bigger and make it bigger by multiplying for an arbitrary big value
-                MarbleWidgetInputHandler::d->m_widget->zoomViewBy( (scaleFactor-1)*200);
+                marbleWidget->zoomViewBy( (scaleFactor-1)*200);
                 MarbleWidgetInputHandler::d->m_mouseWheelTimer->start( 400 );
             }
         }
@@ -659,69 +792,62 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
     return QObject::eventFilter( o, e );
 }
 
-bool MarbleWidgetDefaultInputHandler::keyEvent( MarbleMap * map, QEvent* e )
+bool MarbleWidgetDefaultInputHandler::keyEvent( MarbleWidget * widget, QEvent* e )
 {
-    int polarity = map->viewParams()->viewport()->polarity();
-
-    //  if ( o == marbleWidget ){
     if ( e->type() == QEvent::KeyPress ) {
         QKeyEvent const * const k = dynamic_cast<QKeyEvent const * const>( e );
         Q_ASSERT( k );
 
-        int dirx = 0;
-        int diry = 0;
         switch ( k->key() ) {
         case Qt::Key_Left:
-
-            // Depending on whether the planet is "upright" or
-            // "upside down" we need to choose the direction
-            //  of the rotation
-
-            if ( polarity < 0 )
-                dirx = -1;
-            else
-                dirx = 1;
+            widget->moveLeft();
             break;
         case Qt::Key_Up:
-            diry = 1;
+            widget->moveUp();
             break;
         case Qt::Key_Right:
-
-            // Depending on whether the planet is "upright" or
-            // "upside down" we need to choose the direction
-            //  of the rotation
-
-            if ( polarity < 0 )
-                dirx = 1;
-            else
-                dirx = -1;
+            widget->moveRight();
             break;
         case Qt::Key_Down:
-            diry = -1;
+            widget->moveDown();
             break;
         case Qt::Key_Plus:
-            map->zoomIn();
+            widget->zoomIn();
             break;
         case Qt::Key_Minus:
-            map->zoomOut();
+            widget->zoomOut();
             break;
         case Qt::Key_Home:
-            map->goHome();
+            widget->goHome();
             break;
         default:
             break;
         }
 
-        if ( dirx != 0 || diry != 0 ) {
-            map->rotateBy( -map->moveStep() * (qreal)(dirx),
-                           -map->moveStep() * (qreal)(diry) );
-        }
-
         return true;
     }
 
     return false;
 }
 
+void MarbleWidgetDefaultInputHandler::addMeasurePoint()
+{
+    QPoint  p = d->m_addMeasurePointAction->data().toPoint();
+
+    qreal  lat;
+    qreal  lon;
+
+    MarbleWidgetInputHandler::d->m_widget->geoCoordinates( p.x(), p.y(), lon, lat, GeoDataCoordinates::Radian );
+    MeasureTool *measureTool = MarbleWidgetInputHandler::d->m_widget->map()->measureTool();
+    measureTool->addMeasurePoint( lon, lat );
+}
+
+void MarbleWidgetDefaultInputHandler::setNumberOfMeasurePoints( int newNumber )
+{
+    const bool enableMeasureActions = ( newNumber > 0 );
+    d->m_removeMeasurePointsAction->setEnabled(enableMeasureActions);
+    d->m_removeLastMeasurePointAction->setEnabled(enableMeasureActions);
+}
+
 
 #include "MarbleWidgetInputHandler.moc"
