@@ -198,13 +198,8 @@ void QtMarbleConfigDialog::syncSettings()
             proxy.setType( QNetworkProxy::HttpProxy );
         }
     }
-
-    QString hostName = proxyUrl();
-    if ( hostName.startsWith("http://" ) ) {
-        hostName = hostName.mid( QString( "http://" ).size() );
-    }
     
-    proxy.setHostName( hostName );
+    proxy.setHostName( proxyUrl() );
     proxy.setPort( proxyPort() );
     
     if ( proxyAuth() ) {
