@@ -8,14 +8,16 @@
 // Copyright 2009      Patrick Spendrin <ps_ml@gmx.de>
 //
 
-#ifndef GEODATAGEOMETRYPRIVATE_H
-#define GEODATAGEOMETRYPRIVATE_H
+#ifndef MARBLE_GEODATAGEOMETRYPRIVATE_H
+#define MARBLE_GEODATAGEOMETRYPRIVATE_H
 
 #include <QtCore/QString>
 #include <QtCore/QAtomicInt>
 
 #include "GeoDataGeometry.h"
 
+#include "GeoDataTypes.h"
+
 namespace Marble
 {
 
@@ -47,6 +49,11 @@ class GeoDataGeometryPrivate
         return copy;
     }
 
+    virtual QString nodeType() const
+    {
+        return GeoDataTypes::GeoDataGeometryType;
+    }
+
     virtual EnumGeometryId geometryId() const
     {
         return InvalidGeometryId;
@@ -60,4 +67,4 @@ class GeoDataGeometryPrivate
 
 } // namespace Marble
 
-#endif //GeoDataGeometryPRIVATE_H
+#endif
