@@ -19,7 +19,8 @@
 
 // Qt
 #include <QtCore/QDataStream>
-#include <QtCore/QDebug>
+
+#include "MarbleDebug.h"
 
 namespace Marble
 {
@@ -39,8 +40,9 @@ GeoDataPlacemark::GeoDataPlacemark( const GeoDataFeature& other )
 }
 
 GeoDataPlacemark::GeoDataPlacemark( const QString& name )
-    : GeoDataFeature( name )
+    : GeoDataFeature( new GeoDataPlacemarkPrivate )
 {
+    d->m_name = name;
 }
 
 GeoDataPlacemark::~GeoDataPlacemark()
@@ -57,6 +59,11 @@ GeoDataPlacemarkPrivate* GeoDataPlacemark::p() const
     return static_cast<GeoDataPlacemarkPrivate*>(d);
 }
 
+QString GeoDataPlacemark::nodeType() const
+{
+    return p()->nodeType();
+}
+
 GeoDataGeometry* GeoDataPlacemark::geometry() const
 {
     if( p()->m_geometry )
@@ -66,7 +73,7 @@ GeoDataGeometry* GeoDataPlacemark::geometry() const
 }
 
 GeoDataCoordinates GeoDataPlacemark::coordinate() const
-{
+{    
     if( p()->m_geometry && 
       ( p()->m_geometry->geometryId() == GeoDataLineStringId || 
         p()->m_geometry->geometryId() == GeoDataLinearRingId ) ) {
@@ -189,6 +196,19 @@ void GeoDataPlacemark::pack( QDataStream& stream ) const
     p()->m_coordinate.pack( stream );
 }
 
+QXmlStreamWriter& GeoDataPlacemark::pack( QXmlStreamWriter& stream ) const
+{
+    stream.writeStartElement( "placemark" );
+
+    stream.writeEndElement();
+    return stream;
+}
+
+QXmlStreamWriter& GeoDataPlacemark::operator <<( QXmlStreamWriter& stream ) const
+{
+    pack( stream );
+    return stream;
+}
 
 void GeoDataPlacemark::unpack( QDataStream& stream )
 {
