@@ -9,9 +9,8 @@
 // Copyright 2007      Inge Wallin  <ingwa@kde.org>
 //
 
-
-#ifndef GLOBAL_H
-#define GLOBAL_H
+#ifndef MARBLE_GLOBAL_H
+#define MARBLE_GLOBAL_H
 
 
 #include <math.h>
@@ -22,6 +21,7 @@
 
 // #define QT_STRICT_ITERATORS
 
+
 namespace Marble
 {
 
@@ -177,6 +177,17 @@ enum DownloadUsage {
     DownloadBrowse      ///< Browsing mode, normal operation of Marble, like a web browser
 };
 
+/** 
+ * @brief Describes possible flight mode (interpolation between source
+ *        and target camera positions)
+ */
+enum FlyToMode {
+    Automatic, ///< A sane value is chosen automatically depending on animation settings and the action
+    Instant, ///< Change camera position immediately (no interpolation)
+    Linear, ///< Linear interpolation of lon, lat and distance to ground
+    Jump ///< Linear interpolation of lon and lat, distance increases towards the middle point, then decreases    
+};
+
 const int defaultLevelZeroColumns = 2;
 const int defaultLevelZeroRows = 1;
 
@@ -192,32 +203,33 @@ const qreal FT2M = 1.0 / M2FT;
 const qreal DEG2RAD = M_PI / 180.0;
 const qreal RAD2DEG = 180.0 / M_PI;
 
+// Converstion meter vs kilometer
+const qreal KM2METER = 1000.0;
+const qreal METER2KM = 1.0 / KM2METER;
 
 // Version definitions to use with an external application (as digiKam)
 
-// String for about dialog.
-const QString MARBLE_VERSION_STRING = QString::fromLatin1( "0.9.2" );
+// String for about dialog and http user agent
+// FIXME: check if blanks are allowed in user agent version numbers
+const QString MARBLE_VERSION_STRING = QString::fromLatin1( "0.10.0 SVN" );
 
 // API Version id:
 // form : 0xMMmmpp
 //        MM = major revision.
 //        mm = minor revision.
 //        pp = patch revision.
-#define MARBLE_VERSION 0x000902
+#define MARBLE_VERSION 0x000a00
 
 static const char NOT_AVAILABLE[] = QT_TR_NOOP("not available");
 
 const int tileDigits = 6;
 
-// Usage deprecated:
-const qreal RAD2INT = 21600.0 / M_PI;
-
 // Average earth radius in m
 // Deprecated: Please use model()->planetRadius() instead.
 const qreal EARTH_RADIUS = 6378000.0;
 
 // Maximum level of base tiles
-const quint8 maxBaseTileLevel = 4;
+const int maxBaseTileLevel = 4;
 
 class MarbleGlobalPrivate;
 class MarbleLocale;
@@ -229,7 +241,22 @@ class  MARBLE_EXPORT MarbleGlobal
     ~MarbleGlobal();
 
     MarbleLocale * locale() const;
+    
+    enum Profile {
+        Default = 0x0,
+        SmallScreen = 0x1
+    };
+    
+    Q_DECLARE_FLAGS( Profiles, Profile )
 
+    Profiles profiles() const;
+    void setProfiles( Profiles profiles );
+    
+    /**
+     * Automatically detects the profile.
+     */
+    static Profiles detectProfiles();
+    
  private:
     MarbleGlobal();
 
@@ -239,7 +266,8 @@ class  MARBLE_EXPORT MarbleGlobal
 
 }
 
-Q_DECLARE_OPERATORS_FOR_FLAGS(Marble::TessellationFlags)
-Q_DECLARE_OPERATORS_FOR_FLAGS(Marble::LabelPositionFlags)
+Q_DECLARE_OPERATORS_FOR_FLAGS( Marble::TessellationFlags )
+Q_DECLARE_OPERATORS_FOR_FLAGS( Marble::LabelPositionFlags )
+Q_DECLARE_OPERATORS_FOR_FLAGS( Marble::MarbleGlobal::Profiles )
 
 #endif
