@@ -12,9 +12,9 @@
 #include "WaypointContainer.h"
 
 #include "AbstractLayerData.h"
-#include "BoundingBox.h"
 #include "ClipPainter.h"
 
+using namespace Marble;
 
 WaypointContainer::WaypointContainer()
     : AbstractLayerContainer()
@@ -34,16 +34,6 @@ void WaypointContainer::processVisable()
 {
 }
 
-void WaypointContainer::draw( ClipPainter *painter, 
-                           const QSize &canvasSize, ViewParams *viewParams,
-                           const BoundingBox &box )
-{
-    Q_UNUSED( box );
-
-    // FIXME: Use the bounding box.
-    draw( painter, canvasSize, viewParams );
-}
-
 void WaypointContainer::draw( ClipPainter *painter, 
                               const QSize &canvasSize,
                               ViewParams *viewParams )
@@ -55,7 +45,7 @@ void WaypointContainer::draw( ClipPainter *painter,
     painter->setPen( QPen( Qt::black ) );
     painter->setBrush( QBrush( Qt::white ) );
     
-    for ( it = this->begin(); it < this->constEnd(); ++it ) {
+    for ( it = this->constBegin(); it < this->constEnd(); ++it ) {
         draw = (*it)->getPixelPos( canvasSize, viewParams,
                                    &position );
 
