@@ -24,11 +24,12 @@
 #include "GeoOnfParser.h"
 #include "GeoDataDocument.h"
 #include "GeoDataPlacemark.h"
+#include "MarbleDebug.h"
 
 #include <QtCore/QString>
 #include <QtCore/QBuffer>
 #include <QtCore/QVector>
-#include <QtCore/QtDebug>
+#include <QtCore/QUrl>
 
 #include <QtNetwork/QHttp>
 
@@ -64,7 +65,7 @@ void OnfRunner::fail()
     //The manager needs to know when parsing starts and stops
     //in order to have a balanced count of active runners. So
     //we emit runnerFinished() to balance the previous failed runnerStarted()
-    QVector<GeoDataPlacemark*> empty;
+    QVector<GeoDataPlacemark> empty;
     emit runnerFinished( this, empty );
     return;
 }
@@ -83,25 +84,26 @@ void OnfRunner::run()
     }
     */
     //make a new buffer
-    if( m_buffer ) {
-        delete m_buffer;
-    }
+    delete m_buffer;
     m_buffer = new QBuffer;
-    qDebug() << "ONF search: GET /namefinder/search.xml?find=" << m_input;
-    m_http->get( "/namefinder/search.xml?find=" + m_input, m_buffer );
+    QString base = "/namefinder/search.xml?find=%1";
+    QString input = QUrl::toPercentEncoding(m_input);
+    QString request = base.arg(input);
+    mDebug() << "ONF search: GET " << request;
+    m_http->get( request, m_buffer );
 }
 
 void OnfRunner::slotRequestFinished( int id, bool error )
 {
     if( error ) {
-        qDebug() << "ONF request" << id << "failed:" << m_http->error()
+        mDebug() << "ONF request" << id << "failed:" << m_http->error()
                  << m_http->errorString();
         fail();
         return;
     }
     
-    qDebug() << "ONF search result buffer size:" << m_buffer->size();
-    //qDebug() << m_buffer->data();
+    mDebug() << "ONF search result buffer size:" << m_buffer->size();
+    //mDebug() << m_buffer->data();
     
     QByteArray array = m_buffer->data();
     QBuffer data( &array );
@@ -117,12 +119,15 @@ void OnfRunner::slotRequestFinished( int id, bool error )
     GeoDataDocument *results = static_cast<GeoDataDocument*>( parser.releaseDocument() );
     Q_ASSERT( results );
 
-    QVector<GeoDataPlacemark*> placemarks = results->placemarks();
+    QVector<GeoDataPlacemark> placemarks = results->placemarks();
 
-    foreach( GeoDataPlacemark* placemark, placemarks ) {
-        placemark->setVisualCategory( category() );
+    QVector<GeoDataPlacemark>::iterator it = placemarks.begin();
+    QVector<GeoDataPlacemark>::iterator end = placemarks.end();
+    for(; it != end; ++it ) {
+        it->setVisualCategory( category() );
     }
     emit runnerFinished( this, placemarks );
+    delete results;
     return;
 }
 
