@@ -1,6 +1,6 @@
 /*
     Copyright 2008 Henry de Valence <hdevalence@gmail.com>
-    
+
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
@@ -14,16 +14,17 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public 
+    You should have received a copy of the GNU Lesser General Public
     License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#ifndef ONFRUNNER_H
-#define ONFRUNNER_H
+#ifndef MARBLE_ONFRUNNER_H
+#define MARBLE_ONFRUNNER_H
 
 #include "MarbleAbstractRunner.h"
 
 #include <QtCore/QString>
+
 class QHttp;
 class QBuffer;
 
@@ -34,13 +35,16 @@ class OnfRunner : public MarbleAbstractRunner
 {
     Q_OBJECT
 public:
-    OnfRunner(QObject *parent = 0);
+    explicit OnfRunner(QObject *parent = 0);
     ~OnfRunner();
     GeoDataFeature::GeoDataVisualCategory category() const;
-    
+
 public slots:
-    void parse(const QString &input);
     void slotRequestFinished( int id, bool error );
+
+protected:
+    virtual void run();
+
 private:
     void fail();
     QHttp *m_http;
