@@ -13,7 +13,6 @@
 #include "GpsdThread.h"
 #include "MarbleDebug.h"
 
-
 using namespace Marble;
 
 QString GpsdPositionProviderPlugin::name() const
@@ -28,7 +27,7 @@ QString GpsdPositionProviderPlugin::nameId() const
 
 QString GpsdPositionProviderPlugin::guiString() const
 {
-    return tr( "Gpsd position provider plugin" );
+    return tr( "gpsd" );
 }
 
 QString GpsdPositionProviderPlugin::description() const
@@ -43,9 +42,14 @@ QIcon GpsdPositionProviderPlugin::icon() const
 
 void GpsdPositionProviderPlugin::initialize()
 {
+    m_status = PositionProviderStatusAcquiring;
+    emit statusChanged( m_status );
+
     m_thread = new GpsdThread;
     connect( m_thread, SIGNAL( gpsdInfo( gps_data_t ) ),
              this, SLOT( update( gps_data_t ) ) );
+    connect( m_thread, SIGNAL( statusChanged( PositionProviderStatus ) ),
+             this, SIGNAL( statusChanged( PositionProviderStatus ) ) );
     m_thread->start();
 }
 
@@ -102,20 +106,23 @@ GpsdPositionProviderPlugin::GpsdPositionProviderPlugin() : m_thread( 0 )
 
 GpsdPositionProviderPlugin::~GpsdPositionProviderPlugin()
 {
-    if (m_thread) {
+    if ( m_thread ) {
         m_thread->exit();
         
-        if (!m_thread->wait( 5000 ) )
-        {
-          mDebug() << "Failed to stop GpsdThread, terminating it.";
-          m_thread->terminate();
+        if ( !m_thread->wait( 5000 ) ) {
+            mDebug() << "Failed to stop GpsdThread";
+        }
+        else {
+            delete m_thread;
         }
-
-        m_thread->wait();
-        delete m_thread;
     }
 }
 
+QString GpsdPositionProviderPlugin::error() const
+{
+    return m_thread->error();
+}
+
 Q_EXPORT_PLUGIN2( GpsdPositionProviderPlugin, Marble::GpsdPositionProviderPlugin )
 
 
